Type: Package
Package: circumplex
Title: Analysis and Visualization of Circular Data
Version: 1.1.0
Authors@R: 
    c(person(given = "Jeffrey",
             family = "Girard",
             role = c("aut", "cre"),
             email = "me@jmgirard.com",
             comment = c(ORCID = "0000-0002-7359-3746")),
      person(given = "Johannes",
             family = "Zimmermann",
             role = "aut",
             comment = c(ORCID = "0000-0001-6975-2356")),
      person(given = "Aidan",
             family = "Wright",
             role = "aut",
             comment = c(ORCID = "0000-0002-2369-0601")))
Description: Circumplex models, which organize constructs in a circle around two 
    underlying dimensions, are popular for studying interpersonal functioning, 
    mood/affect, and vocational preferences/environments. This package provides 
    tools for analyzing and visualizing circular data, including scoring 
    functions for relevant instruments and a generalization of the bootstrapped 
    structural summary method from Zimmermann & Wright (2017) 
    <doi:10.1177/1073191115621795> and functions for creating publication-ready 
    tables and figures from the results.
License: GPL-3
URL: https://github.com/jmgirard/circumplex,
        http://circumplex.jmgirard.com/
BugReports: https://github.com/jmgirard/circumplex/issues
Depends: R (>= 3.4)
Imports: boot (>= 1.3-18), ggforce (>= 0.3.0), ggplot2 (>= 3.3.0),
        htmlTable (>= 1.13.3), Rcpp, rlang, stats
Suggests: covr (>= 3.5.0), ggrepel, kableExtra (>= 1.1.0), knitr (>=
        1.28), RColorBrewer, rmarkdown (>= 2.1), roxygen2 (>= 7.1.0),
        testthat (>= 3.0.0), vdiffr
LinkingTo: Rcpp, RcppArmadillo (>= 0.11)
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-24 19:35:57 UTC; jmgirard
Author: Jeffrey Girard [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7359-3746>),
  Johannes Zimmermann [aut] (ORCID:
    <https://orcid.org/0000-0001-6975-2356>),
  Aidan Wright [aut] (ORCID: <https://orcid.org/0000-0002-2369-0601>)
Maintainer: Jeffrey Girard <me@jmgirard.com>
Repository: CRAN
Date/Publication: 2026-05-24 20:00:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-25 02:08:01 UTC; unix
Archs: circumplex.so.dSYM

Type: Package
Package: circumplex
Title: Analysis and Visualization of Circular Data
Version: 1.1.0
Authors@R: 
    c(person(given = "Jeffrey",
             family = "Girard",
             role = c("aut", "cre"),
             email = "me@jmgirard.com",
             comment = c(ORCID = "0000-0002-7359-3746")),
      person(given = "Johannes",
             family = "Zimmermann",
             role = "aut",
             comment = c(ORCID = "0000-0001-6975-2356")),
      person(given = "Aidan",
             family = "Wright",
             role = "aut",
             comment = c(ORCID = "0000-0002-2369-0601")))
Description: Circumplex models, which organize constructs in a circle around two 
    underlying dimensions, are popular for studying interpersonal functioning, 
    mood/affect, and vocational preferences/environments. This package provides 
    tools for analyzing and visualizing circular data, including scoring 
    functions for relevant instruments and a generalization of the bootstrapped 
    structural summary method from Zimmermann & Wright (2017) 
    <doi:10.1177/1073191115621795> and functions for creating publication-ready 
    tables and figures from the results.
License: GPL-3
URL: https://github.com/jmgirard/circumplex,
        http://circumplex.jmgirard.com/
BugReports: https://github.com/jmgirard/circumplex/issues
Depends: R (>= 3.4)
Imports: boot (>= 1.3-18), ggforce (>= 0.3.0), ggplot2 (>= 3.3.0),
        htmlTable (>= 1.13.3), Rcpp, rlang, stats
Suggests: covr (>= 3.5.0), ggrepel, kableExtra (>= 1.1.0), knitr (>=
        1.28), RColorBrewer, rmarkdown (>= 2.1), roxygen2 (>= 7.1.0),
        testthat (>= 3.0.0), vdiffr
LinkingTo: Rcpp, RcppArmadillo (>= 0.11)
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-24 19:35:57 UTC; jmgirard
Author: Jeffrey Girard [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7359-3746>),
  Johannes Zimmermann [aut] (ORCID:
    <https://orcid.org/0000-0001-6975-2356>),
  Aidan Wright [aut] (ORCID: <https://orcid.org/0000-0002-2369-0601>)
Maintainer: Jeffrey Girard <me@jmgirard.com>
Repository: CRAN
Date/Publication: 2026-05-24 20:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-25 03:56:52 UTC; unix
Archs: circumplex.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(as_degree,circumplex_degree)
S3method(as_degree,circumplex_radian)
S3method(as_degree,default)
S3method(as_radian,circumplex_degree)
S3method(as_radian,circumplex_radian)
S3method(as_radian,default)
S3method(print,circumplex_degree)
S3method(print,circumplex_instrument)
S3method(print,circumplex_radian)
S3method(print,circumplex_ssm)
S3method(quantile,circumplex_contrast_radian)
S3method(quantile,circumplex_radian)
S3method(summary,circumplex_instrument)
S3method(summary,circumplex_ssm)
export(PANO)
export(anchors)
export(ggsave)
export(html_render)
export(instruments)
export(ipsatize)
export(items)
export(norm_standardize)
export(norms)
export(octants)
export(poles)
export(quadrants)
export(scales)
export(score)
export(self_standardize)
export(ssm_analyze)
export(ssm_parameters)
export(ssm_plot_circle)
export(ssm_plot_contrast)
export(ssm_plot_curve)
export(ssm_score)
export(ssm_table)
importFrom(Rcpp,sourceCpp)
importFrom(ggplot2,ggsave)
importFrom(rlang,.data)
importFrom(stats,quantile)
useDynLib(circumplex, .registration = TRUE)

# circumplex

<!-- markdownlint-disable MD033 -->
<img src="https://raw.githubusercontent.com/MitchellAcoustics/circumplex/main/docs/img/logo-dark.png" align="right" alt="" width="200" />

[![PyPI version](https://badge.fury.io/py/circumplex.svg)](https://badge.fury.io/py/circumplex)
[![Documentation Status](https://readthedocs.org/projects/circumplex/badge/?version=latest)](https://circumplex.readthedocs.io/en/latest/?badge=latest)
[![License: GPL v3](https://img.shields.io/badge/License-GPLv3-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)

[![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/pre-commit/pre-commit)
[![Tests status][tests-badge]][tests-link]
[![Linting status][linting-badge]][linting-link]
[![Documentation status][documentation-badge]][documentation-link]
[![License][license-badge]](./LICENSE)

<!-- prettier-ignore-start -->
[tests-badge]:              https://github.com/MitchellAcoustics/circumplex/actions/workflows/tests.yml/badge.svg
[tests-link]:               https://github.com/MitchellAcoustics/circumplex/actions/workflows/tests.yml
[linting-badge]:            https://github.com/MitchellAcoustics/circumplex/actions/workflows/linting.yml/badge.svg
[linting-link]:             https://github.com/MitchellAcoustics/circumplex/actions/workflows/linting.yml
[documentation-badge]:      https://github.com/MitchellAcoustics/circumplex/actions/workflows/docs.yml/badge.svg
[documentation-link]:       https://github.com/MitchellAcoustics/circumplex/actions/workflows/docs.yml
[license-badge]:            https://img.shields.io/badge/License-GPLv3-blue.svg

<!-- prettier-ignore-end -->

_circumplex_ is a Python package for analyzing and visualizing circumplex data. It provides a set of tools for analyzing and visualizing circumplex data, following the Structural Summary Method. This project is a Python implementation based on the R [circumplex](https://github.com/jmgirard/circumplex) package. Our goal is to provide a similar functionality and experience for Python users.

> [!WARNING]
> This project is still under development. We're working hard to make it as good as possible, but there may be bugs or missing features. If you find any issues, please let us know by submitting an issue on Github.

## Getting Started

To get started with _circumplex_, you'll need to install it first. You can do this by running the following command:

```bash
pip install circumplex
```

## Documentation

This documentation is designed to help you understand and use _circumplex_ effectively. It's divided into several sections:

- **Tutorials**: Practical examples showing how to use our project in real-world scenarios.
- **API Reference**: Detailed information about our project's API.
- **Contribute**: Information on how you can contribute to our project.

## Contributing

We welcome contributions from the community. If you're interested in contributing, please get in touch or submit an issue on Github.

## License

This project is licensed under the GNU GPLv3 License. For more information, please see the `LICENSE` file.

## Project layout

```text
    mkdocs.yml     # The configuration file.
    docs/
        index.md   # The documentation homepage.
        about.md   # The about page.
        LICENSE # The license page.
        tutorials/ # Tutorial pages.
            Introduction to SSM Analysis.ipynb
        ...        # Other markdown pages, images and other files.
    src/circumplex/
        circumplex.py
```

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
set.seed(12345)
library(ggforce)
library(kableExtra)
library(ggplot2)
library(tibble)
library(dplyr)
library(knitr)

## ----setup--------------------------------------------------------------------
library(circumplex)

## ----model, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"----
angles <- c(90, 135, 180, 225, 270, 315, 360, 45)
alabel <- c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO")

# Create plot ------------------------------------------------------------------

ggplot2::ggplot() +
  # Require plot to be square and remove default styling
  ggplot2::coord_fixed() +
  ggplot2::theme_void() +
  # Expand the axes multiplicatively to fit the labels
  ggplot2::scale_x_continuous(expand = c(0.10, 0)) +
  ggplot2::scale_y_continuous(expand = c(0.10, 0)) +
  # Draw line segments corresponding to the octants
  ggplot2::geom_segment(
    ggplot2::aes(
      x = 0,
      y = 0,
      xend = 5 * cos(angles[c(1, 3, 5, 7)] * pi / 180),
      yend = 5 * sin(angles[c(1, 3, 5, 7)] * pi / 180)
    ),
    color = "gray60",
    size = 1
  ) +
  # Draw inner labels for the octant angles
  ggplot2::geom_label(
    aes(
      x = 3 * cos(angles * pi / 180),
      y = 3 * sin(angles * pi / 180),
      label = sprintf("%d\u00B0", angles)
    ),
    size = 5,
    color = "black",
    label.size = NA,
    hjust = "center",
    vjust = "center"
  ) +
  # Draw the circle
  ggforce::geom_circle(ggplot2::aes(x0 = 0, y0 = 0, r = 5),
    color = "gray50", size = 1.5
  ) +
  # Draw outer labels for octant abbreviations
  ggplot2::geom_label(
    ggplot2::aes(
      x = 5.1 * cos(angles * pi / 180),
      y = 5.1 * sin(angles * pi / 180),
      label = alabel
    ),
    size = 5,
    color = "black",
    label.size = NA,
    hjust = "outward",
    vjust = "outward"
  )

## ----column, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"----
data("jz2017")
rmat <- jz2017 %>% 
  dplyr::select(NARPD, PA:NO) %>% 
  cor(method = "pearson")
r <- rmat[2:9, 1]

# Format data for plotting
dat_r <- tibble::tibble(
  Scale = factor(c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO")),
  r = r
)

# Create column plot
ggplot2::ggplot(dat_r, ggplot2::aes(x = Scale, y = r)) +
  ggplot2::geom_hline(yintercept = 0, size = 1.25, color = "darkgray") +
  ggplot2::geom_col(position = ggplot2::position_dodge(.9), fill = "red") +
  ggplot2::scale_y_continuous(
    limits = c(-0.02, 0.5)
  ) +
  ggplot2::labs(title = "Scores") +
  ggplot2::theme(
    axis.title = ggplot2::element_blank(),
    panel.grid.major = ggplot2::element_line(size = 1.0),
    panel.grid.minor.y = ggplot2::element_line(size = 0.5),
    panel.grid.minor.x = ggplot2::element_blank()
  )

## ----path, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"----
dat_r <- tibble::tibble(
  Scale = factor(
    c("LM", "PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
    levels = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO")
  ),
  est = r[c(7, 1:8)],
  Angle = c(0, octants())
) %>% 
  dplyr::arrange(Angle)

# Plot correlations as connected point ranges with 95% CI ranges
ggplot2::ggplot(dat_r, ggplot2::aes(x = Angle, y = est)) +
  ggplot2::geom_hline(yintercept = 0, size = 1.25, color = "darkgray") +
  ggplot2::geom_point(size = 3, color = "red") +
  ggplot2::geom_path(size = 1.25, color = "red") +
  ggplot2::geom_label(aes(label = Scale), nudge_y = 0.075) +
  ggplot2::scale_x_continuous(
    limits = c(0, 360),
    breaks = c(0, octants()),
    expand = c(0.05, 0),
    labels = function(x) sprintf("%.0f\U00B0", x)
  ) +
  ggplot2::scale_y_continuous(
    limits = c(-0.02, 0.5)
  ) +
  ggplot2::labs(title = "Scores by Angle") +
  ggplot2::theme(
    axis.title = ggplot2::element_blank(),
    plot.margin = ggplot2::unit(c(10, 30, 10, 10), "points"),
    panel.grid.major = ggplot2::element_line(size = 1.0),
    panel.grid.minor.y = ggplot2::element_line(size = 0.5),
    panel.grid.minor.x = ggplot2::element_blank()
  )

## ----curve, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"----
# Calculate SSM parameters
sp <- ssm_parameters(r, octants())

# Create function for SSM cosine model
f <- function(x) {
  sp$Elev + sp$Ampl * cos((x - sp$Disp) * pi / 180)
}

# Plot correlations along with SSM cosine model
ggplot2::ggplot(dat_r, ggplot2::aes(x = Angle, y = est)) +
  ggplot2::geom_hline(yintercept = 0, size = 1.25, color = "darkgray") +
  ggplot2::geom_point(size = 3) +
  ggplot2::geom_path(size = 1.25) +
  ggplot2::stat_function(fun = f, size = 2, color = "red") +
  ggplot2::scale_x_continuous(
    limits = c(0, 360),
    breaks = c(0, octants()),
    expand = c(0.01, 0),
    labels = function(x) sprintf("%.0f\U00B0", x)
  ) +
  ggplot2::scale_y_continuous(
    limits = c(-0.02, 0.5)
  ) +
  ggplot2::labs(title = "Cosine Curve Estimated by SSM") +
  ggplot2::theme(
    axis.title = ggplot2::element_blank(),
    plot.margin = ggplot2::unit(c(10, 30, 10, 10), "points"),
    panel.grid.major = ggplot2::element_line(size = 1.0),
    panel.grid.minor.y = ggplot2::element_line(size = 0.5),
    panel.grid.minor.x = ggplot2::element_blank()
  )

## ----residuals, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"----
# Plot correlations as path, SSM cosine model, and differences
ggplot2::ggplot(dat_r, ggplot2::aes(x = Angle, y = est)) +
  ggplot2::geom_hline(yintercept = 0, size = 1.25, color = "darkgray") +
  ggplot2::stat_function(fun = f, size = 2, color = "gray20") +
  ggplot2::geom_point(size = 5.5, color = "black") +
  ggplot2::geom_path(size = 1.25, color = "black") +
  ggplot2::geom_segment(
    ggplot2::aes(x = Angle, xend = Angle, y = est, yend = f(Angle)),
    size = 4, linetype = "solid", color = "red"
  ) +
  ggplot2::scale_x_continuous(
    limits = c(0, 360),
    breaks = c(0, octants()),
    expand = c(0.01, 0),
    labels = function(x) sprintf("%.0f\U00B0", x)
  ) +
  ggplot2::scale_y_continuous(
    limits = c(-0.02, 0.475)
  ) +
  ggplot2::labs(title = sprintf("Fit = %.2f", sp[[6]])) +
  ggplot2::theme(
    axis.title = ggplot2::element_blank(),
    plot.margin = ggplot2::unit(c(10, 30, 10, 10), "points"),
    panel.grid.major = ggplot2::element_line(size = 1.0),
    panel.grid.minor.y = ggplot2::element_line(size = 0.5),
    panel.grid.minor.x = ggplot2::element_blank()
  )

## ----elev, echo = FALSE, out.width = "100%"-----------------------------------
knitr::include_graphics("VIG1-e.gif")

## ----ampl, echo = FALSE, out.width = "100%"-----------------------------------
knitr::include_graphics("VIG1-a.gif")

## ----disp, echo = FALSE, out.width = "100%"-----------------------------------
knitr::include_graphics("VIG1-d.gif")

## ----jz2017-------------------------------------------------------------------
data("jz2017")
print(jz2017)

## ----iipsc, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"----
angles <- c(90, 135, 180, 225, 270, 315, 360, 45)
flabel <- c(
  "Domineering",
  "Vindictive",
  "Cold",
  "Socially\nAvoidant",
  "Nonassertive",
  "Easily\nExploited",
  "Overly\nNurturant",
  "Intrusive"
)
alabel <- c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO")

# Create plot ------------------------------------------------------------------

ggplot2::ggplot() +
  # Require plot to be square and remove default styling
  ggplot2::coord_fixed() +
  ggplot2::theme_void() +
  # Expand both axes multiplicatively to fit the labels
  ggplot2::scale_x_continuous(expand = c(0.30, 0)) +
  ggplot2::scale_y_continuous(expand = c(0.10, 0)) +
  # Draw line segments corresponding to the octants
  ggplot2::geom_segment(
    ggplot2::aes(
      x = 0,
      y = 0,
      xend = 5 * cos(angles * pi / 180),
      yend = 5 * sin(angles * pi / 180)
    ),
    color = "gray60",
    size = 0.5
  ) +
  # Draw inner labels for the octant abbreviations
  ggplot2::geom_label(
    ggplot2::aes(
      x = 3.75 * cos(angles * pi / 180),
      y = 3.75 * sin(angles * pi / 180),
      label = alabel
    ),
    size = 5,
    color = "gray40",
    label.size = NA,
    hjust = "center",
    vjust = "center"
  ) +
  # Draw inner labels for the octant angles
  ggplot2::geom_label(
    ggplot2::aes(
      x = 2 * cos(angles * pi / 180),
      y = 2 * sin(angles * pi / 180),
      label = sprintf("%d\u00B0", angles)
    ),
    size = 4,
    color = "gray50",
    label.size = NA,
    hjust = "center",
    vjust = "center"
  ) +
  # Draw the circle
  ggforce::geom_circle(
    ggplot2::aes(x0 = 0, y0 = 0, r = 5),
    color = "gray50", 
    size = 1.5
  ) +
  # Draw outer labels for the octant names
  ggplot2::geom_label(
    ggplot2::aes(
      x = 5.1 * cos(angles * pi / 180),
      y = 5.1 * sin(angles * pi / 180),
      label = flabel
    ),
    size = 5,
    color = "black",
    label.size = NA,
    hjust = "outward",
    vjust = "outward"
  )

## -----------------------------------------------------------------------------
data("iipsc")
jz2017s <- standardize(
  .data = jz2017,
  scales = c(PA, BC, DE, FG, HI, JK, LM, NO),
  angles = c(90, 135, 180, 225, 270, 315, 360, 45),
  instrument = iipsc,
  sample = 1
)
print(jz2017s)

## ----analyze------------------------------------------------------------------
results <- ssm_analyze(
  .data = jz2017s,
  scales = c(PA_z, BC_z, DE_z, FG_z, HI_z, JK_z, LM_z, NO_z),
  angles = c(90, 135, 180, 225, 270, 315, 360, 45)
)

## ----summary1a----------------------------------------------------------------
summary(results)

## ----summary1b----------------------------------------------------------------
results2 <- ssm_analyze(jz2017s, PA_z:NO_z, octants())
summary(results2)

## ----table1, echo = FALSE-----------------------------------------------------
ssm_table(results2, render = FALSE) %>%
  kable(caption = circumplex:::dcaption(results2)) %>%
  kable_styling(full_width = TRUE, font_size = 14)

## ----plot1, fig.width = 7.2, fig.height = 4, out.width = "100%"---------------
ssm_plot(results2)

## ----summary2-----------------------------------------------------------------
results3 <- ssm_analyze(jz2017, PA:NO, octants(), measures = NARPD)
summary(results3)

## ----table2, echo = FALSE-----------------------------------------------------
ssm_table(results3, render = FALSE) %>%
  kable(caption = circumplex:::dcaption(results3)) %>%
  kable_styling(full_width = TRUE, font_size = 14)

## ----plot2, fig.width = 7.5, fig.height = 4, out.width = "100%"---------------
ssm_plot(results3)

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggplot2)
library(ggforce)
library(tibble)
library(kableExtra)
set.seed(12345)

## ----setup--------------------------------------------------------------------
library(circumplex)

## ----group--------------------------------------------------------------------
data("jz2017")
data("iipsc")
jz2017s <- standardize(jz2017, PA:NO, octants(), instrument = iipsc, sample = 1)
results <- ssm_analyze(jz2017s, PA:NO, octants(), grouping = Gender)
summary(results)

## ----group_table, echo = FALSE------------------------------------------------
ssm_table(results, render = FALSE) %>% 
  kable(caption = circumplex:::dcaption(results)) %>% 
  kable_styling(full_width = TRUE, font_size = 12)

## ----group_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"----------
ssm_plot(results)

## ----group_plot_lowfit, fig.width = 7.5, fig.height = 4, out.width = "100%"----
ssm_plot(results, lowfit = FALSE)

## ----measures-----------------------------------------------------------------
results2 <- ssm_analyze(jz2017, PA:NO, octants(), measures = c(NARPD, ASPD))
summary(results2)

## ----measures_table, echo = FALSE---------------------------------------------
ssm_table(results2, render = FALSE) %>% 
  kable(caption = circumplex:::dcaption(results2)) %>% 
  kable_styling(full_width = TRUE, font_size = 12)

## ----measures_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"-------
ssm_plot(results2)

## ----general------------------------------------------------------------------
results3 <- ssm_analyze(jz2017, PA:NO, octants(), grouping = Gender, measures = PARPD:SZTPD)
summary(results3)

## ----general_table, echo = FALSE----------------------------------------------
ssm_table(results3, render = FALSE) %>% 
  kable(caption = circumplex:::dcaption(results3)) %>% 
  kable_styling(full_width = TRUE, font_size = 12)

## ----general_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"--------
ssm_plot(results3)

## ----model_contrast-----------------------------------------------------------
results4 <- ssm_analyze(jz2017s, PA:NO, octants(), grouping = Gender, contrast = "model")
summary(results4)

## ----model_table, echo = FALSE------------------------------------------------
ssm_table(results4, render = FALSE) %>% 
  kable(caption = circumplex:::dcaption(results4)) %>% 
  kable_styling(full_width = TRUE, font_size = 12)

## ----model_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"----------
ssm_plot(results4)

## ----measure_contrast---------------------------------------------------------
results5 <- ssm_analyze(jz2017, PA:NO, octants(), measures = c(NARPD, ASPD),
  contrast = "test")
summary(results5)

## ----measure_contrast_table, echo = FALSE, results = "asis"-------------------
ssm_table(results5, render = FALSE) %>% 
  kable(caption = circumplex:::dcaption(results5)) %>% 
  kable_styling(full_width = TRUE, font_size = 12)

## ----measure_contrast_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"----
ssm_plot(results5)

## ----group_contrast-----------------------------------------------------------
results6 <- ssm_analyze(jz2017, PA:NO, octants(), measures = BORPD, 
  grouping = Gender, contrast = "test")
summary(results6)

## ----group_contrast_table, echo = FALSE, results = "asis"---------------------
ssm_table(results6, render = FALSE) %>% 
  kable(caption = circumplex:::dcaption(results6)) %>% 
  kable_styling(full_width = TRUE, font_size = 12)

## ----group_contrast_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"----
ssm_plot(results6)

## ----taxonomy, echo = FALSE---------------------------------------------------
msr <- c("FALSE", "FALSE", "FALSE", "TRUE",  "TRUE",  "TRUE",  "TRUE")
grp <- c("FALSE", "TRUE",  "TRUE",  "FALSE", "FALSE", "TRUE",  "TRUE")
ctr <- c("FALSE", "FALSE", "TRUE",  "FALSE", "TRUE",  "FALSE", "TRUE")
tab <- tibble(
  `#` = c(1, 2, 3, 4, 5, 6, 7),
  Usage = c(
    "Examine overall mean profile",
    "Examine groups' mean profiles",
    "Compare groups' mean profiles",
    "Examine variables' correlation profiles",
    "Compare variables' correlation profiles",
    "Examine groups' correlation profiles",
    "Compare groups' correlation profiles"
  ),
  measures = cell_spec(msr, color = ifelse(msr == "TRUE", "blue", "black")),
  grouping = cell_spec(grp, color = ifelse(grp == "TRUE", "blue", "black")),
  contrast = cell_spec(ctr, color = ifelse(ctr == "TRUE", "blue", "black"))
)
knitr::kable(tab, escape = FALSE) %>% 
  column_spec(2, width = "3in") %>% 
  add_header_above(c("", "", "Arguments Needed" = 3))

## ----append, echo = FALSE-----------------------------------------------------
res1 <- ssm_analyze(jz2017s, PA:NO, octants())
res2 <- ssm_analyze(jz2017s, PA:NO, octants(), grouping = Gender)
tab1 <- ssm_table(res1, xy = FALSE, render = FALSE)
tab2 <- ssm_table(res2, xy = FALSE, render = FALSE)
ssm_append(tab1, tab2, render = FALSE) %>% 
  kable() %>% 
  kable_styling(full_width = TRUE, font_size = 12)

## ----amax, fig.width = 7.5, fig.height = 4, out.width = "100%"----------------
ssm_plot(results4, amax = 0.6)

## ----custom, fig.width = 7.5, fig.height = 4, out.width = "100%"--------------
ssm_plot(results6, xy = FALSE, color = "blue", linesize = 1,
  axislabel = "BORPD: Male - Female")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
set.seed(12345)
library(ggforce)
library(kableExtra)
library(ggplot2)
library(knitr)

## ----setup--------------------------------------------------------------------
library(circumplex)

## ----model, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"----
angles <- c(90, 135, 180, 225, 270, 315, 360, 45)
alabel <- c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO")

# Create plot ------------------------------------------------------------------

ggplot2::ggplot() +
  # Require plot to be square and remove default styling
  ggplot2::coord_fixed() +
  ggplot2::theme_void() +
  # Expand the axes multiplicatively to fit the labels
  ggplot2::scale_x_continuous(expand = c(0.10, 0)) +
  ggplot2::scale_y_continuous(expand = c(0.10, 0)) +
  # Draw line segments corresponding to the octants
  ggplot2::geom_segment(
    ggplot2::aes(
      x = 0,
      y = 0,
      xend = 5 * cos(angles[c(1, 3, 5, 7)] * pi / 180),
      yend = 5 * sin(angles[c(1, 3, 5, 7)] * pi / 180)
    ),
    color = "gray60",
    linewidth = 1
  ) +
  # Draw inner labels for the octant angles
  ggplot2::geom_label(
    ggplot2::aes(
      x = 3 * cos(angles * pi / 180),
      y = 3 * sin(angles * pi / 180),
      label = sprintf("%d\u00B0", angles)
    ),
    size = 5,
    color = "black",
    label.size = NA,
    hjust = "center",
    vjust = "center"
  ) +
  # Draw the circle
  ggforce::geom_circle(ggplot2::aes(x0 = 0, y0 = 0, r = 5),
    color = "gray50", linewidth = 1.5
  ) +
  # Draw outer labels for octant abbreviations
  ggplot2::geom_label(
    ggplot2::aes(
      x = 5.1 * cos(angles * pi / 180),
      y = 5.1 * sin(angles * pi / 180),
      label = alabel
    ),
    size = 5,
    color = "black",
    label.size = NA,
    hjust = "outward",
    vjust = "outward"
  )

## ----column, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"----
data("jz2017")
rmat <- stats::cor(jz2017[c("NARPD", PANO())], method = "pearson")
r <- rmat[2:9, 1]

# Format data for plotting
dat_r <- data.frame(Scale = factor(PANO()), r = r)

# Create column plot
ggplot2::ggplot(dat_r, ggplot2::aes(x = .data$Scale, y = .data$r)) +
  ggplot2::geom_hline(yintercept = 0, linewidth = 1.25, color = "darkgray") +
  ggplot2::geom_col(position = ggplot2::position_dodge(.9), fill = "red") +
  ggplot2::scale_y_continuous(limits = c(-0.02, 0.5)) +
  ggplot2::labs(title = "Scores") +
  ggplot2::theme(
    axis.title = ggplot2::element_blank(),
    panel.grid.major = ggplot2::element_line(linewidth = 1.0),
    panel.grid.minor.y = ggplot2::element_line(linewidth = 0.5),
    panel.grid.minor.x = ggplot2::element_blank()
  )

## ----path, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"----
dat_r <- data.frame(
  Scale = factor(PANO()),
  est = r,
  Angle = octants()
)

dat_r <- dat_r[order(dat_r$Angle), ]

# Plot correlations as connected point ranges with 95% CI ranges
ggplot2::ggplot(dat_r, ggplot2::aes(x = .data$Angle, y = .data$est)) +
  ggplot2::geom_hline(yintercept = 0, linewidth = 1.25, color = "darkgray") +
  ggplot2::geom_point(size = 3, color = "red") +
  ggplot2::geom_path(linewidth = 1.25, color = "red") +
  ggplot2::geom_label(ggplot2::aes(label = Scale), nudge_y = 0.075) +
  ggplot2::scale_x_continuous(
    breaks = octants(),
    expand = c(0.05, 0),
    labels = function(x) sprintf("%.0f\U00B0", x)
  ) +
  ggplot2::scale_y_continuous(
    limits = c(-0.02, 0.5)
  ) +
  ggplot2::labs(title = "Scores by Angle") +
  ggplot2::theme(
    axis.title = ggplot2::element_blank(),
    plot.margin = ggplot2::unit(c(10, 30, 10, 10), "points"),
    panel.grid.major = ggplot2::element_line(linewidth = 1.0),
    panel.grid.minor.y = ggplot2::element_line(linewidth = 0.5),
    panel.grid.minor.x = ggplot2::element_blank()
  )

## ----curve, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"----
# Calculate SSM parameters
sp <- ssm_parameters(r, octants())

# Create function for SSM cosine model
f <- function(x) {
  sp$Elev + sp$Ampl * cos((x - sp$Disp) * pi / 180)
}

# Plot correlations along with SSM cosine model
ggplot2::ggplot(dat_r, ggplot2::aes(x = .data$Angle, y = .data$est)) +
  ggplot2::geom_hline(yintercept = 0, linewidth = 1.25, color = "darkgray") +
  ggplot2::geom_point(size = 3) +
  ggplot2::geom_path(linewidth = 1.25) +
  ggplot2::stat_function(fun = f, linewidth = 2, color = "red") +
  ggplot2::scale_x_continuous(
    breaks = octants(),
    expand = c(0.01, 0),
    labels = function(x) sprintf("%.0f\U00B0", x)
  ) +
  ggplot2::scale_y_continuous(
    limits = c(-0.02, 0.5)
  ) +
  ggplot2::labs(title = "Cosine Curve Estimated by SSM") +
  ggplot2::theme(
    axis.title = ggplot2::element_blank(),
    plot.margin = ggplot2::unit(c(10, 30, 10, 10), "points"),
    panel.grid.major = ggplot2::element_line(linewidth = 1.0),
    panel.grid.minor.y = ggplot2::element_line(linewidth = 0.5),
    panel.grid.minor.x = ggplot2::element_blank()
  )

## ----residuals, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"----
# Plot correlations as path, SSM cosine model, and differences
ggplot2::ggplot(dat_r, ggplot2::aes(x = .data$Angle, y = .data$est)) +
  ggplot2::geom_hline(yintercept = 0, linewidth = 1.25, color = "darkgray") +
  ggplot2::stat_function(fun = f, linewidth = 2, color = "gray20") +
  ggplot2::geom_point(size = 5.5, color = "black") +
  ggplot2::geom_path(linewidth = 1.25, color = "black") +
  ggplot2::geom_segment(
    ggplot2::aes(
      x = .data$Angle, 
      xend = .data$Angle, 
      y = .data$est, 
      yend = f(.data$Angle)
    ),
    linewidth = 4, 
    linetype = "solid", 
    color = "red"
  ) +
  ggplot2::scale_x_continuous(
    breaks = octants(),
    expand = c(0.01, 0),
    labels = function(x) sprintf("%.0f\U00B0", x)
  ) +
  ggplot2::scale_y_continuous(
    limits = c(-0.02, 0.475)
  ) +
  ggplot2::labs(title = sprintf("Fit = %.2f", sp[[6]])) +
  ggplot2::theme(
    axis.title = ggplot2::element_blank(),
    plot.margin = ggplot2::unit(c(10, 30, 10, 10), "points"),
    panel.grid.major = ggplot2::element_line(linewidth = 1.0),
    panel.grid.minor.y = ggplot2::element_line(linewidth = 0.5),
    panel.grid.minor.x = ggplot2::element_blank()
  )

## ----elev, echo = FALSE, out.width = "100%"-----------------------------------
knitr::include_graphics("VIG1-e.gif")

## ----ampl, echo = FALSE, out.width = "100%"-----------------------------------
knitr::include_graphics("VIG1-a.gif")

## ----disp, echo = FALSE, out.width = "100%"-----------------------------------
knitr::include_graphics("VIG1-d.gif")

## ----jz2017-------------------------------------------------------------------
data("jz2017")
head(jz2017)

## ----iipsc, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"----
angles <- c(90, 135, 180, 225, 270, 315, 360, 45)
flabel <- c(
  "Domineering",
  "Vindictive",
  "Cold",
  "Socially\nAvoidant",
  "Nonassertive",
  "Easily\nExploited",
  "Overly\nNurturant",
  "Intrusive"
)
alabel <- c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO")

# Create plot ------------------------------------------------------------------

ggplot2::ggplot() +
  # Require plot to be square and remove default styling
  ggplot2::coord_fixed() +
  ggplot2::theme_void() +
  # Expand both axes multiplicatively to fit the labels
  ggplot2::scale_x_continuous(expand = c(0.30, 0)) +
  ggplot2::scale_y_continuous(expand = c(0.10, 0)) +
  # Draw line segments corresponding to the octants
  ggplot2::geom_segment(
    ggplot2::aes(
      x = 0,
      y = 0,
      xend = 5 * cos(angles * pi / 180),
      yend = 5 * sin(angles * pi / 180)
    ),
    color = "gray60",
    linewidth = 0.5
  ) +
  # Draw inner labels for the octant abbreviations
  ggplot2::geom_label(
    ggplot2::aes(
      x = 3.75 * cos(angles * pi / 180),
      y = 3.75 * sin(angles * pi / 180),
      label = alabel
    ),
    size = 5,
    color = "gray40",
    label.size = NA,
    hjust = "center",
    vjust = "center"
  ) +
  # Draw inner labels for the octant angles
  ggplot2::geom_label(
    ggplot2::aes(
      x = 2 * cos(angles * pi / 180),
      y = 2 * sin(angles * pi / 180),
      label = sprintf("%d\u00B0", angles)
    ),
    size = 4,
    color = "gray50",
    label.size = NA,
    hjust = "center",
    vjust = "center"
  ) +
  # Draw the circle
  ggforce::geom_circle(
    ggplot2::aes(x0 = 0, y0 = 0, r = 5),
    color = "gray50", 
    linewidth = 1.5
  ) +
  # Draw outer labels for the octant names
  ggplot2::geom_label(
    ggplot2::aes(
      x = 5.1 * cos(angles * pi / 180),
      y = 5.1 * sin(angles * pi / 180),
      label = flabel
    ),
    size = 5,
    color = "black",
    label.size = NA,
    hjust = "outward",
    vjust = "outward"
  )

## ----analyze------------------------------------------------------------------
results <- ssm_analyze(
  data = jz2017,
  scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
  angles = c(90, 135, 180, 225, 270, 315, 360, 45)
)

## ----summary1a----------------------------------------------------------------
summary(results)

## ----summary1b----------------------------------------------------------------
results2 <- ssm_analyze(data = jz2017, scales = PANO(), angles = octants())
summary(results2)

## ----table1, echo = FALSE-----------------------------------------------------
ssm_table(results2, render = FALSE) %>%
  knitr::kable(caption = circumplex:::dcaption(results2)) %>%
  kableExtra::kable_styling(full_width = TRUE, font_size = 14)

## ----plot1, fig.width = 7.2, fig.height = 4, out.width = "100%"---------------
ssm_plot_circle(results2)

## ----plot2, fig.width = 7.2, fig.height = 4, out.width = "100%"---------------
ssm_plot_curve(results2)

## ----summary2-----------------------------------------------------------------
results3 <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(),
  measures = "NARPD"
)
summary(results3)

## ----table2, echo = FALSE-----------------------------------------------------
ssm_table(results3, render = FALSE) %>%
  knitr::kable(caption = circumplex:::dcaption(results3)) %>%
  kableExtra::kable_styling(full_width = TRUE, font_size = 14)

## ----plot3, fig.width = 7.5, fig.height = 4, out.width = "100%"---------------
ssm_plot_circle(results3)

## ----plot4, fig.width = 7.5, fig.height = 4, out.width = "100%"---------------
ssm_plot_curve(results3)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggplot2)
library(ggforce)
library(kableExtra)
set.seed(12345)

## ----setup--------------------------------------------------------------------
library(circumplex)

## ----group--------------------------------------------------------------------
data("jz2017")
results <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(), 
  grouping = "Gender"
)
summary(results)

## ----group_table, echo = FALSE------------------------------------------------
ssm_table(results, render = FALSE) %>% 
  knitr::kable(caption = circumplex:::dcaption(results)) %>% 
  kableExtra::kable_styling(full_width = TRUE, font_size = 12)

## ----group_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"----------
ssm_plot_circle(results)

## ----group_plot2, fig.width = 7.5, fig.height = 4, out.width = "100%"---------
ssm_plot_curve(results)

## ----measures-----------------------------------------------------------------
results2 <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(),
  measures = c("NARPD", "ASPD")
)
summary(results2)

## ----measures_table, echo = FALSE---------------------------------------------
ssm_table(results2, render = FALSE) %>% 
  knitr::kable(caption = circumplex:::dcaption(results2)) %>% 
  kableExtra::kable_styling(full_width = TRUE, font_size = 12)

## ----measures_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"-------
ssm_plot_circle(results2)

## ----measures_plot2, fig.width = 7.5, fig.height = 4, out.width = "100%"------
ssm_plot_curve(results2)

## ----general------------------------------------------------------------------
results3 <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(), 
  grouping = "Gender", 
  measures = 10:12)
summary(results3)

## ----general_table, echo = FALSE----------------------------------------------
ssm_table(results3, render = FALSE) %>% 
  knitr::kable(caption = circumplex:::dcaption(results3)) %>% 
  kableExtra::kable_styling(full_width = TRUE, font_size = 12)

## ----general_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"--------
ssm_plot_circle(results3)

## ----general_plot2, fig.width = 7.5, fig.height = 4, out.width = "100%"-------
ssm_plot_circle(results3, drop_lowfit = TRUE)

## ----general_plot3, fig.width = 7.5, fig.height = 4, out.width = "100%"-------
ssm_plot_curve(results3)

## ----general_plot4, fig.width = 7.5, fig.height = 4, out.width = "100%"-------
ssm_plot_curve(results3, drop_lowfit = TRUE)

## ----model_contrast-----------------------------------------------------------
results4 <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(), 
  grouping = "Gender", 
  contrast = TRUE
)
summary(results4)

## ----model_table, echo = FALSE------------------------------------------------
ssm_table(results4, render = FALSE) %>% 
  knitr::kable(caption = circumplex:::dcaption(results4)) %>% 
  kableExtra::kable_styling(full_width = TRUE, font_size = 12)

## ----model_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"----------
ssm_plot_contrast(results4)

## ----measure_contrast---------------------------------------------------------
results5 <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(), 
  measures = c("NARPD", "ASPD"),
  contrast = TRUE
)
summary(results5)

## ----measure_contrast_table, echo = FALSE, results = "asis"-------------------
ssm_table(results5, render = FALSE) %>% 
  knitr::kable(caption = circumplex:::dcaption(results5)) %>% 
  kableExtra::kable_styling(full_width = TRUE, font_size = 12)

## ----measure_contrast_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"----
ssm_plot_contrast(results5)

## ----group_contrast-----------------------------------------------------------
results6 <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(), 
  measures = "BORPD", 
  grouping = "Gender", 
  contrast = TRUE
)
summary(results6)

## ----group_contrast_table, echo = FALSE, results = "asis"---------------------
ssm_table(results6, render = FALSE) %>% 
  knitr::kable(caption = circumplex:::dcaption(results6)) %>% 
  kableExtra::kable_styling(full_width = TRUE, font_size = 12)

## ----group_contrast_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"----
ssm_plot_contrast(results6)

## ----taxonomy, echo = FALSE---------------------------------------------------
msr <- c("FALSE", "FALSE", "FALSE", "TRUE",  "TRUE",  "TRUE",  "TRUE")
grp <- c("FALSE", "TRUE",  "TRUE",  "FALSE", "FALSE", "TRUE",  "TRUE")
ctr <- c("FALSE", "FALSE", "TRUE",  "FALSE", "TRUE",  "FALSE", "TRUE")
tab <- data.frame(
  Usage = c(
    "Examine overall mean profile",
    "Examine groups' mean profiles",
    "Compare groups' mean profiles",
    "Examine variables' correlation profiles",
    "Compare variables' correlation profiles",
    "Examine groups' correlation profiles",
    "Compare groups' correlation profiles"
  ),
  measures = msr,
  grouping = grp,
  contrast = ctr
)
knitr::kable(tab, escape = FALSE) %>% 
  column_spec(1, width = "3in") %>% 
  add_header_above(c("", "Arguments Needed" = 3))

#!/usr/bin/env Rscript

# Export R circumplex test fixtures for Python regression testing
# This script generates JSON files containing expected outputs from R analyses
# to validate numerical parity in the Python port

library(circumplex)
library(jsonlite)

# Create output directory
output_dir <- "python_test_fixtures"
dir.create(output_dir, showWarnings = FALSE)

# Helper function to convert results to serializable format
prepare_results <- function(res) {
    list(
        results = as.data.frame(res$results),
        scores = as.data.frame(res$scores),
        details = res$details,
        type = res$type
    )
}

# Load example datasets
data("jz2017")
data("aw2009")

cat("Generating test fixtures for Python regression tests...\n\n")

# =============================================================================
# Test 1: Single-group mean-based SSM
# =============================================================================
cat("1. Single-group mean-based SSM (aw2009)...\n")
set.seed(12345)
res1 <- ssm_analyze(aw2009, scales = 1:8, boots = 2000, interval = 0.95)

# Extract key values for validation
fixture1 <- list(
    dataset = "aw2009",
    analysis_type = "single_group_mean",
    seed = 12345,
    input = list(
        scales = names(aw2009)[1:8],
        boots = 2000,
        interval = 0.95,
        listwise = TRUE
    ),
    expected = list(
        # Parameter estimates
        e_est = round(res1$results$e_est, 3),
        x_est = round(res1$results$x_est, 3),
        y_est = round(res1$results$y_est, 3),
        a_est = round(res1$results$a_est, 3),
        d_est = as.numeric(round(res1$results$d_est, 3)),
        fit_est = round(res1$results$fit_est, 3),
        # Confidence intervals
        e_lci = round(res1$results$e_lci, 3),
        e_uci = round(res1$results$e_uci, 3),
        x_lci = round(res1$results$x_lci, 3),
        x_uci = round(res1$results$x_uci, 3),
        y_lci = round(res1$results$y_lci, 3),
        y_uci = round(res1$results$y_uci, 3),
        a_lci = round(res1$results$a_lci, 3),
        a_uci = round(res1$results$a_uci, 3),
        d_lci = as.numeric(round(res1$results$d_lci, 3)),
        d_uci = as.numeric(round(res1$results$d_uci, 3)),
        # Scale scores
        scores = as.list(round(res1$scores[1, names(aw2009)[1:8]], 3))
    )
)
write_json(fixture1, file.path(output_dir, "ssm_single_group_mean.json"),
    auto_unbox = TRUE, pretty = TRUE
)

# =============================================================================
# Test 2: Multiple-group mean-based SSM
# =============================================================================
cat("2. Multiple-group mean-based SSM (jz2017)...\n")
set.seed(12345)
res2 <- ssm_analyze(jz2017, scales = 2:9, grouping = "Gender", boots = 2000)

fixture2 <- list(
    dataset = "jz2017",
    analysis_type = "multi_group_mean",
    seed = 12345,
    input = list(
        scales = names(jz2017)[2:9],
        grouping = "Gender",
        boots = 2000,
        interval = 0.95
    ),
    expected = list(
        # Parameter estimates (2 groups)
        e_est = round(res2$results$e_est, 3),
        x_est = round(res2$results$x_est, 3),
        y_est = round(res2$results$y_est, 3),
        a_est = round(res2$results$a_est, 3),
        d_est = as.numeric(round(res2$results$d_est, 3)),
        fit_est = round(res2$results$fit_est, 3),
        labels = as.character(res2$results$Label),
        # Confidence intervals
        e_lci = round(res2$results$e_lci, 3),
        e_uci = round(res2$results$e_uci, 3),
        x_lci = round(res2$results$x_lci, 3),
        x_uci = round(res2$results$x_uci, 3),
        y_lci = round(res2$results$y_lci, 3),
        y_uci = round(res2$results$y_uci, 3),
        a_lci = round(res2$results$a_lci, 3),
        a_uci = round(res2$results$a_uci, 3),
        d_lci = as.numeric(round(res2$results$d_lci, 3)),
        d_uci = as.numeric(round(res2$results$d_uci, 3)),
        # Scale scores (both groups)
        scores_female = as.list(round(res2$scores[1, names(jz2017)[2:9]], 3)),
        scores_male = as.list(round(res2$scores[2, names(jz2017)[2:9]], 3))
    )
)
write_json(fixture2, file.path(output_dir, "ssm_multi_group_mean.json"),
    auto_unbox = TRUE, pretty = TRUE
)

# =============================================================================
# Test 3: Multiple-group mean-based SSM with contrast
# =============================================================================
cat("3. Multiple-group mean-based SSM with contrast (jz2017)...\n")
set.seed(12345)
res3 <- ssm_analyze(jz2017,
    scales = 2:9, grouping = "Gender",
    contrast = TRUE, boots = 2000
)

fixture3 <- list(
    dataset = "jz2017",
    analysis_type = "multi_group_mean_contrast",
    seed = 12345,
    input = list(
        scales = names(jz2017)[2:9],
        grouping = "Gender",
        contrast = TRUE,
        boots = 2000
    ),
    expected = list(
        # All 3 rows: Female, Male, Male - Female
        e_est = round(res3$results$e_est, 3),
        x_est = round(res3$results$x_est, 3),
        y_est = round(res3$results$y_est, 3),
        a_est = round(res3$results$a_est, 3),
        d_est = as.numeric(round(res3$results$d_est, 3)),
        fit_est = round(res3$results$fit_est, 3),
        labels = as.character(res3$results$Label),
        # CIs for all rows
        e_lci = round(res3$results$e_lci, 3),
        e_uci = round(res3$results$e_uci, 3),
        a_lci = round(res3$results$a_lci, 3),
        a_uci = round(res3$results$a_uci, 3),
        d_lci = as.numeric(round(res3$results$d_lci, 3)),
        d_uci = as.numeric(round(res3$results$d_uci, 3))
    )
)
write_json(fixture3, file.path(output_dir, "ssm_multi_group_contrast.json"),
    auto_unbox = TRUE, pretty = TRUE
)

# =============================================================================
# Test 4: Single-group correlation-based SSM
# =============================================================================
cat("4. Single-group correlation-based SSM (jz2017)...\n")
set.seed(12345)
res4 <- ssm_analyze(jz2017, scales = 2:9, measures = "PARPD", boots = 2000)

fixture4 <- list(
    dataset = "jz2017",
    analysis_type = "single_group_correlation",
    seed = 12345,
    input = list(
        scales = names(jz2017)[2:9],
        measures = "PARPD",
        boots = 2000
    ),
    expected = list(
        e_est = round(res4$results$e_est, 3),
        x_est = round(res4$results$x_est, 3),
        y_est = round(res4$results$y_est, 3),
        a_est = round(res4$results$a_est, 3),
        d_est = as.numeric(round(res4$results$d_est, 1)),
        fit_est = round(res4$results$fit_est, 3),
        e_lci = round(res4$results$e_lci, 3),
        e_uci = round(res4$results$e_uci, 3),
        x_lci = round(res4$results$x_lci, 3),
        x_uci = round(res4$results$x_uci, 3),
        y_lci = round(res4$results$y_lci, 3),
        y_uci = round(res4$results$y_uci, 3)
    )
)
write_json(fixture4, file.path(output_dir, "ssm_single_group_correlation.json"),
    auto_unbox = TRUE, pretty = TRUE
)

# =============================================================================
# Test 5: Multi-group correlation-based SSM (no contrast)
# =============================================================================
cat("5. Multi-group correlation-based SSM (jz2017)...\n")
set.seed(12345)
res5 <- ssm_analyze(jz2017,
    scales = 2:9, measures = "PARPD",
    grouping = "Gender", boots = 2000
)

fixture5 <- list(
    dataset = "jz2017",
    analysis_type = "multi_group_correlation",
    seed = 12345,
    input = list(
        scales = names(jz2017)[2:9],
        measures = "PARPD",
        grouping = "Gender",
        boots = 2000
    ),
    expected = list(
        # Both groups
        e_est = round(res5$results$e_est, 3),
        x_est = round(res5$results$x_est, 3),
        y_est = round(res5$results$y_est, 3),
        a_est = round(res5$results$a_est, 3),
        d_est = as.numeric(round(res5$results$d_est, 1)),
        fit_est = round(res5$results$fit_est, 3),
        labels = as.character(res5$results$Label)
    )
)
write_json(fixture5, file.path(output_dir, "ssm_multi_group_correlation.json"),
    auto_unbox = TRUE, pretty = TRUE
)

# =============================================================================
# Test 6: Measure-contrast correlation-based SSM
# =============================================================================
cat("6. Measure-contrast correlation-based SSM (jz2017)...\n")
set.seed(12345)
res6 <- ssm_analyze(jz2017,
    scales = 2:9, measures = c("ASPD", "NARPD"),
    contrast = TRUE, boots = 2000
)

fixture6 <- list(
    dataset = "jz2017",
    analysis_type = "measure_contrast_correlation",
    seed = 12345,
    input = list(
        scales = names(jz2017)[2:9],
        measures = c("ASPD", "NARPD"),
        contrast = TRUE,
        boots = 2000
    ),
    expected = list(
        # All 3 rows: ASPD, NARPD, NARPD - ASPD
        e_est = round(res6$results$e_est, 3),
        x_est = round(res6$results$x_est, 3),
        y_est = round(res6$results$y_est, 3),
        a_est = round(res6$results$a_est, 3),
        d_est = as.numeric(round(res6$results$d_est, 1)),
        fit_est = round(res6$results$fit_est, 3),
        labels = as.character(res6$results$Label),
        # Some CIs
        e_lci = round(res6$results$e_lci, 3),
        e_uci = round(res6$results$e_uci, 3),
        a_lci = round(res6$results$a_lci, 3),
        a_uci = round(res6$results$a_uci, 3),
        # Scores for all three
        scores_aspd = as.list(round(res6$scores[1, names(jz2017)[2:9]], 3)),
        scores_narpd = as.list(round(res6$scores[2, names(jz2017)[2:9]], 3)),
        scores_contrast = as.list(round(res6$scores[3, names(jz2017)[2:9]], 3))
    )
)
write_json(fixture6, file.path(output_dir, "ssm_measure_contrast_correlation.json"),
    auto_unbox = TRUE, pretty = TRUE
)

# =============================================================================
# Test 7: Group-contrast correlation-based SSM
# =============================================================================
cat("7. Group-contrast correlation-based SSM (jz2017)...\n")
set.seed(12345)
res7 <- ssm_analyze(jz2017,
    scales = 2:9, measures = "NARPD",
    grouping = "Gender", contrast = TRUE, boots = 2000
)

fixture7 <- list(
    dataset = "jz2017",
    analysis_type = "group_contrast_correlation",
    seed = 12345,
    input = list(
        scales = names(jz2017)[2:9],
        measures = "NARPD",
        grouping = "Gender",
        contrast = TRUE,
        boots = 2000
    ),
    expected = list(
        # All 3 rows: Female, Male, Male - Female
        e_est = round(res7$results$e_est, 3),
        x_est = round(res7$results$x_est, 3),
        y_est = round(res7$results$y_est, 3),
        a_est = round(res7$results$a_est, 3),
        d_est = as.numeric(round(res7$results$d_est, 1)),
        fit_est = round(res7$results$fit_est, 3),
        labels = as.character(res7$results$Label),
        # CIs
        e_lci = round(res7$results$e_lci, 3),
        e_uci = round(res7$results$e_uci, 3),
        a_lci = round(res7$results$a_lci, 3),
        a_uci = round(res7$results$a_uci, 3)
    )
)
write_json(fixture7, file.path(output_dir, "ssm_group_contrast_correlation.json"),
    auto_unbox = TRUE, pretty = TRUE
)

# =============================================================================
# Test 8: SSM parameters function (no bootstrap)
# =============================================================================
cat("8. SSM parameters calculation...\n")

# Test data
test_scores <- c(0.374, -0.572, -0.520, 0.016, 0.688, 1.142, 1.578, 0.678)
test_angles <- octants()

params <- ssm_parameters(test_scores, test_angles)

fixture8 <- list(
    function_name = "ssm_parameters",
    input = list(
        scores = test_scores,
        angles = as.numeric(test_angles)
    ),
    expected = list(
        elevation = round(params[[1]], 6),
        x_value = round(params[[2]], 6),
        y_value = round(params[[3]], 6),
        amplitude = round(params[[4]], 6),
        displacement = round(params[[5]], 6), # radians
        fit = round(params[[6]], 6)
    )
)
write_json(fixture8, file.path(output_dir, "ssm_parameters.json"),
    auto_unbox = TRUE, pretty = TRUE
)

# =============================================================================
# Export example datasets as CSV
# =============================================================================
cat("\nExporting datasets to CSV...\n")
write.csv(jz2017, file.path(output_dir, "jz2017.csv"), row.names = FALSE)
write.csv(aw2009, file.path(output_dir, "aw2009.csv"), row.names = FALSE)

cat("\n✓ Test fixtures exported to", output_dir, "\n")
cat("✓ Datasets exported as CSV\n")
cat("\nFiles created:\n")
list.files(output_dir, full.names = FALSE)

#' Standardized octant scores on hypothetical circumplex scales
#'
#' A small example dataset containing standardized scores on eight hypothetical
#' circumplex scales. Taken from Wright, Pincus, Conroy, & Hilsenroth (2009).
#'
#' @format A data frame with 5 observations and 8 variables:
#' \describe{
#'   \item{PA}{circumplex scale at 90 degrees}
#'   \item{BC}{circumplex scale at 135 degrees}
#'   \item{DE}{circumplex scale at 180 degrees}
#'   \item{FG}{circumplex scale at 225 degrees}
#'   \item{HI}{circumplex scale at 270 degrees}
#'   \item{JK}{circumplex scale at 315 degrees}
#'   \item{LM}{circumplex scale at 360 degrees}
#'   \item{NO}{circumplex scale at 45 degrees}
#' }
#' @source \doi{10.1080/00223890902935696}
"aw2009"

#' Raw octant scores on real circumplex scales with covariates
#'
#' A large example dataset containing gender, raw mean scores on the Inventory
#' of Interpersonal Problems - Short Circumplex (IIP-SC), and raw sum scores on
#' the Personality Diagnostic Questionnaire - 4th Edition Plus (PDQ-4+).
#'
#' @format A data frame with 1166 observations and 19 variables:
#' \describe{
#'   \item{Gender}{Self-reported Gender}
#'   \item{PA}{Domineering Problems (IIP-SC) 90 degrees}
#'   \item{BC}{Vindictive Problems (IIP-SC) 135 degrees}
#'   \item{DE}{Cold Problems (IIP-SC) 180 degrees}
#'   \item{FG}{Socially Avoidant Problems (IIP-SC) 225 degrees}
#'   \item{HI}{Nonassertive Problems(IIP-SC) 270 degrees}
#'   \item{JK}{Easily Exploited Problems (IIP-SC) 315 degrees}
#'   \item{LM}{Overly Nurturant Problems (IIP-SC) 360 degrees}
#'   \item{NO}{Intrusive Problems (IIP-SC) 45 degrees}
#'   \item{PARPD}{Paranoid PD Symptoms (PDQ-4+)}
#'   \item{SCZPD}{Schizoid PD Symptoms (PDQ-4+)}
#'   \item{SZTPD}{Schizotypal PD Symptoms (PDQ-4+)}
#'   \item{ASPD}{Antisocial PD Symptoms (PDQ-4+)}
#'   \item{BORPD}{Borderline PD Symptoms (PDQ-4+)}
#'   \item{HISPD}{Histrionic PD Symptoms (PDQ-4+)}
#'   \item{NARPD}{Narcissistic PD Symptoms (PDQ-4+)}
#'   \item{AVPD}{Avoidant PD Symptoms (PDQ-4+)}
#'   \item{DPNPD}{Dependent PD Symptoms (PDQ-4+)}
#'   \item{OCPD}{Obsessive-Compulsive PD Symptoms (PDQ-4+)}
#' }
#' @source \doi{10.1177/1073191115621795}
"jz2017"

#' Raw item responses on real circumplex scales
#'
#' A small example dataset containing raw item responses on the Inventory of
#' Interpersonal Problems, Short Circumplex (IIP-SC). This data set is useful
#' for testing functions that operate on item-level data.
#'
#' @format A data frame with 10 observations and 32 variables.
"raw_iipsc"

# Perform bootstrap to get confidence intervals around SSM parameters
ssm_bootstrap <- function(bs_input, bs_function, scales, measures = NULL, 
                          angles, boots, interval, contrast, listwise, ...) {

  # Perform bootstrapping ------------------------------------------------------
  bs_results <- 
    boot::boot(
      data = bs_input,
      statistic = bs_function,
      R = boots,
      scales = scales,
      measures = measures,
      angles = angles,
      contrast = contrast,
      listwise = listwise,
      ...
    )

  # Extract point estimates from bootstrap results -----------------------------
  bs_est <- reshape_params(bs_results$t0, suffix = "est")
  bs_t <- bs_results$t
  bs_t <- as.data.frame(bs_t)
  colnames(bs_t) <- paste0(
    c("e", "x", "y", "a", "d", "fit"), 
    rep(1:nrow(bs_est), each = 6)
  )
  
  # Set the units of the displacement results to radians -----------------------
  if (contrast) {
    # Don't set to rad for contrasted d parameter (we want to allow negatives)
    d_vars <- 1:((ncol(bs_t) - 6) / 6) * 6 - 1
  } else {
    d_vars <- 1:(ncol(bs_t) / 6) * 6 - 1
  }
  bs_t[d_vars] <- lapply(bs_t[d_vars], new_radian)

  # Calculate the lower bounds of the confidence intervals ---------------------
  bs_lci <- sapply(bs_t, quantile, probs = ((1 - interval) / 2))
  bs_lci <- reshape_params(bs_lci, suffix = "lci")
  bs_lci$fit_lci <- NULL

  # Calculate the upper bounds of the confidence intervals ---------------------
  bs_uci <- sapply(bs_t, quantile, probs = (1 - (1 - interval) / 2))
  bs_uci <- reshape_params(bs_uci, suffix = "uci")
  bs_uci$fit_uci <- NULL

  # Combine the results in one data frame and convert radians to degrees -------
  out <- cbind(bs_est, bs_lci, bs_uci)
  out[c("d_est", "d_lci", "d_uci")] <- lapply(
    out[c("d_est", "d_lci", "d_uci")], 
    function(x) as_degree(as_radian(x))
  )
  
  out
}

# Calculate SSM parameters per group (or parameter differences)
ssm_by_group <- function(scores, angles, contrast) {
  
  # Calculate SSM parameters per group  
  results <- group_parameters(scores, angles)
  
  # If contrasting, append SSM parameter differences
  if (contrast) {
    results <- c(results, param_diff(results[7:12], results[1:6]))
  }

  results
}

# Calculate quantiles for circular data in radians
#' @export
quantile.circumplex_radian <- function(x, na.rm = TRUE, ...) {
  if (all(is.na(x))) return(NA)
  x <- unclass(x)
  mean_angle <- atan2(mean(sin(x), na.rm = na.rm), mean(cos(x), na.rm = na.rm))
  angles_centered <- (x - mean_angle + pi) %% (2 * pi) - pi
  quantiles_centered <- stats::quantile(angles_centered, na.rm = na.rm, ...)
  out <- (quantiles_centered + mean_angle) %% (2 * pi)
  out[abs(out - (2 * pi)) < (.Machine$double.eps * 2)] <- 0
  as_radian(out)
}

#' Child and Adolescent Interpersonal Survey
#'
#' Information about the Child and Adolescent Interpersonal Survey (CAIS).
#'
#' @source Sodano, S. M., & Tracey, T. J. G. (2006). Interpersonal traits in
#'   childhood: Development of the Child and Adolescent Interpersonal Survey.
#'   *Journal of Personality Assessment, 87*(3), 317–329.
#' @source \doi{10.1207/s15327752jpa8703_12}
#' @keywords internal
#' @examples
#' summary(cais)
#' scales(cais, items = TRUE)
"cais"


#' Circumplex Scales of Interpersonal Efficacy
#'
#' Information about the Circumplex Scales of Interpersonal Efficacy (CSIE).
#'
#' @source Locke, K. D., & Sadler, P. (2007). Self-efficacy, values, and
#'   complementarity in dyadic interactions: Integrating interpersonal and
#'   social-cognitive theory. *Personality and Social Psychology Bulletin,
#'   33*(1), 94-109.
#' @source \url{https://kennethlocke.org/CSIE/CSIE.html}
#' @keywords internal
#' @examples
#' summary(csie)
#' scales(csie, items = TRUE)
"csie"

#' Circumplex Scales of Intergroup Goals
#'
#' Information about the Circumplex Scales of Intergroup Goals (CSIG).
#'
#' @source Locke, K. D. (2014). Circumplex scales of intergroup goals: An
#'   interpersonal circle model of goals for interactions between groups.
#'   *Personality and Social Psychology Bulletin, 40*(4), 433-449.
#' @source \url{https://kennethlocke.org/CSIG/CSIG.html}
#' @keywords internal
#' @examples
#' summary(csig)
#' scales(csig, items = TRUE)
"csig"

#' Circumplex Scales of Interpersonal Problems
#'
#' Information about the Circumplex Scales of Interpersonal Problems (CSIP).
#'
#' @source Boudreaux, M. J., Ozer, D. J., Oltmanns, T. F., & Wright, A. G. C.
#'   (2018). Development and validation of the circumplex scales of
#'   interpersonal problems. *Psychological Assessment, 30*(5), 594-609.
#' @source \doi{10.1037/pas0000505}
#' @keywords internal
#' @examples
#' summary(csip)
#' scales(csip, items = TRUE)
"csip"

#' Circumplex Scales of Interpersonal Values
#'
#' Information about the Circumplex Scales of Interpersonal Values (CSIV).
#'
#' @source Locke, K. D. (2000). Circumplex scales of interpersonal values:
#'   Reliability, validity, and applicability to interpersonal problems and
#'   personality disorders. *Journal of Personality Assessment, 75*(2), 249-267.
#' @source \url{https://kennethlocke.org/CSIV/CSIV.html}
#' @keywords internal
#' @examples
#' summary(csiv)
#' scales(csiv, items = TRUE)
"csiv"

#' Interpersonal Emotion Inventory
#'
#' Information about the Interpersonal Emotion Inventory (IEI).
#'
#' @source Horner, M. S., Locke, K. D., & Hulsey, T. L. (in press). Assessing
#'   affective dimensions of the interpersonal circumplex: development and
#'   validation of the interpersonal emotion inventory. *Journal of Personality
#'   Assessment*. Advanced online publication.
#' @source \doi{10.1080/00223891.2024.2400266}
#' @source \url{https://kennethlocke.org/IEI/IEI.html}
#' @keywords internal
#' @examples
#' summary(iei)
#' scales(iei, items = TRUE)
"iei"

#' Interpersonal Goals Inventory for Children, Revised Version
#'
#' Information about the Interpersonal Goals Inventory for Children, Revised
#' Version (IGI-CR).
#'
#' @source Trucco, E. M., Wright, A. G. C., & Colder, C. R. (2013). A revised
#'   interpersonal circumplex inventory of children’s social goals. *Assessment,
#'   20*(1), 98-113.
#' @source \doi{10.1177/1073191111411672}
#' @keywords internal
#' @examples
#' summary(igicr)
#' scales(igicr, items = TRUE)
"igicr"

#' Inventory of Interpersonal Problems, Brief Version
#'
#' Information about the Inventory of Interpersonal Problems, Brief Version
#' (IIP-32). Note that, although we have permission to provide some information
#' about the IIP-32, Mind Garden Inc. has exclusive rights to distribute it in
#' full.
#'
#' @source Horowitz, L. M., Alden, L. E., Wiggins, J. S., & Pincus, A. L.
#'   (2000). *IIP-64/IIP-32 professional manual.* San Antonio, TX: The
#'   Psychological Corporation.
#' @source \url{https://www.mindgarden.com/113-inventory-of-interpersonal-problems}
#' @keywords internal
#' @examples
#' summary(iip32)
"iip32"

#' Inventory of Interpersonal Problems
#'
#' Information about the Inventory of Interpersonal Problems (IIP-64). Note
#' that, although we have permission to provide some information about the
#' IIP-64, Mind Garden Inc. has exclusive rights to distribute it in full.
#'
#' @source Horowitz, L. M., Alden, L. E., Wiggins, J. S., & Pincus, A. L.
#'   (2000). *IIP-64/IIP-32 professional manual.* San Antonio, TX: The
#'   Psychological Corporation.
#' @source \url{https://www.mindgarden.com/113-inventory-of-interpersonal-problems}
#' @keywords internal
#' @examples
#' summary(iip64)
"iip64"

#' Inventory of Interpersonal Problems, Short Circumplex
#'
#' Information about the Inventory of Interpersonal Problems Short Circumplex
#' (IIP-SC).
#'
#' @source Soldz, S., Budman, S., Demby, A., & Merry, J. (1995). A short form of
#'   the Inventory of Interpersonal Problems Circumplex Scales. *Assessment,
#'   2*(1), 53-63.
#' @source \doi{10.1080/00223890802388665}
#' @keywords internal
#' @examples
#' summary(iipsc)
"iipsc"

#' Inventory of Interpersonal Strengths, Brief Version
#'
#' Information about the Inventory of Interpersonal Strengths, Brief Version
#' (IIS-32).
#'
#' @source Hatcher, R. L., & Rogers, D. T. (2012). The IIS-32: A brief inventory
#'   of interpersonal strengths. *Journal of Personality Assessment, 94*(6),
#'   638-646.
#' @source \doi{10.1080/00223891.2012.681818}
#' @keywords internal
#' @examples
#' summary(iis32)
#' scales(iis32, items = TRUE)
"iis32"

#' Inventory of Interpersonal Strengths
#'
#' Information about the Inventory of Interpersonal Strengths (IIS-64).
#'
#' @source Hatcher, R. L., & Rogers, D. T. (2009). Development and validation of
#'   a measure of interpersonal strengths: The Inventory of Interpersonal
#'   Strengths. *Psychological Assessment, 21*(4), 554-569.
#' @source \doi{10.1037/a0017269}
#' @keywords internal
#' @examples
#' summary(iis64)
#' scales(iis64, items = TRUE)
"iis64"

#' Inventory of Influence Tactics Circumplex
#'
#' Information about the Inventory of Influence Tactics Circumplex (IIT-C).
#'
#' @source Bliton, C. F., & Pincus, A. L. (in press). Construction and
#'   Validation of the Interpersonal Influence Tactics Circumplex (IIT-C)
#'   Scales. *Assessment*.
#' @source \doi{10.1177/1073191119864661}
#' @keywords internal
#' @examples
#' summary(iitc)
#' scales(iitc, items = TRUE)
"iitc"

#' IPIP Interpersonal Circumplex
#'
#' Information about the IPIP Interpersonal Circumplex (IPIP-IPC).
#'
#' @source Markey, P. M., & Markey, C. N. (2009). A brief assessment of the
#'   interpersonal circumplex: The IPIP-IPC. *Assessment, 16*(4), 352-361.
#' @source \doi{10.1177/1073191109340382}
#' @keywords internal
#' @examples
#' summary(ipipipc)
#' scales(ipipipc, items = TRUE)
"ipipipc"

#' Interpersonal Sensitivities Circumplex
#'
#' Information about the Interpersonal Sensitivities Circumplex (ISC).
#'
#' @source Hopwood, C. J., Ansell, E. B., Pincus, A. L., Wright, A. G. C.,
#'   Lukowitsky, M. R., & Roche, M. J. (2011). The circumplex structure of
#'   interpersonal sensitivities. *Journal of Personality, 79*(4), 707-740.
#' @source \doi{10.1111/j.1467-6494.2011.00696.x}
#' @keywords internal
#' @examples
#' summary(isc)
#' scales(isc, items = TRUE)
"isc"

new_instrument <- function(Scales, Anchors, Items, Norms, Details, ...) {
  new_s3_scalar(
    Scales = Scales,
    Anchors = Anchors,
    Items = Items,
    Norms = Norms,
    Details = Details,
    ...,
    class = "circumplex_instrument"
  )
}

is_instrument <- function(x) {
  typeof(x) == "list" & class(x) == "circumplex_instrument"
}

#' @export
print.circumplex_instrument <- function(x, ...) {
  cat(
    x$Details$Abbrev, ": ", x$Details$Name, "\n",
    x$Details$Items, " items, ", x$Details$Scales, " scales, ", 
    nrow(x$Norms[[2]]), " normative data sets", "\n",
    x$Details$Reference, "\n",
    "<", x$Details$URL, ">", "\n",
    sep = ""
  )
}

#' @export
summary.circumplex_instrument <- function(object, scales = TRUE, anchors = TRUE,
                               items = TRUE, norms = TRUE, ...) {
  x <- object
  print(x)
  if (scales) {
    cat("\n")
    scales(x)
  }
  if (anchors) {
    cat("\n")
    anchors(x)
  }
  if (items) {
    cat("\n")
    items(x)
  }
  if (norms) {
    cat("\n")
    norms(x)
  }
}

#' Display the scales of a circumplex instrument
#'
#' Display the scales of a circumplex instrument including the total number of
#' scales and each scale's abbreviation, hypothetical angle, and text label.
#'
#' @param x Required. An object of the instrument class.
#' @param items Optional. A logical determining whether the items for each scale
#'   should be displayed below its other information (default = FALSE).
#' @return The same input object. Prints text to console.
#' @family instrument functions
#' @export
#' @examples
#' scales(csip)
#' scales(csip, items = TRUE)
scales <- function(x, items = FALSE) {
  stopifnot(is_instrument(x))
  stopifnot(is.logical(items) && length(items) == 1)

  cat("The ", x$Details$Abbrev, " contains ", x$Details$Scales, 
      " circumplex scales.\n", sep = "")
  for (i in 1:nrow(x$Scales)) {
    xi <- x$Scales[i, ]
    cat(xi$Abbrev, ": ", xi$Label, " (", xi$Angle, " degrees)", "\n", sep = "")
    if (items == TRUE) {
      item_nums <- as.integer(strsplit(xi$Items, ",")[[1]])
      for (j in 1:length(item_nums)) {
        num_j <- item_nums[[j]]
        item_j <- x$Items[[num_j, "Text"]]
        cat("    ", num_j, ". ", item_j, "\n", sep = "")
      }
    }
  }

  invisible(x)
}

#' Display the items of a circumplex instrument
#'
#' Display the items of a circumplex instrument including the total number of
#' items and each item's number and text. The item ordering/numbering displayed
#' here is the same ordering/numbering assumed by the \code{score()} function.
#'
#' @param x Required. An object of the instrument class.
#' @return The same input object. Prints text to console.
#' @family instrument functions
#' @export
#' @examples
#' items(csip)
items <- function(x) {
  stopifnot(is_instrument(x))

  cat("The ", x$Details$Abbrev, " contains ", x$Details$Items, " items (", 
    x$Details$Status, "):\n",
    ifelse(x$Details$Prefix != "", paste("Prefix: ", x$Details$Prefix, "\n", sep = ""), ""),
    ifelse(x$Details$Suffix != "", paste("Suffix: ", x$Details$Suffix, "\n", sep = ""), ""),
    sep = ""
  )
  for (i in 1:nrow(x$Items)) {
    xi <- x$Items[i, ]
    if (!is.na(xi$Number)) {
      cat(xi$Number, ". ", sep = "")
    }
    cat(xi$Text, "\n", sep = "")
  }

  invisible(x)
}

#' Display the anchors of a circumplex instrument
#'
#' Display the anchors of a circumplex instrument including the total number of
#' anchors and each anchor's numerical value and text label. Anchors are the
#' response options that respondants select from (e.g., 0 = No, 1 = Yes).
#'
#' @param x Required. An object of the instrument class.
#' @return The same input object. Prints text to console.
#' @family instrument functions
#' @export
#' @examples
#' anchors(csip)
anchors <- function(x) {
  stopifnot(is_instrument(x))

  cat(
    "The ", x$Details$Abbrev, " is rated using the following ",
    nrow(x$Anchors), "-point scale.", "\n", sep = ""
  )
  for (i in seq_along(x$Anchors$Value)) {
    cat(x$Anchors$Value[[i]], ". ", x$Anchors$Label[[i]], "\n",sep = "")
  }

  invisible(x)
}

#' Display the norms for a circumplex instrument
#'
#' Display the norms for a circumplex instrument including the total number of
#' normative data sets available and each data set's number, sample size,
#' population, and source reference and hyperlink. If another normative data set
#' exists that is not yet included in the package, please let us know.
#'
#' @param x Required. An object of the instrument class.
#' @return The same input object. Prints text to console.
#' @family instrument functions
#' @export
#' @examples
#' norms(csip)
norms <- function(x) {
  
  stopifnot(is_instrument(x))

  samples <- x$Norms[[2]]
  n_norms <- nrow(samples)

  cat("The ", x$Details$Abbrev, " currently has ", n_norms, 
      " normative data set(s):", "\n", sep = "")

  for (i in 1:n_norms) {
    sample_i <- samples$Sample[[i]]
    size_i <- samples$Size[[i]]
    pop_i <- samples$Population[[i]]
    cat(
      sample_i, ". ", size_i, " ", pop_i, "\n",
      samples$Reference[[i]], "\n",
      "<", samples$URL[[i]], ">", "\n",
      sep = ""
    )
  }

  invisible(x)
}


#' List all available instruments
#'
#' The circumplex package includes information about numerous circumplex
#' instruments including instructions for scoring and standardizing items.
#' Individual instruments can be loaded using the \code{instrument} function.
#'
#' @family instrument functions
#' @export
#' @examples
#' instruments()
instruments <- function() {

  # TODO: Find a way to automate this - maybe data$results minus example data?

  cat(
    "The circumplex package currently includes 14 instruments:\n",
    "1. CAIS: Child and Adolescent Interpersonal Survey (cais)\n",
    "2. CSIE: Circumplex Scales of Interpersonal Efficacy (csie)\n",
    "3. CSIG: Circumplex Scales of Intergroup Goals (csig)\n",
    "4. CSIP: Circumplex Scales of Interpersonal Problems (csip)\n",
    "5. CSIV: Circumplex Scales of Interpersonal Values (csiv)\n",
    "6. IEI: Interpersonal Emotion Inventory (iei)\n",
    "7. IGI-CR: Interpersonal Goals Inventory for Children, Revised Version (igicr)\n",
    "8. IIP-32: Inventory of Interpersonal Problems, Brief Version (iip32)\n",
    "9. IIP-64: Inventory of Interpersonal Problems (iip64)\n",
    "10. IIP-SC: Inventory of Interpersonal Problems, Short Circumplex (iipsc)\n",
    "11. IIS-32: Inventory of Interpersonal Strengths, Brief Version (iis32)\n",
    "12. IIS-64: Inventory of Interpersonal Strengths (iis64)\n",
    "13. IIT-C: Inventory of Influence Tactics Circumplex (iitc)\n",
    "14. IPIP-IPC: IPIP Interpersonal Circumplex (ipipipc)\n",
    "15. ISC: Interpersonal Sensitivities Circumplex (isc)\n"
  )
}

test_that("S3 class constructors for instrument class work as expected", {
  i <- new_instrument(list(), list(), list(), list(), list())
  expect_s3_class(i, "circumplex_instrument")
})


test_that("The print method for the S3 instrument class produces the right output", {
  expect_snapshot_output(print(isc))
})


test_that("The summary method for the S3 instrument class produces the right output", {
  expect_snapshot_output(summary(isc))
  expect_snapshot_output(summary(isc, scales = FALSE, anchors = FALSE, items = FALSE, norms = FALSE))
  expect_snapshot_output(summary(isc, scales = TRUE, anchors = FALSE, items = FALSE, norms = FALSE))
  expect_snapshot_output(summary(isc, scales = FALSE, anchors = TRUE, items = FALSE, norms = FALSE))
  expect_snapshot_output(summary(isc, scales = FALSE, anchors = FALSE, items = TRUE, norms = FALSE))
  expect_snapshot_output(summary(isc, scales = FALSE, anchors = FALSE, items = FALSE, norms = TRUE))
})


test_that("The sub-summary functions produce the expected output", {
  expect_snapshot_output(scales(isc))
  expect_snapshot_output(scales(isc, items = TRUE))
  expect_snapshot_output(items(isc))
  expect_snapshot_output(anchors(isc))
  expect_snapshot_output(norms(isc))
})


test_that("The instruments function produces the expected output", {
  expect_snapshot_output(instruments())
})

#' Create a Circular Plot of SSM Results
#'
#' Take in the results of a Structural Summary Method analysis and plot the
#' point and interval estimate for each row (e.g., group or measure) in a
#' circular space quantified by displacement and amplitude.
#'
#' @param ssm_object Required. The output of `ssm_analyze()`.
#' @param amax A positive real number corresponding to the radius of the circle.
#'   It is used to scale the amplitude values and will determine which amplitude
#'   labels are drawn.
#' @param legend_font_size A positive real number corresponding to the size (in
#'   pt) of the text labels in the legend (default = 12).
#' @param scale_font_size A positive real number corresponding to the size (in
#'   pt) of the text labels for the amplitude and displacement scales (default =
#'   12).
#' @param drop_lowfit A logical determining whether profiles with low model fit
#'   (<.70) should be omitted or plotted with dashed borders (default = FALSE).
#' @param repel An experimental argument for plotting text labels instead of
#'   colors.
#' @param angle_labels A character vector specifying text labels to plot around
#'   the circle for each scale. Can also specify NULL to default to numerical
#'   angle labels or a vector of empty strings ("") to hide the labels. If not
#'   NULL, must have the same length and ordering as the `angles` argument to
#'   `ssm_analyze()`. (default = NULL)
#' @param palette A string corresponding to the palette to be used from
#'   ColorBrewer for the color and fill aesthetics. If set to NULL, all points
#'   will appear blue and no legend will be there (useful for showing the
#'   coverage of a high number of variables).
#' @param vary_shapes A logical determining whether profiles should each get
#'   their own shape or vary only by fill color. This only works when the number
#'   of profiles is five or less. (default = FALSE)
#' @param ... Currently ignored.
#' @return A ggplot variable containing a completed circular plot.
#' @export
#' @examples
#' \donttest{
#' data("jz2017")
#' res <- ssm_analyze(
#'   jz2017,
#'   scales = 2:9,
#'   measures = c("NARPD", "ASPD")
#' )
#' ssm_plot_circle(res)
#' }
ssm_plot_circle <- function(ssm_object,
                            amax = NULL,
                            legend_font_size = 12,
                            scale_font_size = 12,
                            drop_lowfit = FALSE,
                            repel = FALSE,
                            angle_labels = NULL,
                            palette = "Set2",
                            vary_shapes = FALSE,
                            ...) {
  
  df <- ssm_object$results
  angles <- as.integer(round(ssm_object$details$angles))
  
  stopifnot(is_null_or_num(amax, n = 1))
  stopifnot(is_null_or_char(angle_labels, n = length(angles)))
  
  if (is.null(amax)) {
    amax <- pretty_max(ssm_object$results$a_uci)
  }
  
  if (ssm_object$details$contrast) {
    df <- df[1:2, ]
  }
  
  # Convert results to numbers usable by ggplot and ggforce
  df_plot <- df
  df_plot[["d_uci"]] <- ifelse(
    test = df_plot[["d_uci"]] < df_plot[["d_lci"]],
    yes = ggrad(df_plot[["d_uci"]] + 360),
    no = ggrad(df_plot[["d_uci"]])
  )
  df_plot[["d_lci"]] <- ggrad(df_plot[["d_lci"]])
  df_plot[c("a_lci", "a_uci", "x_est", "y_est")] <- sapply(
    df_plot[c("a_lci", "a_uci", "x_est", "y_est")],
    function(x) x * 10 / (2 * amax)
  )
  
  if (!is.null(palette)) {
    df_plot[["Label"]] <- factor(
      df_plot[["Label"]],
      levels = unique(as.character(df_plot[["Label"]]))
    )
  } else {
    df_plot[["Label"]] <- factor("All")
  }
  
  n_labels <- nlevels(df_plot$Label)
  
  # Remove profiles with low model fit (unless overrided)
  n <- nrow(df_plot)
  if (drop_lowfit) {
    df_plot <- df_plot[df_plot$fit_est >= .70, ]
    if (nrow(df_plot) < 1) {
      stop("After removing profiles with low fit, there were none left to plot.")
    }
  }
  df_plot[["lnty"]] <- ifelse(df_plot[["fit_est"]] >= .70, "solid", "dotted")
  
  ## Create circle base
  p <- circle_base(
    angles = angles,
    amax = amax,
    fontsize = scale_font_size,
    labels = angle_labels
  )
  
  ## Set color scales depending on palette
  if (is.null(palette)) {
    fill_color <- "#0072B2"
    p <- p +
      ggplot2::scale_color_manual(values = fill_color) +
      ggplot2::scale_fill_manual(values = fill_color) +
      ggplot2::guides(color = "none", fill = "none")
  } else {
    p <- p +
      ggplot2::scale_color_brewer(palette = palette) +
      ggplot2::scale_fill_brewer(palette = palette)
  }
  
  p <- p +
    ggplot2::scale_linetype_identity() +
    ggplot2::theme(
      legend.position = ifelse(repel | n_labels == 1 || is.null(palette), "none", "right"),
      legend.text = ggplot2::element_text(size = legend_font_size)
    )
  
  ## Add arc bars
  p <- p +
    ggforce::geom_arc_bar(
      data = df_plot,
      mapping = ggplot2::aes(
        x0 = 0,
        y0 = 0,
        r0 = .data$a_lci,
        r = .data$a_uci,
        start = .data$d_lci,
        end = .data$d_uci,
        fill = .data$Label,
        color = .data$Label,
        linetype = .data$lnty
      ),
      alpha = 0.4,
      linewidth = 1
    )
  
  ## Add points
  if (vary_shapes) {
    stopifnot(n_labels <= 5)
    p <- p +
      ggplot2::geom_point(
        data = df_plot,
        mapping = ggplot2::aes(
          x = .data$x_est,
          y = .data$y_est,
          fill = .data$Label,
          shape = .data$Label
        ),
        size = 3,
        color = "black"
      ) +
      ggplot2::scale_shape_manual(values = 21:(21 + n_labels - 1)) +
      ggplot2::guides(
        color = ggplot2::guide_legend("Profile"),
        fill = ggplot2::guide_legend("Profile"),
        shape = ggplot2::guide_legend("Profile")
      )
  } else {
    p <- p +
      ggplot2::geom_point(
        data = df_plot,
        mapping = ggplot2::aes(
          x = .data$x_est,
          y = .data$y_est,
          fill = .data$Label
        ),
        shape = 21,
        size = 3,
        color = "black"
      ) +
      ggplot2::guides(
        color = ggplot2::guide_legend("Profile"),
        fill = ggplot2::guide_legend("Profile")
      )
  }
  
  if (repel) {
    requireNamespace("ggrepel")
    p <- p +
      ggrepel::geom_label_repel(
        data = df_plot,
        mapping = ggplot2::aes(
          x = .data$x_est,
          y = .data$y_est,
          label = .data$Label
        ),
        nudge_x = -8 - df_plot$x_est,
        direction = "y",
        hjust = 1,
        size = legend_font_size / 2.8346438836889
      )
  }
  
  p
}


#' Create a Curve Plot of SSM Results
#'
#' Take in the results of a Structural Summary Method analysis and plot the
#' scores by angle and the estimated SSM curve.
#'
#' @param ssm_object Required. The results output of `ssm_analyze()`.
#' @param angle_labels Optional. Either NULL or a character vector that
#'   determines the x-axis labels. If NULL, the labels will be the angle
#'   numbers. If a character vector, must be the same length and in the same
#'   order as the `angles` argument to `ssm_analyze()` (default = NULL).
#' @param base_size Optional. A positive number corresponding to the base font
#'   size in pts (default = 11).
#' @param drop_lowfit Optional. A logical indicating whether to omit profiles with
#'   low fit (<.70) or include them with dashed lines (default = FALSE).
#' @param ... Additional arguments will be ignored.
#' @return A ggplot object depicting the SSM curve(s) of each profile.
#' @export
#' @examples
#' \donttest{
#' data("jz2017")
#' res <- ssm_analyze(
#'   jz2017,
#'   scales = 2:9,
#'   measures = 10:13
#' )
#' ssm_plot_curve(res)
#' ssm_plot_curve(res, angle_lables = PANO())
#' }
ssm_plot_curve <- function(ssm_object,
                           angle_labels = NULL,
                           base_size = 11,
                           drop_lowfit = FALSE,
                           ...) {

  stopifnot(class(ssm_object) == "circumplex_ssm")

  results <- ssm_object$results
  scores <- ssm_object$scores
  angles <- ssm_object$details$angles

  stopifnot(is_num(base_size, n = 1) && base_size > 0)
  stopifnot(is_null_or_char(angle_labels, n = length(angles)))
  stopifnot(is_flag(drop_lowfit))

  if (is.null(angle_labels)) {
    angle_labels <- function(x) sprintf("%.0f\U00B0", x)
    xlabel <- "Angle"
  } else {
    xlabel <- "Scale"
  }

  # Drop the contrast row if contrast
  if (ssm_object$details$contrast) {
    results <- results[1:2, ]
    scores <- scores[1:2, ]
  }

  # Drop profiles with low fit if requested
  if (drop_lowfit) {
    idx <- results$fit_est >= .70
    results <- results[idx, ]
    scores <- scores[idx, ]
  }

  # Drop the info columns
  scores_only <- scores[, -c(1:3)]

  # Reshape scores to long format
  score_df <- data.frame(
    Label = rep(scores$Label, times = length(angles)),
    Scale = rep(colnames(scores_only), each = nrow(scores)),
    Angle = rep(angles, each = nrow(scores)),
    Score = as.vector(unlist(scores_only))
  )
  curve_fit <- function(p, x) {
    p$e_est + p$a_est * cos((x - p$d_est) * pi / 180)
  }
  all_angles <- seq(from = min(angles), to = max(angles), length.out = 100)
  param_list <- split(results, results$Label)
  pred_mat <- sapply(param_list, FUN = curve_fit, x = all_angles)

  pred_df <- data.frame(
    Label = rep(colnames(pred_mat), each = nrow(pred_mat)),
    Angle = rep(all_angles, times = ncol(pred_mat)),
    Score = as.vector(pred_mat)
  )
  pred_df <- merge(pred_df, results[c("Label", "fit_est")])
  pred_df$lnty <- ifelse(pred_df$fit_est >= .70, "solid", "dashed")

  # Create ggplot
  ggplot2::ggplot() +
    ggplot2::facet_wrap(~Label) +
    # Curve
    ggplot2::geom_line(
      data = pred_df,
      mapping = ggplot2::aes(
        x = .data$Angle,
        y = .data$Score,
        linetype = .data$lnty,
        color = .data$Label
      ),
      linewidth = 1.25
    ) +
    # Connectors
    ggplot2::geom_line(
      data = score_df,
      mapping = ggplot2::aes(
        x = .data$Angle,
        y = .data$Score,
        group = .data$Label
      ),
      color = "black"
    ) +
    # Points
    ggplot2::geom_point(
      data = score_df,
      mapping = ggplot2::aes(
        x = .data$Angle,
        y = .data$Score,
        group = .data$Label
      ),
      color = "black"
    ) +
    ggplot2::scale_x_continuous(
      breaks = angles,
      labels = angle_labels
    ) +
    ggplot2::scale_linetype_identity() +
    ggplot2::labs(x = xlabel) +
    ggplot2::theme_bw() +
    ggplot2::theme(
      legend.position = "none",
      panel.grid.major.x = ggplot2::element_blank(),
      panel.grid.minor.x = ggplot2::element_blank()
    )
}

#' Create a Difference Plot of SSM Contrast Results
#'
#' Take in the results of a Structural Summary Method analysis with pairwise
#' contrasts and plot the point and interval estimates for each parameter's
#' contrast (e.g., between groups or measures).
#'
#' @param ssm_object Required. The results output of \code{ssm_analyze()}.
#' @param drop_xy A logical determining whether the X-Value and Y-Value
#'   parameters should be removed from the plot (default = FALSE).
#' @param sig_color Optional. A string corresponding to the color to use to
#'   denote significant contrasts (default = "#fc8d62").
#' @param ns_color Optional. A string corresponding to the color to use to
#'   denote non-significant contrasts (default = "white").
#' @param linesize Optional. A positive number corresponding to the size of the
#'   point range elements in mm (default = 1.5).
#' @param fontsize Optional. A positive number corresponding to the size of the
#'   axis labels, numbers, and facet headings in pt (default = 12).
#' @param ... Additional arguments will be ignored.
#' @return A ggplot variable containing difference point-ranges faceted by SSM
#'   parameter. An interval that does not contain the value of zero has p<.05.
#' @export
#' @examples
#' \donttest{
#' data("jz2017")
#' res <- ssm_analyze(
#'   jz2017,
#'   scales = 2:9,
#'   measures = c("NARPD", "ASPD"),
#'   contrast = TRUE
#' )
#' ssm_plot_contrast(res)
#' }
ssm_plot_contrast <- function(ssm_object, drop_xy = FALSE,
                              sig_color = "#fc8d62", ns_color = "white",
                              linesize = 1.25, fontsize = 12, ...) {

  stopifnot(ssm_object$details$contrast)

  # Prepare all estimates
  plabs <- c(
    e = expression(paste(Delta, " Elevation")),
    x = expression(paste(Delta, " X Value")),
    y = expression(paste(Delta, " Y Value")),
    a = expression(paste(Delta, " Amplitude")),
    d = expression(paste(Delta, " Displacement"))
  )
  pvals <- c("e", "x", "y", "a", "d")
  res <- ssm_object$results[nrow(ssm_object$results), ]

  plot_df <-
    data.frame(
      Parameter = factor(pvals, levels = pvals, labels = plabs),
      Difference = c(res$e_est, res$x_est, res$y_est, res$a_est, res$d_est),
      lci = c(res$e_lci, res$x_lci, res$y_lci, res$a_lci, res$d_lci),
      uci = c(res$e_uci, res$x_uci, res$y_uci, res$a_uci, res$d_uci)
    )

  plot_df$sig <- sign(plot_df$lci) == sign(plot_df$uci)

  # Drop x and y estimates if requested
  if (drop_xy) {
    plot_df <- plot_df[-c(2, 3), ]
  }

  p <-
    ggplot2::ggplot(plot_df) +
    ggplot2::facet_wrap(
      ~Parameter,
      nrow = 1,
      scales = "free",
      labeller = ggplot2::label_parsed
    ) +
    ggplot2::geom_hline(
      yintercept = 0,
      linewidth = linesize,
      color = "darkgray"
    ) +
    ggplot2::geom_errorbar(
      ggplot2::aes(
        x = "1",
        ymin = .data$lci,
        ymax = .data$uci
      ),
      linewidth = linesize,
      width = 0.15
    ) +
    ggplot2::geom_point(
      ggplot2::aes(
        x = "1",
        y = .data$Difference,
        fill = .data$sig
      ),
      size = linesize * 3,
      stroke = linesize,
      shape = 21
    ) +
    ggplot2::scale_fill_manual(
      name = "Significant",
      values = c("TRUE" = sig_color, "FALSE" = ns_color)
    ) +
    ggplot2::labs(y = paste0("Contrast (", res$Label, ")")) +
    ggplot2::theme_bw(base_size = fontsize) +
    ggplot2::theme(
      legend.position = "top",
      axis.text.x = ggplot2::element_blank(),
      axis.title.x = ggplot2::element_blank(),
      panel.grid.major.x = ggplot2::element_blank(),
      panel.grid.minor.y = ggplot2::element_line(linetype = "dashed"),
      axis.ticks.x = ggplot2::element_blank()
    )

  p
}

# Create an Empty Circular Plot
circle_base <- function(angles, labels = NULL, amin = 0,
                        amax = 0.5, fontsize = 12) {

  if (is.null(labels)) labels <- paste0(angles, "\u00B0")

  ggplot2::ggplot() +
    # Require plot to be square and remove default styling
    ggplot2::coord_fixed(clip = "off") +
    ggplot2::theme_void(base_size = fontsize) +
    # Expand the axes multiplicatively to fit labels
    ggplot2::scale_x_continuous(expand = c(0.25, 0)) +
    ggplot2::scale_y_continuous(expand = c(0.10, 0)) +
    # Draw lowest circle
    ggforce::geom_circle(
      mapping = ggplot2::aes(x0 = 0, y0 = 0, r = 5),
      color = "gray50",
      fill = "white",
      linewidth = 1.5
    ) +
    # Draw segments corresponding to displacement scale
    ggplot2::geom_segment(
      ggplot2::aes(
        x = 0,
        y = 0,
        xend = 5 * cos(angles * pi / 180),
        yend = 5 * sin(angles * pi / 180)
      ),
      color = "gray60",
      linewidth = 0.5
    ) +
    # Draw circles corresponding to amplitude scale
    ggforce::geom_circle(
      ggplot2::aes(x0 = 0, y0 = 0, r = 1:4),
      color = "gray60",
      linewidth = 0.5
    ) +
    # Draw labels for amplitude scale
    ggplot2::geom_label(
      ggplot2::aes(
        x = c(2, 4),
        y = 0,
        label = sprintf(
          "%.2f",
          seq(from = amin, to = amax, length.out = 6)[c(3, 5)]
        )
      ),
      color = "gray20",
      fill = "white",
      linewidth = NA,
      size = fontsize / 2.8346438836889
    ) +
    # Draw labels for displacement scale
    ggplot2::geom_label(
      ggplot2::aes(
        x = 5.1 * cos(angles * pi / 180),
        y = 5.1 * sin(angles * pi / 180),
        label = labels
      ),
      color = "gray20",
      fill = "transparent",
      linewidth = NA,
      hjust = "outward",
      vjust = "outward",
      size = fontsize / 2.8346438836889
    )
}

test_that("Single-group mean-based SSM plot is correct", {
  data("aw2009")
  res <- ssm_analyze(aw2009, scales = 1:8)
  p <- ssm_plot_circle(res)

  # Test the output object
  expect_true(ggplot2::is_ggplot(p))
  vdiffr::expect_doppelganger("single group mean ssm", p)

  p2 <- ssm_plot_circle(res, palette = NULL)
  vdiffr::expect_doppelganger("single group mean ssm no palette", p2)
  
  p3 <- ssm_plot_curve(res, angle_labels = PANO())
  vdiffr::expect_doppelganger("single group mean ssm with labels", p3)
})

test_that("Single-group correlation-based SSM plot is correct", {
  data("jz2017")
  res <- ssm_analyze(jz2017, scales = 2:9, measures = "PARPD")
  p <- ssm_plot_circle(res)
  
  # Test the output object
  expect_true(ggplot2::is_ggplot(p))
  vdiffr::expect_doppelganger("single group correlation ssm", p)
})

test_that("Measure-contrast SSM plot is correct", {
  data("jz2017")
  res <- ssm_analyze(
    jz2017,
    scales = 2:9,
    measures = c("ASPD", "NARPD"),
    contrast = TRUE
  )
  p <- ssm_plot_contrast(res, drop_xy = TRUE)

  # Test the output object
  expect_true(ggplot2::is_ggplot(p))
  vdiffr::expect_doppelganger("measure-contrast ssm", p)
  
  p2 <- ssm_plot_circle(res, drop_xy = TRUE)
  
  # Test the output object
  expect_true(ggplot2::is_ggplot(p))
  vdiffr::expect_doppelganger("measure-contrast-circle ssm", p2)
  
  p3 <- ssm_plot_curve(res, drop_lowfit = TRUE)
  vdiffr::expect_doppelganger("measure-contrast-curve ssm", p3)
})

test_that("Group-contrast correlation-based SSM plot is correct", {
  data("jz2017")
  res <- ssm_analyze(
    jz2017, 
    scales = 2:9, 
    measures = "NARPD",
    grouping = "Gender",
    contrast = TRUE
  )
  p <- ssm_plot_contrast(res)

  # Test the output object
  expect_true(ggplot2::is_ggplot(p))
  vdiffr::expect_doppelganger("group-constrast correlation ssm", p)
})

test_that("Removing plots with low fit works as expected", {
  data("jz2017")
  res <- ssm_analyze(jz2017, scales = 2:9, measures = "OCPD")
  expect_error(ssm_plot_circle(res, drop_lowfit = TRUE))
})

test_that("many plots works as expected", {
  data("jz2017")
  res <- ssm_analyze(jz2017, scales = 2:9, measures = 10:13)
  p <- ssm_plot_circle(res)
  vdiffr::expect_doppelganger("many_circle-plots", p)
  p2 <- ssm_plot_curve(res)
  vdiffr::expect_doppelganger("many_curve-plots", p2)
  p3 <- ssm_plot_circle(res, repel = TRUE)
  vdiffr::expect_doppelganger("many_circle repel", p3)
})

test_that("things are working at 0/360", {
  data("jz2017")
  set.seed(12345)
  dat <- jz2017[sample(1:nrow(jz2017), size = 100), ]
  res <- ssm_analyze(dat, 2:9, measures = 19)
  p <- ssm_plot_circle(res)
  vdiffr::expect_doppelganger("cross-zero circle", p)
})

#' Save a ggplot with sensible defaults
#'
#' See \code{ggplot2} for details.
#'
#' @name ggsave
#' @rdname ggsave
#' @keywords internal
#' @export
#' @importFrom ggplot2 ggsave
NULL

# Compute differences between two sets of SSM parameters -----------------------
param_diff <- function(p1, p2) {
  stopifnot(is.numeric(p1))
  stopifnot(is.numeric(p2))
  pd <- p1 - p2
  pd[[5]] <- angle_dist(as_radian(p1[[5]]), as_radian(p2[[5]]))
  pd
}

# Reshape parameters from wide to long format ----------------------------------
reshape_params <- function(v, suffix) {
  # Convert vector to matrix
  out <- matrix(v, ncol = 6, byrow = TRUE)
  # Add column names
  colnames(out) <- paste0(c("e_", "x_", "y_", "a_", "d_", "fit_"), suffix)
  # Convert to data frame
  as.data.frame(out)
}

# Calculate angular distance ---------------------------------------------------
angle_dist <- function(x, y) {
  ((x - y + pi) %% (2 * pi)) - pi
}

# Convert degrees to ggplot's radian format ------------------------------------
ggrad <- function(v) {
  v <- as.numeric(v)
  (v - 90) * (-pi / 180)
}

# Convert percent number to a formatted string ---------------------------------
str_percent <- function(x, digits = 2) {
  paste0(floor(x * 10^(digits + 2)) / (10^digits), "%")
}

# Determine good max amplitude value for circle plot ---------------------------
pretty_max <- function(v) {
  
  # What is the largest value?
  amax <- max(v, na.rm = TRUE)
  options <- c(
    -5.00, -4.00, -3.00, -2.50, -2.00,
    -1.50, -1.25, -1.00, -0.75, -0.50,
    -0.25, -0.20, -0.15, -0.10, -0.05,
    0,
    0.05, 0.10, 0.15, 0.20, 0.25,
    0.50, 0.75, 1.00, 1.25, 1.50,
    2.00, 2.50, 3.00, 4.00, 5.00
  )
  # If negative, decrease scalar (how much buffer space to add)
  if (amax < 0 ) {
    scalar <- 0.5
  } else {
    scalar <- 1.5
  }
  # Which options are good candidates?
  match <- options > amax * scalar
  # Are there any candidates?
  if (sum(match) >= 1) {
    # Take the smallest candidate 
    out <- options[match][[1]]
  } else {
    # Multiply the max and scalar
    out <- amax * scalar
  }
  out
}

# Determine good min amplitude value for circle plot ---------------------------
pretty_min <- function(v) {
  
  # What is the smallest value?
  amin <- min(v, na.rm = TRUE)
  options <- c(
    -5.00, -4.00, -3.00, -2.50, -2.00,
    -1.50, -1.25, -1.00, -0.75, -0.50,
    -0.25, -0.20, -0.15, -0.10, -0.05,
    0,
    0.05, 0.10, 0.15, 0.20, 0.25,
    0.50, 0.75, 1.00, 1.25, 1.50,
    2.00, 2.50, 3.00, 4.00, 5.00
  )
  # If negative, increase scalar (how much buffer space to add)
  if (amin < 0) {
    scalar <- 1.5
  } else {
    scalar <- 0.5
  }
  # Which options are candidates?
  match <- options < amin * scalar
  # Are there any candidates?
  if (sum(match) >= 1) {
    # Take the largest candidate
    candidates <- options[match]
    out <- candidates[length(candidates)]
  } else {
    # Multiply the min and scalar
    out <- amin * scalar
  }
  out
}


# Rescale numeric vector to specified min and max -------------------------
rescale <- function(x, to = c(0, 1), from = range(x, na.rm = TRUE)) {
  min_to <- to[1]
  max_to <- to[2]

  min_from <- from[1]
  max_from <- from[2]
  
  (x - min_from) / (max_from - min_from) * (max_to - min_to) + min_to
}

# Assertions --------------------------------------------------------------

is_count <- function(x) {
  all(
    is.numeric(x),
    ceiling(x) == floor(x),
    x >= 0
  )
}

is_char <- function(x, n = NULL) {
  if (is.null(n)) {
    is.character(x)
  } else {
    stopifnot(is_count(n))
    is.character(x) && length(x) == n
  }
}

is_null_or_char <- function(x, n = NULL) {
  is.null(x) || is_char(x, n = NULL)
}

is_var <- function(x, n = NULL) {
  if (is.null(n)) {
    is.character(x) || is.numeric(x)
  } else {
    stopifnot(is_count(n))
    (is.character(x) || is.numeric(x)) && length(x) == n
  }
}

is_null_or_var <- function(x, n = NULL) {
  is.null(x) || is_var(x, n)
}

is_flag <- function(x) {
  is.logical(x) && length(x) == 1
}

is_num <- function(x, n = NULL) {
  if (is.null(n)) {
    is.numeric(x)
  } else {
    stopifnot(is_count(n))
    is.numeric(x) && length(x) == n
  }
}

is_null_or_num <- function(x, n = NULL) {
  is.null(x) || is_num(x, n)
}

test_that("Percentages are converted to strings properly", {
  expect_equal(str_percent(0.95, 2), "95%")
  expect_equal(str_percent(0.999, 2), "99.9%")
  expect_equal(str_percent(0.9999, 2), "99.99%")
  expect_equal(str_percent(0.99999, 2), "99.99%")
})

test_that("pretty max outputs the correct values", {
  expect_warning(pretty_max(NA))
  expect_equal(pretty_max(0), 0.05)
  expect_equal(pretty_max(1), 2)
  expect_equal(pretty_max(2.4), 4)
  expect_equal(pretty_max(5), 7.5)
})

test_that("angle convenience functions work", {
  expect_equal(octants(), as_degree(c(90, 135, 180, 225, 270, 315, 360, 45)))
  expect_equal(poles(), as_degree(c(90, 180, 270, 360)))
  expect_equal(quadrants(), as_degree(c(135, 225, 315, 45)))
})

test_that("scales shortcut functions work", {
  expect_equal(PANO(), c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"))
})

test_that("assertions work", {
  expect_true(is_num(1))
  expect_false(is_num("A"))
  expect_false(is_count(-1))
})

test_that("rescale works", {
  expect_equal(rescale(1:3), c(0.0, 0.5, 1.0))
})

test_that("prettifying works", {
  expect_equal(pretty_min(0.5), 0.2)
  expect_equal(pretty_min(-0.5), -1.0)
  expect_equal(pretty_min(-10), -15)
})

#' Ipsatize circumplex items using deviation scoring across variables
#'
#' Rescore each circumplex item using deviation scoring across variables. In
#' other words, subtract each observation's mean response from each response.
#' This effectively removes the presence of a general factor, which can make
#' certain circumplex fit analyses more powerful.
#'
#' @param data Required. A data frame or matrix containing at least circumplex
#'   scales.
#' @param items Required. A character vector containing the column names, or a
#'   numeric vector containing column indexes, of item variables in `data` to be
#'   ipsatized.
#' @param na.rm Optional. A logical that determines whether missing values
#'   should be ignored during the calculation of the mean during ipsatization
#'   (default = TRUE).
#' @param prefix Optional. A string that will be added to the start of each
#'   `items` name in the output (default = "").
#' @param suffix Optional. A string that will be added to the end of each
#'   `items` name in the output (default = "_i").
#' @param append Optional. A logical that determines whether to append the
#'   ipsatized scores to `data` in the output or just return the ipsatized
#'   scores alone (default = TRUE).
#' @return A data frame that matches `data` except that the variables specified
#'   in `items` have been rescored using ipsatization.
#' @family tidying functions
#' @export
#' @examples
#' data("raw_iipsc")
#' ipsatize(raw_iipsc, items = 1:32)
#' ipsatize(raw_iipsc, items = sprintf("IIP%02d", 1:32))
ipsatize <- function(data, items, na.rm = TRUE, 
                     prefix = "", suffix = "_i", append = TRUE) {
  
  stopifnot(is.data.frame(data) || is.matrix(data))
  stopifnot(is_var(items))
  stopifnot(is_flag(na.rm))
  stopifnot(is_char(prefix, n = 1))
  stopifnot(is_char(suffix, n = 1))
  stopifnot(is_flag(append))

  item_data <- data[items]
  item_names <- colnames(item_data)
  rmean <- rowMeans(item_data, na.rm = na.rm)
  scores <- sapply(item_data, function(x) x - rmean)
  colnames(scores) <- paste0(prefix, item_names, suffix)
  
  if (append) {
    cbind(data, scores)
  } else {
    as.data.frame(scores)
  }
}

#' Score circumplex scales from item responses
#'
#' Calculate mean scores on circumplex scales from item responses by using a set
#' of scoring instructions, which may be loaded from the package or created as a
#' custom data frame.
#'
#' @param data Required. A data frame containing at least circumplex scales.
#' @param items Required. The variable names or column numbers for the variables
#'   in \code{.data} that contain all the circumplex items from a single
#'   circumplex measure, in ascending order from item 1 to item N.
#' @param instrument Required. An instrument object from the package. To see the
#'   available circumplex instruments, use \code{instruments()}.
#' @param na.rm Optional. A logical that determines if missing values should be
#'   omitted from the calculation of scores (default = TRUE). When set to TRUE,
#'   scales with missing data are essentially calculated with mean imputation.
#' @param prefix Optional. A string to include at the beginning of the newly
#'   calculated scale variables' names, before \code{Abbrev} from \code{key} and
#'   \code{suffix} (default = "").
#' @param suffix Optional. A string to include at the end of the newly
#'   calculated scale variables' names, after \code{Abbrev} from \code{key} and
#'   \code{prefix} (default = "").
#' @param append Optional. A logical that determines whether the calculated
#'   score variables will be appended to `data` or returned on their own
#'   (default = TRUE).
#' @return A data frame that matches \code{.data} except that new variables are
#'   appended that contain mean scores on each variable included in \code{key}.
#' @family tidying functions
#' @export
#' @examples
#' data("raw_iipsc")
#' score(raw_iipsc, items = 1:32, instrument = iipsc, prefix = "IIPSC_")
score <- function(data, items, instrument, na.rm = TRUE, 
                  prefix = "", suffix = "", append = TRUE) {
  
  stopifnot(is.data.frame(data) || is.matrix(data))
  stopifnot(is_var(items))
  stopifnot(class(instrument) == "circumplex_instrument")
  stopifnot(is_flag(na.rm))
  stopifnot(is_char(prefix))
  stopifnot(is_char(suffix))
  stopifnot(is_flag(append))
  
  item_data <- data[items]
  n_items <- length(items)
  key <- instrument$Scales

  scores <- matrix(NA, nrow = nrow(item_data), ncol = nrow(key))
  colnames(scores) <- paste0(prefix, key$Abbrev, suffix)
  
  for (i in 1:nrow(key)) {
    item_nums <- as.numeric(strsplit(key$Items[[i]], ",")[[1]])
    
    if (max(item_nums) > n_items) {
      stop("Key is asking for more items than were provided to function.")
    }
    
    scores[, i] <- rowMeans(item_data[item_nums], na.rm)
  }
  scores[is.nan(scores)] <- NA_real_
  
  if (append) {
    cbind(data, scores)
  } else {
    as.data.frame(scores)
  }
}

#' Standardize circumplex scales using normative data
#'
#' Take in a data frame containing circumplex scales, angle definitions for each
#' scale, and normative data (from the package or custom) and return that same
#' data frame with each specified circumplex scale transformed into standard
#' scores (i.e., z-scores) based on comparison to the normative data.
#'
#' @param data Required. A data frame or matrix containing at least circumplex
#'   scales.
#' @param scales Required. A character vector containing the column names, or a
#'   numeric vector containing the column indexes, for the variables (scale
#'   scores) to be standardized.
#' @param angles Required. A numeric vector containing the angular displacement
#'   of each circumplex scale included in `scales` (in degrees). Can use the
#'   `octants()`, `poles()`, or `quadrants()` convenience functions.
#' @param instrument Required. An instrument object from the package. To see the
#'   available circumplex instruments, see `instruments()`.
#' @param sample Required. An integer corresponding to the normative sample to
#'   use in standardizing the scale scores (default = 1). See `?norms` to
#'   see the normative samples available for an instrument.
#' @param prefix Optional. A string to include at the beginning of the newly
#'   calculated scale variables' names, before the scale name and `suffix`
#'   (default = "").
#' @param suffix Optional. A string to include at the end of the newly
#'   calculated scale variables' names, after the scale name and `prefix`
#'   (default = "_z").
#' @param append Optional. A logical that determines whether the calculated
#'   standardized scores should be added as columns to `data` in the output or
#'   the standardized scores alone should be output (default = TRUE).
#' @return A data frame that contains the norm-standardized versions of `scales`.
#' @export
#' @family tidying functions
#' @examples
#' data("jz2017")
#' norm_standardize(jz2017, scales = 2:9, instrument = iipsc, sample = 1)
norm_standardize <- function(data, scales, angles = octants(), instrument, 
                       sample = 1, prefix = "", suffix = "_z", append = TRUE) {
  
  stopifnot(is.data.frame(data) || is.matrix(data))
  stopifnot(is_var(scales))
  stopifnot(is.numeric(angles))
  stopifnot(length(scales) == length(angles))
  stopifnot(class(instrument) == "circumplex_instrument")
  stopifnot(is_num(sample, n = 1))
  stopifnot(is_char(prefix, n = 1))
  stopifnot(is_char(suffix, n = 1))
  stopifnot(is_flag(append))
  
  
  key <- instrument$Norms[[1]]
  key <- key[key$Sample == sample, ]
  
  stopifnot(length(scales) == nrow(key))
  
  scale_data <- data[scales]
  scale_names <- colnames(scale_data)
  
  scores <- matrix(NA, nrow = nrow(scale_data), ncol = length(scales))
  colnames(scores) <- paste0(prefix, scale_names, suffix)
  
  for (i in 1:length(scales)) {
    index_i <- key$Angle == angles[[i]]
    m_i <- key$M[index_i]
    s_i <- key$SD[index_i]
    scores[, i] <- (scale_data[[i]] - m_i) / s_i
  }
  scores[is.nan(scores)] <- NA_real_
  
  if (append) {
    cbind(data, scores)
  } else {
    as.data.frame(scores) 
  }
}

#' Standardize circumplex scales using sample data
#'
#' Take in a data frame containing circumplex scales (or items) and return that
#' same data frame with each specified variable transformed into standard scores
#' (i.e., z-scores) based on observed means and SDs.
#'
#' @param data Required. A data frame or matrix containing at least circumplex
#'   scales.
#' @param scales Required. A character vector containing the column names, or a
#'   numeric vector containing the column indexes, for the variables (scale
#'   scores) to be standardized.
#' @param na.rm Optional. A logical that determines whether to remove missing
#'   values from scales when calculating the means and SDs used for
#'   standardization (default = TRUE).
#' @param prefix Optional. A string to include at the beginning of the newly
#'   calculated scale variables' names, before the scale name and `suffix`
#'   (default = "").
#' @param suffix Optional. A string to include at the end of the newly
#'   calculated scale variables' names, after the scale name and `prefix`
#'   (default = "_z").
#' @param append Optional. A logical that determines whether the calculated
#'   standardized scores should be added as columns to `data` in the output or
#'   the standardized scores alone should be output (default = TRUE).
#' @return A data frame that contains the self-standardized versions of
#'   `scales`.
#' @export
#' @family tidying functions
#' @examples
#' self_standardize(aw2009, scales = 1:8)
self_standardize <- function(data, scales, na.rm = TRUE,
                             prefix = "", suffix = "_z", append = TRUE) {
  
  stopifnot(is.data.frame(data) || is.matrix(data))
  stopifnot(is_var(scales))
  stopifnot(is_flag(na.rm))
  stopifnot(is_char(prefix, n = 1))
  stopifnot(is_char(suffix, n = 1))
  stopifnot(is_flag(append))
  
  scale_data <- data[scales]
  scale_names <- colnames(scale_data)

  zscore <- function(x, na.rm = na.rm) {
    (x - mean(x, na.rm = na.rm)) / stats::sd(x, na.rm = na.rm)
  }
  
  scores <- sapply(scale_data, FUN = zscore, na.rm = na.rm)
  colnames(scores) <- paste0(prefix, scale_names, suffix)
  scores[is.nan(scores)] <- NA_real_
  
  if (append) {
    cbind(data, scores)
  } else {
    as.data.frame(scores) 
  }
}

test_that("ipsatize works", {
  data("raw_iipsc")
  items <- 1:32
  datin <- ipsatize(raw_iipsc, items = items, append = FALSE)
  datia <- ipsatize(raw_iipsc, items = items, append = TRUE)
  expect_equal(ncol(datin), length(items))
  expect_equal(ncol(datia), ncol(raw_iipsc) + length(items))
  expect_equal(datin[[1]][1], -1.0)
  expect_equal(datin[[2]][7], -0.5)
})


test_that("score works", {
  set.seed(12345)
  old <- data.frame(
    matrix(sample(0:4, size = 32 * 5, replace = TRUE), nrow = 5, ncol = 32)
  )
  new <- score(old, items = 1:32, instrument = iipsc)
  new2 <- score(old, items = 1:32, instrument = iipsc, append = FALSE)
    
  expect_equal(new$PA, c(0.5, 1.25, 2, 0.75, 3.5))
  expect_equal(new$BC, c(1.75, 0.75, 1.5, 1.75, 1.75))
  expect_equal(new$DE, c(1.75, 2.25, 1.75, 3, 3))
  expect_equal(new$FG, c(2.5, 1.75, 3.5, 1.5, 1.75))
  expect_equal(new$HI, c(2.25, 2.25, 3, 2, 1.25))
  expect_equal(new$JK, c(1.5, 2.5, 1.75, 2.25, 1.75))
  expect_equal(new$LM, c(2, 2.5, 1.5, 2.75, 1.5))
  expect_equal(new$NO, c(1.75, 2, 1.75, 2.25, 2.5))
  expect_error(score(old, 1:30, iipsc))
  expect_equal(ncol(new), ncol(new2) + ncol(old))
})


test_that("norm_standardize works", {
  set.seed(12345)
  old <- data.frame(
    matrix(runif(8 * 5, min = 0, max = 4), nrow = 5, ncol = 8)
  )
  new <- norm_standardize(
    old, 
    scales = 1:8, 
    instrument = iipsc, 
    sample = 1
  )
  new2 <- norm_standardize(
    old, 
    scales = 1:8,
    instrument = iipsc,
    sample = 1,
    append = FALSE
  )
  expect_equal(round(new$X1_z, 4), c(3.2176, 4.1562, 3.4605, 4.2189, 1.6150))
  expect_equal(round(new$X2_z, 4), c(-0.1841, 0.7361, 1.8035, 3.07, 4.5891))
  expect_equal(round(new$X3_z, 4), c(-0.8911, -0.3398, 2.3892, -1.0473, 0.7776))
  expect_equal(round(new$X4_z, 4), c(0.8469, 0.5331, 0.5936, -0.35, 2.912))
  expect_equal(round(new$X5_z, 4), c(0.4316, -0.1235, 2.6685, 1.5409, 1.2658))
  expect_equal(round(new$X6_z, 4), c(0.2045, 1.653, 0.9281, -0.562, 0.649))
  expect_equal(round(new$X7_z, 4), c(2.0691, -1.7467, -0.8656, 1.5301, 0.0187))
  expect_equal(round(new$X8_z, 4), c(0.5269, 3.0627, 3.2395, 1.8059, -0.6111))
  expect_error(norm_standardize(
    old, scales = 1:5, instrument = iipsc, sample = 1
  ))
  expect_equal(ncol(new), ncol(new2) + ncol(old))
})


test_that("self_standardize works", {
  old <- aw2009
  new <- self_standardize(
    old,
    scales = 1:8
  )
  new2 <- self_standardize(
    old,
    scales = 1:8,
    append = FALSE
  )
  expect_equal(round(new$PA_z, 4), c(-1.6857, 0.8705, 0.6172, 0.1105, 0.0875))
  expect_equal(round(new$BC_z, 4), c(-0.8356, -0.8356, -0.1393, 0.2178, 1.5927))
  expect_error(self_standardize(aw2009, 2:9))
  expect_equal(ncol(new), ncol(new2) + ncol(old))
})

# S3 vector constructors -------------------------------------------------------

# Create a new S3 class from a numeric
new_s3_num <- function(x, ..., class) {
  stopifnot(is.numeric(x))
  stopifnot(is.character(class))
  structure(x, ..., class = class)
}

# Create a new S3 class from a list
new_s3_lst <- function(x, ..., class) {
  stopifnot(is.list(x))
  stopifnot(is.character(class))
  structure(x, ..., class = class)
}

# Create a new S3 class from a scalar
new_s3_scalar <- function(..., class) {
  new_s3_lst(list(...), class = class)
}

# Class degree -----------------------------------------------------------------

# S3 Constructor
new_degree <- function(x) {
  new_s3_num(x, class = c("circumplex_degree", "numeric"))
}

# S3 Generic
as_degree <- function(x, ...) {
  UseMethod("as_degree")
}

# S3 Method
#' @method as_degree default
#' @export
as_degree.default <- function(x, ...) {
  new_degree(x)
}

# S3 Method
#' @method as_degree circumplex_degree
#' @export
as_degree.circumplex_degree <- function(x, ...) {
  x
}

# S3 Method
#' @method as_degree circumplex_radian
#' @export
as_degree.circumplex_radian <- function(x, ...) {
  new_degree(x * (180 / pi))
}

# Class radian -----------------------------------------------------------------

# S3 Constructor
new_radian <- function(x) {
  new_s3_num(x, class = c("circumplex_radian", "numeric"))
}

# S3 Generic
as_radian <- function(x, ...) {
  UseMethod("as_radian")
}

# S3 Method
#' @method as_radian default
#' @export
as_radian.default <- function(x, ...) {
  new_radian(x)
}

# S3 Method 
#' @method as_radian circumplex_radian
#' @export
as_radian.circumplex_radian <- function(x, ...) {
  x
}

# S3 Method
#' @method as_radian circumplex_degree
#' @export
as_radian.circumplex_degree <- function(x, ...) {
  new_radian(x * (pi / 180))
}

# S3 Method
#' @method print circumplex_degree
#' @export
print.circumplex_degree <- function(x, digits = 3, ...) {
  cat(round(x, digits = digits), "\nDegrees\n")
}

# S3 Method
#' @method print circumplex_radian
#' @export
print.circumplex_radian <- function(x, digits = 3, ...) {
  cat(round(x, digits = digits), "\nRadians\n")
}

# Class ssm --------------------------------------------------------------------

# S3 Constructor
new_ssm <- function(results, details, call, ...) {
  new_s3_scalar(
    results = results,
    details = details,
    call = call,
    ...,
    class = "circumplex_ssm"
  )
}

#  Print method for objects of ssm class
#' @method print circumplex_ssm
#' @export
print.circumplex_ssm <- function(x, digits = 3, ...) {
  # Print each result as a block
  for (i in 1:nrow(x$results)) {
    dat <- x$results[i, ]
    v <- c(
      dat$e_est, dat$x_est, dat$y_est, dat$a_est, dat$d_est, dat$fit_est,
      dat$e_lci, dat$x_lci, dat$y_lci, dat$a_lci, dat$d_lci, NA,
      dat$e_uci, dat$x_uci, dat$y_uci, dat$a_uci, dat$d_uci, NA
    )
    m <- round(matrix(v, nrow = 6, ncol = 3), digits)
    prefix <- ifelse(
      test = x$details$contrast && i == nrow(x$results),
      yes = "\u0394 ",
      no = ""
    )
    rownames(m) <- paste0(prefix, c(
      "Elevation", "X-Value", "Y-Value",
      "Amplitude", "Displacement", "Model Fit"
    ))
    colnames(m) <- c("Estimate", "Lower CI", "Upper CI")
    results_type <- ifelse(
      test = x$details$contrast && i == nrow(x$results), 
      yes = "Contrast", 
      no = "Profile"
    )
    cat("\n# ", results_type, " [", dat$Label, "]:\n\n",
      sep = ""
    )
    print.default(m, print.gap = 3L, na.print = "")
    cat("\n")
  }
}

# Summary method for objects of ssm class
#' @method summary circumplex_ssm
#' @export
summary.circumplex_ssm <- function(object, digits = 3, ...) {
  # Print analysis details
  cat(
    "\nStatistical Basis:\t", object$details$score_type, "Scores",
    "\nBootstrap Resamples:\t", object$details$boots,
    "\nConfidence Level:\t", object$details$interval,
    "\nListwise Deletion:\t", object$details$listwise,
    "\nScale Displacements:\t", as.numeric(object$details$angles),
    "\n\n"
  )
  print(object)
}

test_that("Quantile for circular radians works", {
  a <- as_degree(0:180)
  b <- as_radian(a)
  qb <- stats::quantile(b)
  expect_s3_class(qb, "circumplex_radian")
  expect_equal(qb, as_radian(as_degree(c(0, 45, 90, 135, 180))), 
               ignore_attr = TRUE)

  a <- as_degree(180:360)
  b <- as_radian(a)
  qb <- quantile(b)
  expect_s3_class(qb, "circumplex_radian")
  expect_equal(qb, as_radian(as_degree(c(180, 225, 270, 315, 0))),
               ignore_attr = TRUE)
  
  a <- as_degree(c(270:360, 1:90))
  b <- as_radian(a)
  qb <- quantile(b)
  expect_equal(qb, as_radian(as_degree(c(270, 315, 0, 45, 90))),
               ignore_attr = TRUE)
  
  a <- as_degree(c(NA_real_, NA_real_, NA_real_))
  b <- as_radian(a)
  qb <- stats::quantile(b)
  expect_true(is.na(qb))

  a <- as_degree(c(0, 0, 30, 90, NA_real_))
  b <- as_radian(a)
  c <- as_degree(c(0, 0, 30, 90))
  d <- as_radian(c)
  expect_equal(stats::quantile(b), stats::quantile(d))
})

test_that("S3 degree functions work as expected", {
  x <- c(0, 90, 180, 360)
  y <- c(0, pi / 2, pi, pi * 2)

  x1 <- as_degree(x)
  expect_s3_class(x1, "circumplex_degree")
  expect_equal(as.numeric(x1), x)

  x2 <- as_degree(as_degree(x))
  expect_s3_class(x2, "circumplex_degree")
  expect_equal(as.numeric(x2), x)

  x3 <- as_radian(as_degree(x))
  expect_s3_class(x3, "circumplex_radian")
  expect_equal(as.numeric(x3), y)

  y1 <- as_radian(y)
  expect_s3_class(y1, "circumplex_radian")
  expect_equal(as.numeric(y1), y)

  y2 <- as_radian(as_radian(y))
  expect_s3_class(y2, "circumplex_radian")
  expect_equal(as.numeric(y2), y)

  y3 <- as_degree(as_radian(y))
  expect_s3_class(y3, "circumplex_degree")
  expect_equal(as.numeric(y3), x)
})

test_that("The ssm display methods is working", {
  skip_on_cran()

  data("aw2009")
  res <- ssm_analyze(aw2009, scales = 1:8)
  expect_output(print(res), "# Profile \\[All\\]:")
  expect_output(summary(res), "Statistical Basis:\\t Mean Scores")
  expect_output(summary(res), "Bootstrap Resamples:\\t 2000")
  expect_output(summary(res), "Confidence Level:\\t 0\\.95")
  expect_output(summary(res), "Listwise Deletion:\\t TRUE")
  expect_output(summary(res), "Scale Displacements:\\t 90 135 180 225 270 315 360 45")

  data("jz2017")
  res <- ssm_analyze(jz2017, scales = 2:9, grouping = "Gender")
  expect_output(print(res), "# Profile \\[Female\\]:")
  expect_output(print(res), "# Profile \\[Male\\]:")

  res <- ssm_analyze(
    jz2017, 
    scales = 2:9,
    grouping = "Gender",
    contrast = TRUE
  )
  expect_output(print(res), "# Profile \\[Female\\]:")
  expect_output(print(res), "# Profile \\[Male\\]:")
  expect_output(print(res), "# Contrast \\[Male - Female\\]:")
  expect_output(print(res), "\u0394 Elevation")

  res <- ssm_analyze(
    jz2017, 
    scales = 2:9,
    measures = "PARPD",
    grouping = "Gender", 
    contrast = TRUE
  )
  expect_output(print(res), "# Contrast \\[PARPD: Male - Female\\]:")
  expect_output(summary(res), "Statistical Basis:\\t Correlation Scores")
})

test_that("unit classes are working", {
  expect_snapshot(octants())
  expect_snapshot(as_radian(octants()))
})

#' Create HTML table from SSM results or contrasts
#'
#' Take in the results of an SSM analysis and return an HTML table with the
#' desired formatting.
#'
#' @param ssm_object Required. The results output of `ssm_analyze()`.
#' @param caption A string to be displayed above the table (default = NULL).
#' @param drop_xy A logical indicating whether the x-value and y-value parameters
#'   should be omitted from the output (default = FALSE).
#' @param render A logical indicating whether the table should be displayed in
#'   the RStudio viewer or web browser (default = TRUE).
#' @return A data frame containing the information for the HTML table. As a
#'  side-effect, may also output the HTML table to the web viewer.
#' @family ssm functions
#' @family table functions
#' @export
#' @examples
#' \donttest{
#' # Load example data
#' data("jz2017")
#' 
#' # Create table of profile results
#' res <- ssm_analyze(
#'   jz2017,
#'   scales = 2:9,
#'   measures = c("NARPD", "ASPD")
#' )
#' ssm_table(res)
#' 
#' # Create table of contrast results
#' res <- ssm_analyze(
#'   jz2017,
#'   scales = 2:9,
#'   measures = c("NARPD", "ASPD"), 
#'   contrast = TRUE
#' )
#' ssm_table(res)
#' }
#' 
ssm_table <- function(ssm_object, caption = NULL, 
                      drop_xy = FALSE, render = TRUE) {
  
  stopifnot(class(ssm_object) == "circumplex_ssm")
  stopifnot(is_null_or_char(caption, n = 1))
  stopifnot(is_flag(drop_xy))
  stopifnot(is_flag(render))
  
  df <- ssm_object$results
  
  # Create default caption
  if (is.null(caption)) {
    caption <- dcaption(ssm_object)
  }
  
  # Format output data
  table_df <- 
    data.frame(
      Profile = df$Label,
      Elevation = sprintf("%.2f (%.2f, %.2f)", df$e_est, df$e_lci, df$e_uci),
      `X Value` = sprintf("%.2f (%.2f, %.2f)", df$x_est, df$x_lci, df$x_uci),
      `Y Value` = sprintf("%.2f (%.2f, %.2f)", df$y_est, df$y_lci, df$y_uci),
      Amplitude = sprintf("%.2f (%.2f, %.2f)", df$a_est, df$a_lci, df$a_uci),
      Displacement = sprintf("%.1f (%.1f, %.1f)", df$d_est, df$d_lci, df$d_uci),
      Fit = sprintf("%.3f", df$fit_est)
    )
  
  # Rename first column
  colnames(table_df)[[1]] <- ifelse(
    test = ssm_object$details$contrast, 
    yes = "Contrast", 
    no = "Profile"
  )
  
  # Drop the x and y columns if requested
  if (drop_xy) {
    table_df <- table_df[, -c(3, 4)]
  }
  
  # Format and render HTML table if requested
  if (render) {
    html_render(table_df, caption)
  }
  
  table_df
}

# Build the default caption for the ssm_table function
dcaption <- function(ssm_object) {
  if (ssm_object$details$contrast) {
    sprintf(
      "%s-based Structural Summary Statistic Contrasts with %s CIs",
      ssm_object$details$score_type,
      str_percent(ssm_object$details$interval)
    )
  } else {
    sprintf(
      "%s-based Structural Summary Statistics with %s CIs",
      ssm_object$details$score_type,
      str_percent(ssm_object$details$interval)
    )
  }
}

#' Format and render data frame as HTML table
#'
#' Format a data frame as an HTML table and render it to the web viewer.
#'
#' @param df A data frame to be rendered as an HTML table.
#' @param caption A string to be displayed above the table.
#' @param align A string indicating the alignment of the cells (default = "l").
#' @param ... Other arguments to pass to \code{htmlTable}.
#' @return HTML syntax for the \code{df} table.
#' @family table functions
#' @export
html_render <- function(df, caption = NULL, align = "l", ...) {
  
  stopifnot(is_null_or_char(caption, n = 1))
  stopifnot(align %in% c("l", "c", "r"))
  
  t <- htmlTable::htmlTable(
    df,
    caption = caption,
    align = align,
    align.header = align,
    rnames = FALSE,
    css.cell = "padding-right: 1em; min-width: 3em; white-space: nowrap;",
    ...
  )
  print(t, type = "html")
}

test_that("SSM Table captions are correct", {
  data("jz2017")
  res <- ssm_analyze(jz2017, scales = 2:9)
  expect_equal(
    dcaption(res),
    "Mean-based Structural Summary Statistics with 95% CIs"
  )
  
  res <- ssm_analyze(
    jz2017,
    scales = 2:9,
    grouping = "Gender",
    contrast = TRUE
  )
  expect_equal(
    dcaption(res),
    "Mean-based Structural Summary Statistic Contrasts with 95% CIs"
  )
  
  res <- ssm_analyze(
    jz2017, 
    scales = 2:9, 
    measures = "PARPD"
  )
  expect_equal(
    dcaption(res),
    "Correlation-based Structural Summary Statistics with 95% CIs"
  )
  
  res <- ssm_analyze(
    jz2017,
    scales = 2:9,
    measures = "PARPD",
    grouping = "Gender", 
    contrast = TRUE
  )
  expect_equal(
    dcaption(res),
    "Correlation-based Structural Summary Statistic Contrasts with 95% CIs"
  )
})


# test_that("ssm_table is correct", {
#   set.seed(12345)
#   data("jz2017")
#   res <- ssm_analyze(jz2017, scales = 2:9)
#   expect_snapshot(ssm_table(res, render = FALSE))
#   expect_snapshot(ssm_table(res, drop_xy = TRUE, render = FALSE))
#   expect_snapshot(ssm_table(res, render = TRUE))
# })

#' Perform analyses using the Structural Summary Method
#'
#' Calculate SSM parameters with bootstrapped confidence intervals for a variety
#' of different analysis types. Depending on what arguments are supplied, either
#' mean-based or correlation-based analyses will be performed, one or more
#' groups will be used to stratify the data, and contrasts between groups or
#' measures will be calculated.
#'
#' @param data Required. A data frame containing at least circumplex scales.
#' @param scales Required. A character vector of column names, or a numeric
#'   vector of column indexes, from `data` that contains the circumplex scale
#'   scores to be analyzed.
#' @param angles Optional. A numeric vector containing the angular displacement
#'   of each circumplex scale included in `scales` (in degrees). (default =
#'   `octants()`).
#' @param measures Optional. Either `NULL` or a character vector of column names
#'   from `data` that contains one or more variables to be correlated with the
#'   circumplex scales and analyzed using correlation-based SSM analyses.
#' @param grouping Optional. Either `NULL` or a string that contains the column
#'   name from `data` of the variable that indicates the group membership of
#'   each observation.
#' @param contrast Optional. A logical indicating whether to output the
#'   difference between two measures' or two groups' SSM parameters. Can only be
#'   set to TRUE when there are exactly two measures and one group, one measure
#'   and two groups, or no measures and two groups (default = FALSE).
#' @param boots Optional. A single positive whole number indicating how many
#'   bootstrap resamples to use when estimating the confidence intervals
#'   (default = 2000).
#' @param interval Optional. A single positive number between 0 and 1
#'   (exclusive) that indicates what confidence level to use when estimating the
#'   confidence intervals (default = 0.95).
#' @param listwise Optional. A logical indicating whether missing values should
#'   be handled by listwise deletion (TRUE) or pairwise deletion (FALSE). Note
#'   that pairwise deletion may result in different missing data patterns in
#'   each bootstrap resample and is slower to compute (default = TRUE).
#' @param measures_labels Optional. Either `NULL` or a character vector
#'   providing a label for each measure provided in `measures` (in the same
#'   order) to appear in the results as well as tables and plots derived from
#'   the results.
#' @return A list containing the results and description of the analysis.
#'   \item{results}{A data frame with the SSM parameter estimates}
#'   \item{details}{A list with the number of bootstrap resamples (boots),
#'   the confidence interval percentage level (interval), and the angular
#'   displacement of scales (angles)}
#'   \item{call}{A language object containing the function call that created
#'   this object}
#'   \item{scores}{A data frame containing the mean scale scores} \item{type}{A
#'   string indicating what type of SSM analysis was done}
#' @family ssm functions
#' @family analysis functions
#' @export
#' @examples
#' # Load example data
#' data("jz2017")
#'
#' # Single-group mean-based SSM
#' ssm_analyze(
#'   jz2017,
#'   scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO")
#' )
#'
#' # Single-group correlation-based SSM
#' ssm_analyze(
#'   jz2017,
#'   scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
#'   measures = c("NARPD", "ASPD")
#' )
#' \donttest{
#' # Multiple-group mean-based SSM
#' ssm_analyze(
#'   jz2017,
#'   scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
#'   grouping = "Gender"
#' )
#'
#' # Multiple-group mean-based SSM with contrast
#' ssm_analyze(
#'   jz2017,
#'   scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
#'   grouping = "Gender",
#'   contrast = TRUE
#' )
#'
#' # Single-group correlation-based SSM with contrast
#' ssm_analyze(
#'   jz2017,
#'   scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
#'   measures = c("NARPD", "ASPD"),
#'   contrast = TRUE
#' )
#'
#' # Multiple-group correlation-based SSM
#' ssm_analyze(
#'   jz2017,
#'   scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
#'   measures = "NARPD",
#'   grouping = "Gender"
#' )
#'
#' # Multiple-group correlation-based SSM with contrast
#' ssm_analyze(
#'   jz2017,
#'   scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
#'   measures = "NARPD",
#'   grouping = "Gender",
#'   contrast = TRUE
#' )
#' }
#' 
ssm_analyze <- function(data, scales, angles = octants(), 
                        measures = NULL, grouping = NULL, contrast = FALSE, 
                        boots = 2000, interval = 0.95, listwise = TRUE,
                        measures_labels = NULL) {
  
  # Save function call
  call <- match.call()
  
  # Validate arguments
  stopifnot(is.data.frame(data) || is.matrix(data))
  stopifnot(is_var(scales))
  stopifnot(is.numeric(angles))
  stopifnot(length(scales) == length(angles))
  stopifnot(is_null_or_var(measures))
  stopifnot(is_null_or_var(grouping, n = 1))
  stopifnot(is_flag(contrast))
  stopifnot(is.numeric(boots) && boots > 0 && ceiling(boots) == floor(boots))
  stopifnot(is.numeric(interval) && interval > 0 && interval < 1)
  stopifnot(is_flag(listwise))
  stopifnot(is_null_or_char(measures_labels, n = length(measures)))

  if (contrast) {
    n_measures <- length(measures)
    n_groups <- ifelse(is.null(grouping), 1, nlevels(factor(data[[grouping]])))
    group_mean_contrast <- n_measures == 0 && n_groups == 2
    group_corr_contrast <- n_measures == 1 && n_groups == 2
    measure_corr_contrast <- n_measures == 2 && n_groups == 1
    if (!any(group_mean_contrast, group_corr_contrast, measure_corr_contrast)) {
      stop("Contrast can only be TRUE when comparing 2 groups or 2 measures.")
    }
  }
  
  # Convert angles from degrees to radians
  angles <- as_radian(as_degree(angles))

  # Forward to the appropriate subfunction
  if (is.null(measures)) {
    # No Measures = Mean Analysis
    ssm_analyze_means(
      data = data,
      scales = scales,
      angles = angles,
      grouping = grouping,
      contrast = contrast,
      boots = boots,
      interval = interval,
      listwise = listwise,
      call = call
    )
  } else {
    # Measures = Correlation Analysis
    ssm_analyze_corrs(
      data = data,
      scales = scales,
      angles = angles,
      measures = measures,
      grouping = grouping,
      contrast = contrast,
      boots = boots,
      interval = interval,
      listwise = listwise,
      measures_labels = measures_labels,
      call = call
    )
  }
}

# Perform analyses using the mean-based Structural Summary Method --------------

ssm_analyze_means <- function(data, scales, angles, grouping, contrast, 
                              boots, interval, listwise, call) {
  
  # Select circumplex scales and grouping variable (if applicable)
  bs_input <- data[scales]
  scales_names <- colnames(bs_input)
  if (is.null(grouping)) {
    bs_input <- cbind(bs_input, Group = rep("All", times = nrow(data)))
  } else {
    Group <- data[grouping]
    colnames(Group) <- "Group"
    bs_input <- cbind(bs_input, Group)
  }
  
  # Perform listwise deletion if requested
  if (listwise) {
    bs_input <- stats::na.omit(bs_input)
  }
  
  # Set group to factor
  bs_input[[ncol(bs_input)]] <- factor(bs_input[[ncol(bs_input)]])
  
  # Get counts
  n_scales <- length(scales)
  n_groups <- nlevels(bs_input[[ncol(bs_input)]])
  group_levels <- levels(bs_input[[ncol(bs_input)]])
  
  # Calculate mean observed scores
  mat <- as.matrix(bs_input[scales_names])
  grp <- as.integer(bs_input[[ncol(bs_input)]])
  scores <- mean_scores(mat, grp, listwise)
  colnames(scores) <- scales_names
  group_levels <- levels(bs_input[[ncol(bs_input)]])
  if (contrast) {
    scores <- rbind(scores, scores[2, ] - scores[1, ])
  }
  scores <- as.data.frame(scores)
  Group <- group_levels
  Measure <- rep(NA_character_, times = n_groups)
  if (contrast && !is.null(grouping)) {
    Group <- c(Group, paste0(Group[[2]], " - ", Group[[1]]))
    Measure <- c(Measure, Measure[[1]])
  }
  Label <- Group
  scores <- cbind(Label, Group, Measure, scores)

  # Create function that will perform bootstrapping
  bs_function <- function(.data, index, scales, angles, contrast, listwise, ...) {
    resample <- .data[index, ]
    mat <- as.matrix(resample[scales])
    grp <- as.integer(resample[[ncol(resample)]])
    scores_r <- mean_scores(mat, grp, listwise)
    ssm_by_group(scores_r, angles, contrast)
  }
  
  # Perform bootstrapping
  bs_output <- ssm_bootstrap(
    bs_input = bs_input,
    bs_function = bs_function,
    scales = scales_names,
    angles = angles,
    boots = boots,
    interval = interval,
    contrast = contrast,
    listwise = listwise,
    strata = bs_input[[ncol(bs_input)]]
  )
  
  params <- bs_output
  Group <- group_levels
  Measure <- rep(NA_character_, times = n_groups)
  if (contrast && !is.null(grouping)) {
    Group <- c(Group, paste0(Group[[2]], " - ", Group[[1]]))
    Measure <- c(Measure, Measure[[1]])
  }
  Label <- Group
  results <- cbind(Label, Group, Measure, params)
  
  # Collect analysis details
  details <- list(
    boots = boots,
    interval = interval,
    listwise = listwise,
    angles = as_degree(angles),
    contrast = contrast,
    score_type = "Mean"
  )
  
  # Create output ssm object
  out <- new_ssm(
    results = results,
    scores = scores,
    call = call,
    details = details
  )
  
  out
}

# Perform analyses using the correlation-based SSM -----------------------------

ssm_analyze_corrs <- function(data, scales, angles, measures, grouping,
                              contrast, boots, interval, listwise, 
                              measures_labels, call) {
  
  # Select only the scales, measures, and grouping variables
  scales_data <- data[scales]
  scales_names <- colnames(scales_data)
  measures_data <- data[measures]
  measures_names <- colnames(measures_data)
  bs_input <- cbind(scales_data, measures_data)
  if (is.null(grouping)) {
    newcol <- data.frame(Group = rep("All", nrow(data)))
    bs_input <- cbind(bs_input, newcol)
  } else {
    newcol <- data[grouping]
    colnames(newcol) <- "Group"
    bs_input <- cbind(bs_input, newcol)
  }
  
  # Perform listwise deletion if requested
  if (listwise == TRUE) {
    bs_input <- stats::na.omit(bs_input)
  }
  
  # Set group as factor
  bs_input[[ncol(bs_input)]] <- factor(bs_input[[ncol(bs_input)]])
  
  # Get counts
  n_scales <- length(scales)
  n_measures <- length(measures)
  n_groups <- nlevels(bs_input$Group)
  
  # Get names of measures (using labels if provided)
  if (is.null(measures_labels)) {
    measures_labels <- measures_names
  }
  
  # Calculate observed correlation scores
  cs <- as.matrix(bs_input[scales_names])
  mv <- as.matrix(bs_input[measures_names])
  grp <- as.integer(bs_input[[ncol(bs_input)]])
  scores <- corr_scores(cs, mv, grp, listwise)
  colnames(scores) <- scales_names
  group_levels <- levels(bs_input[[ncol(bs_input)]])
  if (contrast) {
    scores <- rbind(scores, scores[2, ] - scores[1, ])
  }
  scores <- as.data.frame(scores)
  Group <- rep(group_levels, each = n_measures)
  Measure <- rep(measures_labels, times = n_groups)
  if (contrast && is.null(grouping)) {
    Group <- c(Group, Group[[1]])
    Measure <- c(Measure, paste0(Measure[[2]], " - ", Measure[[1]]))
  } else if (contrast && !is.null(grouping)) {
    Group <- c(Group, paste0(Group[[2]], " - ", Group[[1]]))
    Measure <- c(Measure, Measure[[1]])
  }
  if (is.null(grouping)) {
    Label <- Measure
  } else {
    Label <- paste0(Measure, ": ", Group)
  }
  scores <- cbind(Label, Group, Measure, scores)


  # Create function that will perform bootstrapping
  bs_function <- function(.data, index, scales, measures, angles, contrast, 
                          listwise, ...) {
    resample <- .data[index, ]
    cs <- as.matrix(resample[scales])
    mv <- as.matrix(resample[measures])
    grp <- as.integer(resample[[ncol(resample)]])
    scores_r <- corr_scores(cs, mv, grp, listwise)
    ssm_by_group(scores_r, angles, contrast)
  }
  
  # Perform bootstrapping
  bs_output <- ssm_bootstrap(
    bs_input = bs_input,
    bs_function = bs_function,
    scales = scales_names,
    measures = measures_names,
    angles = angles,
    boots = boots,
    interval = interval,
    contrast = contrast,
    listwise = listwise,
    strata = bs_input$Group
  )
  
  Group <- rep(group_levels, each = n_measures)
  Measure <- rep(measures_labels, times = n_groups)
  if (contrast && is.null(grouping)) {
    Group <- c(Group, Group[[1]])
    Measure <- c(Measure, paste0(Measure[[2]], " - ", Measure[[1]]))
  } else if (contrast && !is.null(grouping)) {
    Group <- c(Group, paste0(Group[[2]], " - ", Group[[1]]))
    Measure <- c(Measure, Measure[[1]])
  }
  if (is.null(grouping)) {
    Label <- Measure
  } else {
    Label <- paste0(Measure, ": ", Group)
  }
  results <- cbind(Label, Group, Measure, bs_output)


  # Collect analysis details
  details <- list(
    boots = boots,
    interval = interval,
    listwise = listwise,
    angles = as_degree(angles),
    contrast = contrast,
    score_type = "Correlation"
  )
  
  # Create output ssm object
  out <- new_ssm(
    results = results,
    scores = scores,
    call = call,
    details = details
  )
  
  out
}

#' Calculate Structural Summary Method parameters for a set of scores
#'
#' Calculate SSM parameters (without confidence intervals) for a set of scores
#' and generate a data frame with customizable labels for each parameter value.
#' This function requires the input to be a numeric vector (or coercable to one)
#' and returns only the parameters. See \code{\link{ssm_score}()} for a similar
#' function that calculates SSM parameters for each row of a data frame.
#'
#' @param scores Required. A numeric vector (or single row data frame)
#'   containing one score for each of a set of circumplex scales.
#' @param angles Required. A numeric vector containing the angular displacement
#'   of each circumplex scale included in `scores` (in degrees).
#' @param prefix Optional. A string to append to the beginning of all of the SSM
#'   parameters' variable names (default = "").
#' @param suffix Optional. A string to append to the end of all of the SSM
#'   parameters' variable names (default = "").
#' @param e_label Optional. A string representing the variable name of the SSM
#'   elevation parameter (default = "Elev").
#' @param x_label Optional. A string representing the variable name of the SSM
#'   x-value parameter (default = "Xval").
#' @param y_label Optional. A string representing the variable name of the SSM
#'   y-value parameter (default = "Yval").
#' @param a_label Optional. A string representing the variable name of the SSM
#'   amplitude parameter (default = "Ampl").
#' @param d_label Optional. A string representing the variable name of the SSM
#'   displacement parameter (default = "Disp").
#' @param f_label Optional. A string representing the variable name of the SSM
#'   fit or R-squared value (default = "Fit").
#' @return A data frame containing the SSM parameters calculated from `scores`.
#' @family ssm functions
#' @family analysis functions
#' @export
#' @examples
#' # Manually enter octant scores
#' scores <- c(0.55, 0.58, 0.62, 0.76, 1.21, 1.21, 1.48, 0.90)
#' ssm_parameters(scores)
#'
#' # Customize several of the labels
#' ssm_parameters(scores, x_label = "LOV", y_label = "DOM")
#'
#' # Add a prefix to all labels
#' ssm_parameters(scores, prefix = "IIP_")
#' 
ssm_parameters <- function(scores, angles = octants(), prefix = "", suffix = "", 
                           e_label = "Elev", x_label = "Xval", y_label = "Yval",
                           a_label = "Ampl", d_label = "Disp", f_label = "Fit") {

  stopifnot(is.numeric(scores))
  stopifnot(is.numeric(angles))
  stopifnot(length(scores) == length(angles))
  stopifnot(is_char(prefix, n = 1))
  stopifnot(is_char(suffix, n = 1))
  stopifnot(is_char(e_label, n = 1))
  stopifnot(is_char(x_label, n = 1))
  stopifnot(is_char(y_label, n = 1))
  stopifnot(is_char(a_label, n = 1))
  stopifnot(is_char(d_label, n = 1))
  stopifnot(is_char(f_label, n = 1))

  angles <- as_radian(as_degree(angles))
  params <- ssm_parameters_cpp(scores, angles)
  params[[5]] <- as_degree(as_radian(params[[5]]))
  
  rownames(params) <- paste0(
    prefix, 
    c(e_label, x_label, y_label, a_label, d_label, f_label), 
    suffix
  )
  
  as.data.frame(t(params))
}

#' Calculate SSM parameters by row and add results as new columns
#'
#' Calculate the SSM parameters for each row of a data frame and add the results
#' as additional columns. This can be useful when the SSM is being used for the
#' description or visualization of individual data points rather than for
#' statistical inference on groups of data points.
#'
#' @param data Required. A data frame containing at least circumplex scales.
#' @param scales Required. The variable names or column numbers for the
#'   variables in \code{.data} that contain circumplex scales to be analyzed.
#' @param angles Required. A numeric vector containing the angular displacement
#'   of each circumplex scale included in \code{scales} (in degrees).
#' @param append Optional. A logical indicating whether to append the output to
#'   `data` or simply return the output (default = "TRUE").
#' @param ... Optional. Additional parameters to pass to
#'   \code{\link{ssm_parameters}()}, such as \code{prefix} and \code{suffix}.
#' @return A data frame containing \code{.data} plus six additional columns
#'   containing the SSM parameters (calculated rowwise).
#' @family ssm functions
#' @family analysis functions
#' @export
#' @examples
#' data("aw2009")
#' ssm_score(
#'   aw2009,
#'   scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO")
#' )
#' 
ssm_score <- function(data, scales, angles = octants(), append = TRUE, ...) {

  stopifnot(is.data.frame(data) || is.matrix(data))
  stopifnot(is.character(scales))
  stopifnot(is.numeric(angles))
  stopifnot(length(scales) == length(angles))

  scales_mat <- as.matrix(data[scales])
  
  out <- do.call(
    rbind, 
    apply(
      scales_mat,
      MARGIN = 1, 
      FUN = ssm_parameters, 
      ...
    )
  )
  
  if (append) {
    out <- cbind(data, out)
  }
  
  out
}

test_that("Single-group mean-based SSM results are correct", {
  skip_on_cran()
  
  data("aw2009")
  set.seed(12345)
  res <- ssm_analyze(aw2009, scales = 1:8)

  # Test the output object
  expect_type(res, "list")
  expect_s3_class(res, "circumplex_ssm")

  # Test the results subobject
  expect_equal(round(res$results$e_est, 3), 0.423)
  expect_equal(round(res$results$x_est, 3), 0.945)
  expect_equal(round(res$results$y_est, 3), -0.264)
  expect_equal(round(res$results$a_est, 3), 0.981)
  expect_equal(round(res$results$d_est, 1), as_degree(344.4))
  expect_equal(round(res$results$fit_est, 3), 0.954)
  expect_equal(res$results$Label, "All")
  expect_equal(round(res$results$e_lci, 3), 0.129)
  expect_equal(round(res$results$e_uci, 3), 0.708)
  expect_equal(round(res$results$x_lci, 3), 0.654)
  expect_equal(round(res$results$x_uci, 3), 1.251)
  expect_equal(round(res$results$y_lci, 3), -0.946)
  expect_equal(round(res$results$y_uci, 3), 0.300)
  expect_equal(round(res$results$a_lci, 3), 0.662)
  expect_equal(round(res$results$a_uci, 3), 1.403)
  expect_equal(round(res$results$d_lci, 3), as_degree(316.480))
  expect_equal(round(res$results$d_uci, 3), as_degree(17.191))

  # Test the scores subobject
  expect_equal(round(res$scores$PA, 3), 0.374)
  expect_equal(round(res$scores$BC, 3), -0.572)
  expect_equal(round(res$scores$DE, 3), -0.520)
  expect_equal(round(res$scores$FG, 3), 0.016)
  expect_equal(round(res$scores$HI, 3), 0.688)
  expect_equal(round(res$scores$JK, 3), 1.142)
  expect_equal(round(res$scores$LM, 3), 1.578)
  expect_equal(round(res$scores$NO, 3), 0.678)
  expect_equal(res$scores$Label, "All")

  # Test the details subobject
  expect_equal(res$details$boots, 2000)
  expect_equal(res$details$interval, 0.95)
  expect_true(res$details$listwise)
  expect_false(res$details$contrast)
  expect_equal(res$details$angles, octants(), ignore_attr = TRUE)
  expect_equal(res$details$score_type, "Mean")
})

test_that("Multiple-group mean-based SSM results are correct", {
  skip_on_cran()

  data("jz2017")
  set.seed(12345)
  res <- ssm_analyze(jz2017, scales = 2:9, grouping = "Gender")

  # Test the output object
  expect_type(res, "list")
  expect_s3_class(res, "circumplex_ssm")

  # Test the results subobject
  expect_equal(round(res$results$e_est, 3), c(0.946, 0.884))
  expect_equal(round(res$results$x_est, 3), c(0.459, 0.227))
  expect_equal(round(res$results$y_est, 3), c(-0.310, -0.186))
  expect_equal(round(res$results$a_est, 3), c(0.554, 0.294))
  expect_equal(round(res$results$d_est, 3), as_degree(c(325.963, 320.685)))
  expect_equal(round(res$results$fit_est, 3), c(0.889, 0.824))
  expect_equal(res$results$Label, c("Female", "Male"))
  expect_equal(round(res$results$e_lci, 3), c(0.907, 0.839))
  expect_equal(round(res$results$e_uci, 3), c(0.984, 0.928))
  expect_equal(round(res$results$x_lci, 3), c(0.422, 0.191))
  expect_equal(round(res$results$x_uci, 3), c(0.498, 0.262))
  expect_equal(round(res$results$y_lci, 3), c(-0.357, -0.225))
  expect_equal(round(res$results$y_uci, 3), c(-0.266, -0.147))
  expect_equal(round(res$results$a_lci, 3), c(0.511, 0.256))
  expect_equal(round(res$results$a_uci, 3), c(0.600, 0.330))
  expect_equal(round(res$results$d_lci, 3), as_degree(c(321.834, 313.386)))
  expect_equal(round(res$results$d_uci, 3), as_degree(c(329.805, 327.985)))

  # Test the scores subobject
  expect_equal(round(res$scores$PA, 3), c(0.519, 0.585))
  expect_equal(round(res$scores$BC, 3), c(0.504, 0.674))
  expect_equal(round(res$scores$DE, 3), c(0.589, 0.664))
  expect_equal(round(res$scores$FG, 3), c(0.685, 0.856))
  expect_equal(round(res$scores$HI, 3), c(1.330, 1.075))
  expect_equal(round(res$scores$JK, 3), c(1.361, 1.047))
  expect_equal(round(res$scores$LM, 3), c(1.645, 1.300))
  expect_equal(round(res$scores$NO, 3), c(0.933, 0.868))
  expect_equal(res$scores$Label, c("Female", "Male"))

  # Test the details subobject
  expect_equal(res$details$boots, 2000)
  expect_equal(res$details$interval, 0.95)
  expect_true(res$details$listwise)
  expect_equal(res$details$angles, octants(), ignore_attr = TRUE)
  expect_false(res$details$contrast)
  expect_equal(res$details$score_type, "Mean")
})

test_that("Multiple-group mean-based SSM contrast is correct", {
  skip_on_cran()

  data("jz2017")
  set.seed(12345)
  res <- ssm_analyze(
    jz2017, 
    scales = 2:9,
    grouping = "Gender",
    contrast = TRUE
  )

  # Test the output object
  expect_type(res, "list")
  expect_s3_class(res, "circumplex_ssm")

  # Test the results subobject
  expect_equal(round(res$results$e_est, 3), c(0.946, 0.884, -0.062))
  expect_equal(round(res$results$x_est, 3), c(0.459, 0.227, -0.232))
  expect_equal(round(res$results$y_est, 3), c(-0.310, -0.186, 0.124))
  expect_equal(round(res$results$a_est, 3), c(0.554, 0.294, -0.261))
  expect_equal(round(res$results$d_est, 3), as_degree(c(325.963, 320.685, -5.278)))
  expect_equal(round(res$results$fit_est, 3), c(0.889, 0.824, -0.066))
  expect_equal(res$results$Label, c("Female", "Male", "Male - Female"))
  expect_equal(round(res$results$e_lci, 3), c(0.907, 0.839, -0.122))
  expect_equal(round(res$results$e_uci, 3), c(0.984, 0.928, -0.002))
  expect_equal(round(res$results$x_lci, 3), c(0.422, 0.191, -0.285))
  expect_equal(round(res$results$x_uci, 3), c(0.498, 0.262, -0.180))
  expect_equal(round(res$results$y_lci, 3), c(-0.357, -0.225, 0.067))
  expect_equal(round(res$results$y_uci, 3), c(-0.266, -0.147, 0.183))
  expect_equal(round(res$results$a_lci, 3), c(0.511, 0.256, -0.318))
  expect_equal(round(res$results$a_uci, 3), c(0.600, 0.330, -0.205))
  expect_equal(round(res$results$d_lci, 3), as_degree(c(321.834, 313.386, -13.521)))
  expect_equal(round(res$results$d_uci, 3), as_degree(c(329.805, 327.985, 3.029)))

  # Test the details subobject
  expect_equal(res$details$boots, 2000)
  expect_equal(res$details$interval, 0.95)
  expect_true(res$details$listwise)
  expect_equal(res$details$angles, octants(), ignore_attr = TRUE)
  expect_true(res$details$contrast)
  expect_equal(res$details$score_type, "Mean")
})

test_that("Requesting impossible contrasts throws error", {
  data("jz2017")
  jz2017$Three <- sample(c("a", "b", "c"), size = nrow(jz2017), replace = TRUE)
  # One group and no measures
  expect_error(ssm_analyze(jz2017, scales = 2:9, contrast = TRUE))
  # Three groups and no measures
  expect_error(ssm_analyze(
    jz2017, 
    scales = 2:9, 
    grouping = "Three",
    contrast = TRUE
  ))
  # Two groups but two measures
  expect_error(ssm_analyze(
    jz2017,
    scales = 2:9,
    measures = c("PARPD", "NARPD"),
    grouping = "Gender",
    contrast = TRUE
  ))
  # One group and one measure
  expect_error(ssm_analyze(
    jz2017, 
    scales = 2:9, 
    measures = "PARPD", 
    contrast = TRUE
  ))
  
})

test_that("Single-group correlation-based SSM results are correct", {
  skip_on_cran()
  
  data("jz2017")
  set.seed(12345)
  res <- ssm_analyze(jz2017, scales = 2:9, measures = "PARPD")

  # Test the output object
  expect_type(res, "list")
  expect_s3_class(res, "circumplex_ssm")

  # Test the results subobject
  expect_equal(round(res$results$e_est, 3), 0.250)
  expect_equal(round(res$results$x_est, 3), -0.094)
  expect_equal(round(res$results$y_est, 3), 0.117)
  expect_equal(round(res$results$a_est, 3), 0.150)
  expect_equal(round(res$results$d_est, 1), as_degree(128.9))
  expect_equal(round(res$results$fit_est, 3), 0.802)
  expect_equal(res$scores$Group, "All")
  expect_equal(res$scores$Measure, "PARPD")
  expect_equal(res$scores$Label, "PARPD")
  expect_equal(round(res$results$e_lci, 3), 0.218)
  expect_equal(round(res$results$e_uci, 3), 0.282)
  expect_equal(round(res$results$x_lci, 3), -0.128)
  expect_equal(round(res$results$x_uci, 3), -0.062)
  expect_equal(round(res$results$y_lci, 3), 0.081)
  expect_equal(round(res$results$y_uci, 3), 0.153)
  expect_equal(round(res$results$a_lci, 3), 0.113)
  expect_equal(round(res$results$a_uci, 3), 0.189)
  expect_equal(round(res$results$d_lci, 3), as_degree(117.261))
  expect_equal(round(res$results$d_uci, 3), as_degree(141.596))

  # Test the scores subobject
  expect_equal(round(res$scores$PA, 3), 0.329)
  expect_equal(round(res$scores$BC, 3), 0.494)
  expect_equal(round(res$scores$DE, 3), 0.329)
  expect_equal(round(res$scores$FG, 3), 0.203)
  expect_equal(round(res$scores$HI, 3), 0.102)
  expect_equal(round(res$scores$JK, 3), 0.143)
  expect_equal(round(res$scores$LM, 3), 0.207)
  expect_equal(round(res$scores$NO, 3), 0.193)
  expect_equal(res$scores$Group, "All")
  expect_equal(res$scores$Measure, "PARPD")
  expect_equal(res$scores$Label, "PARPD")

  # Test the details subobject
  expect_equal(res$details$boots, 2000)
  expect_equal(res$details$interval, 0.95)
  expect_true(res$details$listwise)
  expect_equal(res$details$angles, octants())
  expect_false(res$details$contrast)
  expect_match(res$details$score_type, "Correlation")
})

test_that("Pairwise and listwise scores are the same with no missingness", {
  skip_on_cran()

  # Single-group mean
  data("jz2017")
  res_lw <- ssm_analyze(jz2017, scales = 2:9, listwise = TRUE)
  res_pw <- ssm_analyze(jz2017, scales = 2:9, listwise = FALSE)
  expect_equal(res_lw$scores, res_pw$scores)

  # Single-group correlation
  res_lw <- ssm_analyze(
    jz2017, 
    scales = 2:9,
    measures = "PARPD",
    listwise = TRUE
  )
  res_pw <- ssm_analyze(
    jz2017, 
    scales = 2:9,
    measures = "PARPD",
    listwise = FALSE
  )
  expect_equal(res_lw$scores, res_pw$scores)

  # Multiple-group mean
  res_lw <- ssm_analyze(
    jz2017, 
    scales = 2:9,
    grouping = "Gender",
    listwise = TRUE
  )
  res_pw <- ssm_analyze(
    jz2017, 
    scales = 2:9, 
    grouping = "Gender",
    listwise = FALSE
  )
  expect_equal(res_lw$scores, res_pw$scores)

  # Multiple-group correlation
  res_lw <- ssm_analyze(
    jz2017, 
    scales = 2:9,
    measures = "PARPD",
    grouping = "Gender", 
    listwise = TRUE
  )
  res_pw <- ssm_analyze(
    jz2017,
    scales = 2:9,
    measures = "PARPD",
    grouping = "Gender", 
    listwise = FALSE
  )
  expect_equal(res_lw$scores, res_pw$scores)
})

test_that("Measure-contrast correlation-based SSM results are correct", {
  skip_on_cran()

  data("jz2017")
  set.seed(12345)
  res <- ssm_analyze(
    jz2017, 
    scales = 2:9,
    measures = c("ASPD", "NARPD"),
    contrast = TRUE
  )

  # Test the output object
  expect_type(res, "list")
  expect_s3_class(res, "circumplex_ssm")

  # Test the results subobject
  expect_equal(round(res$results$e_est, 3), c(0.124, 0.202, 0.079))
  expect_equal(round(res$results$x_est, 3), c(-0.099, -0.062, 0.037))
  expect_equal(round(res$results$y_est, 3), c(0.203, 0.179, -0.024))
  expect_equal(round(res$results$a_est, 3), c(0.226, 0.189, -0.037))
  expect_equal(round(res$results$d_est, 1), as_degree(c(115.9, 109, -7.0)))
  expect_equal(round(res$results$fit_est, 3), c(0.964, 0.957, -0.007))
  expect_equal(res$results$Label, c("ASPD", "NARPD", "NARPD - ASPD"))
  expect_equal(round(res$results$e_lci, 3), c(0.087, 0.169, 0.042))
  expect_equal(round(res$results$e_uci, 3), c(0.158, 0.238, 0.117))
  expect_equal(round(res$results$x_lci, 3), c(-0.133, -0.094, -0.001))
  expect_equal(round(res$results$x_uci, 3), c(-0.064, -0.029, 0.075))
  expect_equal(round(res$results$y_lci, 3), c(0.170, 0.145, -0.063))
  expect_equal(round(res$results$y_uci, 3), c(0.239, 0.213, 0.014))
  expect_equal(round(res$results$a_lci, 3), c(0.191, 0.154, -0.077))
  expect_equal(round(res$results$a_uci, 3), c(0.264, 0.227, 0.003))
  expect_equal(round(res$results$d_lci, 3), as_degree(c(107.327, 98.633, -17.384)))
  expect_equal(round(res$results$d_uci, 3), as_degree(c(124.188, 118.537, 3.245)))

  # Test the scores subobject
  expect_equal(round(res$scores$PA, 3), c(0.368, 0.400, 0.031))
  expect_equal(round(res$scores$BC, 3), c(0.354, 0.385, 0.032))
  expect_equal(round(res$scores$DE, 3), c(0.187, 0.234, 0.047))
  expect_equal(round(res$scores$FG, 3), c(0.045, 0.108, 0.063))
  expect_equal(round(res$scores$HI, 3), c(-0.073, 0.051, 0.124))
  expect_equal(round(res$scores$JK, 3), c(-0.045, 0.058, 0.103))
  expect_equal(round(res$scores$LM, 3), c(-0.018, 0.084, 0.101))
  expect_equal(round(res$scores$NO, 3), c(0.173, 0.300, 0.127))
  expect_equal(res$scores$Group, c("All", "All", "All"))
  expect_equal(res$scores$Measure, c("ASPD", "NARPD", "NARPD - ASPD"))
  expect_equal(res$scores$Label, c("ASPD", "NARPD", "NARPD - ASPD"))

  # Test the details subobject
  expect_equal(res$details$boots, 2000)
  expect_equal(res$details$interval, 0.95)
  expect_true(res$details$listwise)
  expect_equal(res$details$angles, octants())
  expect_true(res$details$contrast)
  expect_equal(res$details$score_type, "Correlation")
})

test_that("Group-contrast correlation-based SSM results are correct", {
  skip_on_cran()

  data("jz2017")
  set.seed(12345)
  res <- ssm_analyze(
    jz2017, 
    scales = 2:9,
    measures = "NARPD",
    grouping = "Gender", 
    contrast = TRUE
  )

  # Test the output object
  expect_type(res, "list")
  expect_s3_class(res, "circumplex_ssm")

  # Test the results subobject
  expect_equal(round(res$results$e_est, 3), c(0.172, 0.244, 0.072))
  expect_equal(round(res$results$x_est, 3), c(-0.080, -0.029, 0.051))
  expect_equal(round(res$results$y_est, 3), c(0.202, 0.146, -0.056))
  expect_equal(round(res$results$a_est, 3), c(0.217, 0.149, -0.068))
  expect_equal(round(res$results$d_est, 1), as_degree(c(111.7, 101.2, -10.4)))
  expect_equal(round(res$results$fit_est, 3), c(0.972, 0.902, -0.071))
  expect_equal(res$results$Label, c("NARPD: Female", "NARPD: Male", "NARPD: Male - Female"))
  expect_equal(round(res$results$e_lci, 3), c(0.126, 0.194, 0.005))
  expect_equal(round(res$results$e_uci, 3), c(0.217, 0.295, 0.142))
  expect_equal(round(res$results$x_lci, 3), c(-0.123, -0.076, -0.015))
  expect_equal(round(res$results$x_uci, 3), c(-0.035, 0.017, 0.111))
  expect_equal(round(res$results$y_lci, 3), c(0.157, 0.101, -0.120))
  expect_equal(round(res$results$y_uci, 3), c(0.247, 0.190, 0.006))
  expect_equal(round(res$results$a_lci, 3), c(0.170, 0.105, -0.133))
  expect_equal(round(res$results$a_uci, 3), c(0.265, 0.195, -0.003))
  expect_equal(round(res$results$d_lci, 3), as_degree(c(100.043, 83.117, -30.168)))
  expect_equal(round(res$results$d_uci, 3), as_degree(c(122.481, 119.726, 12.302)))

  # Test the scores subobject
  expect_equal(round(res$scores$PA, 3), c(0.385, 0.415, 0.029))
  expect_equal(round(res$scores$BC, 3), c(0.377, 0.397, 0.020))
  expect_equal(round(res$scores$DE, 3), c(0.227, 0.240, 0.013))
  expect_equal(round(res$scores$FG, 3), c(0.083, 0.129, 0.045))
  expect_equal(round(res$scores$HI, 3), c(-0.010, 0.138, 0.148))
  expect_equal(round(res$scores$JK, 3), c(-0.007, 0.155, 0.162))
  expect_equal(round(res$scores$LM, 3), c(0.036, 0.158, 0.122))
  expect_equal(round(res$scores$NO, 3), c(0.283, 0.322, 0.039))
  expect_equal(res$scores$Group, c("Female", "Male", "Male - Female"))
  expect_equal(res$scores$Measure, c("NARPD", "NARPD", "NARPD"))
  expect_equal(res$scores$Label, c("NARPD: Female", "NARPD: Male", "NARPD: Male - Female"))

  # Test the details subobject
  expect_equal(res$details$boots, 2000)
  expect_equal(res$details$interval, 0.95)
  expect_true(res$details$listwise)
  expect_equal(res$details$angles, octants())
  expect_true(res$details$contrast)
  expect_equal(res$details$score_type, "Correlation")
})


test_that("ssm_parameters works", {
  data("aw2009")
  scores <- unlist(aw2009[1, ])
  expect_error(ssm_parameters(scores = PANO()))
  expect_error(ssm_parameters(scores = scores, angles = PANO()))
  expect_error(ssm_parameters(scores = scores, angles = quadrants()))
  expect_equal(
    round(ssm_parameters(scores), 2), 
    data.frame(
      Elev = 0.43,
      Xval = 1.25, 
      Yval = -1.31, 
      Ampl = 1.81, 
      Disp = 313.71, 
      Fit = 0.97
    )
  )
})

test_that("ssm_score works", {
  data("aw2009")
  out <- ssm_score(aw2009, scales = PANO(), append = TRUE)
  expect_equal(
    round(out[1:2, 9:14], 2),
    data.frame(
      Elev = c(0.43, 0.23),
      Xval = c(1.25, 1.42),
      Yval = c(-1.31, 0.51),
      Ampl = c(1.81, 1.51),
      Disp = c(313.71, 19.67),
      Fit = c(0.97, 0.92)
    )
  )
})

#' Analysis and visualization of circumplex data
#'
#' circumplex provides functions for analyzing and visualizing circumplex data.
#'
#' Its goal is to unify, modernize, and extend existing methods of working with
#' circumplex data. Its functions share an underlying design philosophy and
#' grammar. The three guiding principles it aspires to are:
#'
#' \itemize{
#' \item \strong{Accessibility:} zero cost, open source, libre; works on many
#'   platforms; easy to use across skill levels
#' \item \strong{Flexibility:} customizable by the user; extendable for other uses;
#'   plays nicely with other packages
#' \item \strong{Consistency:} unit-tested quality control; firm naming conventions;
#'   data flows between functions
#' }
#'
#' To learn more about circumplex, start with the vignettes:
#' `browseVignettes(package = "circumplex")`
#'
## usethis namespace: start
#' @importFrom Rcpp sourceCpp
#' @useDynLib circumplex, .registration = TRUE
## usethis namespace: end
#' @importFrom rlang .data
#' @importFrom stats quantile
#' @keywords internal
"_PACKAGE"

#' Two-letter abbreviations for octant circumplex scales
#'
#' Return a vector of abbreviations for octant circumplex scales, from PA to NO.
#'
#' @param case An optional string the determines whether the abbreviations should be
#'   in uppercase or lowercase. (default = "upper")
#' @return A character vector with eight elements, each corresponding to the
#'   abbreviation of an octant subscale: PA, BC, DE, FG, HI, JK, LM, NO.
#' @export
#' @examples
#' PANO()
#' PANO(case = "lower")
#'
PANO <- function(case = "upper") {
  case <- match.arg(case, choices = c("upper", "lower"))
  out <- c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO")
  if (case == "lower") out <- tolower(out)
  out
}

#' Angular displacements for octant circumplex scales
#'
#' Return a vector of angular displacements, in degrees, for eight equally
#' spaced circumplex scales corresponding to the circumplex octants. Can be
#' passed to the \code{angles} parameter of other functions in this package.
#'
#' @return A numeric vector with eight elements, each corresponding to the
#'   angular displacement (in degrees) of a subscale, in the following order:
#'   PA, BC, DE, FG, HI, JK, LM, NO.
#' @export
#' @examples
#' octants()
octants <- function() {
  as_degree(c(90, 135, 180, 225, 270, 315, 360, 45))
}

#' Angular displacements for pole circumplex scales
#'
#' Return a vector of angular displacements, in degrees, for four equally spaced
#' circumplex scales corresponding to the circumplex poles. Can be passed to the
#' \code{angles} parameter of other functions in this package.
#'
#' @return A numeric vector with four elements, each corresponding to the
#'   angular displacement (in degrees) of a subscale, in the following order:
#'   PA, DE, HI, LM.
#' @export
#' @examples
#' poles()
poles <- function() {
  as_degree(c(90, 180, 270, 360))
}

#' Angular displacements for quadrant circumplex scales
#'
#' Return a vector of angular displacements, in degrees, for four equally spaced
#' circumplex scales corresponding to the circumplex quadrants. Can be passed to
#' the \code{angles} parameter of other functions in this package.
#'
#' @return A numeric vector with eight elements, each corresponding to the
#'   angular displacement (in degrees) of a subscale, in the following order:
#'   BC, FG, JK, NO.
#' @export
#' @examples
#' quadrants()
quadrants <- function() {
  as_degree(c(135, 225, 315, 45))
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

angle_mean <- function(x) {
    .Call(`_circumplex_angle_mean`, x)
}

angle_dev <- function(theta, xv) {
    .Call(`_circumplex_angle_dev`, theta, xv)
}

angle_median <- function(x) {
    .Call(`_circumplex_angle_median`, x)
}

ssm_parameters_cpp <- function(scores, angles) {
    .Call(`_circumplex_ssm_parameters_cpp`, scores, angles)
}

group_parameters <- function(scores, angles) {
    .Call(`_circumplex_group_parameters`, scores, angles)
}

col_means <- function(x) {
    .Call(`_circumplex_col_means`, x)
}

mean_scores <- function(cs, grp, lwd) {
    .Call(`_circumplex_mean_scores`, cs, grp, lwd)
}

pairwise_r <- function(x, y) {
    .Call(`_circumplex_pairwise_r`, x, y)
}

corr_scores <- function(cs, mv, grp, lwd) {
    .Call(`_circumplex_corr_scores`, cs, mv, grp, lwd)
}

test_that("SSM parameters are correct", {
  data("aw2009")
  scores <- colMeans(aw2009)
  res <- ssm_parameters_cpp(scores, as_radian(octants()))

  expect_equal(round(res[[1]], 3), 0.423)
  expect_equal(round(res[[2]], 3), 0.945)
  expect_equal(round(res[[3]], 3), -0.264)
  expect_equal(round(res[[4]], 3), 0.981)
  expect_equal(round(res[[5]], 3), 6.010)
  expect_equal(round(res[[6]], 3), 0.954)
})

test_that("Column means are correct even with missing", {
  mat <- matrix(runif(200), ncol = 2)
  idx <- sample(1:200, 50, replace = FALSE)
  mat[idx] <- NA
  rcm <- colMeans(mat, na.rm = TRUE)
  ccm <- col_means(mat)
  expect_equal(rcm[1], ccm[[1]])
  expect_equal(rcm[2], ccm[[2]])
})

test_that("Pairwise r is correct even with missing", {
  x <- runif(100)
  y <- runif(100)
  xidx <- sample(1:100, 10, replace = FALSE)
  x[xidx] <- NA
  yidx <- sample(1:100, 10, replace = FALSE)
  y[yidx] <- NA
  rcor <- stats::cor(x, y, use = "pairwise.complete.obs")
  ccor <- pairwise_r(x, y)
  expect_equal(rcor, ccor)
})

test_that("Angular mean is correct", {
  am <- angle_mean(c(0, 0, pi / 2))
  expect_equal(round(am, 3), 0.464)

  am <- angle_mean(as_radian(octants()))
  expect_true(is.na(am))
})

test_that("Angular median is correct", {
  # In the case of a single median candidate
  amdn <- angle_median(c(0, pi / 2, pi))
  expect_equal(amdn, pi / 2)
  # In the case of multiple median candidates
  amdn <- angle_median(c(0, pi / 2, pi, pi))
  expect_equal(round(amdn, 3), 2.678)
})

test_that("Angluar deviation from mean is correct", {
  angles <- c(0, pi / 2, pi, pi)
  ad <- angle_dev(angles, angle_mean(angles))
  expect_equal(round(ad, 3), 1.178)
})

---
title: "Intermediate SSM Analysis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Intermediate SSM Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(ggplot2)
library(ggforce)
library(kableExtra)
set.seed(12345)
```

```{r setup}
library(circumplex)
```

## 1. Generalizing the basic analyses

### Generalizing to multiple means
We've already seen how adding the `measures` argument can change `ssm_analyze()` from analyzing means to analyzing correlations. Similarly, we can change it from analyzing all observations as a single group to analyzing subgroups separately. This is done using the `grouping` argument. This argument needs to contain a single variable (name or column number) that specifies each observation's group. For instance, the `Gender` variable in the `jz2017` dataset is a factor with two levels: Female and Male. To analyze each gender separately, we need to add the `grouping = "Gender"` argument to the function call.

```{r group}
data("jz2017")
results <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(), 
  grouping = "Gender"
)
summary(results)
```

Note that the output of `summary()` looks the same as previous mean-based analyses except that there are now two Profile blocks: one for Female and one for Male. A similar modification will occur if we generate a table and figure.

```r
ssm_table(results)
```

```{r group_table, echo = FALSE}
ssm_table(results, render = FALSE) %>% 
  knitr::kable(caption = circumplex:::dcaption(results)) %>% 
  kableExtra::kable_styling(full_width = TRUE, font_size = 12)
```

```{r group_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"}
ssm_plot_circle(results)
```

```{r group_plot2, fig.width = 7.5, fig.height = 4, out.width = "100%"}
ssm_plot_curve(results)
```

Any number of groups can be analyzed in this way, and the output will contain additional profile blocks, the table will contain additional rows, and the figures will contain additional points or curves. The grouping variable just needs to contain more than one level (i.e., unique value).

### Generalizing to multiple measures
Similarly, we can analyze multiple external measures in a single function call by providing a vector of variables to the `measures` argument instead of a single variable. This can be done by wrapping the variable names or column numbers with the `c()` or, if they are adjacent columns, with the column numbers and the `:` shortcut. The package functions were written to analyze all measures and groups within a single bootstrapping, so adding additional measures and groups should still be fast.

```{r measures}
results2 <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(),
  measures = c("NARPD", "ASPD")
)
summary(results2)
```

```r
ssm_table(results2)
```

```{r measures_table, echo = FALSE}
ssm_table(results2, render = FALSE) %>% 
  knitr::kable(caption = circumplex:::dcaption(results2)) %>% 
  kableExtra::kable_styling(full_width = TRUE, font_size = 12)
```

```{r measures_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"}
ssm_plot_circle(results2)
```

```{r measures_plot2, fig.width = 7.5, fig.height = 4, out.width = "100%"}
ssm_plot_curve(results2)
```

### Generalizing to multiple groups and multiple measures
Finally, it is possible to analyze multiple measures within multiple groups. As you might expect, this requires providing both the `measures` and `grouping` arguments to the same function call. Again, any number of measures and groups is possible. The profiles in such an analysis will be named "Measure: Group" as below.

```{r general}
results3 <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(), 
  grouping = "Gender", 
  measures = 10:12)
summary(results3)
```

```r
ssm_table(results3)
```

```{r general_table, echo = FALSE}
ssm_table(results3, render = FALSE) %>% 
  knitr::kable(caption = circumplex:::dcaption(results3)) %>% 
  kableExtra::kable_styling(full_width = TRUE, font_size = 12)
```

```{r general_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"}
ssm_plot_circle(results3)
```

Note that the borders for the "PARPD: Male" and "SZTPD: Male" blocks are dashed instead of solid. This indicates that these profiles have low fit (i.e., $R^2<.7$). We could alternatively hide these such profiles by adding `drop_lowfit = TRUE` as below (noting that the color guide may change as profiles are dropped).

```{r general_plot2, fig.width = 7.5, fig.height = 4, out.width = "100%"}
ssm_plot_circle(results3, drop_lowfit = TRUE)
```

```{r general_plot3, fig.width = 7.5, fig.height = 4, out.width = "100%"}
ssm_plot_curve(results3)
```

```{r general_plot4, fig.width = 7.5, fig.height = 4, out.width = "100%"}
ssm_plot_curve(results3, drop_lowfit = TRUE)
```

## 2. Contrast analyses
The final argument to master in this vignette is `contrast`, which allows us to compare two groups or measures. Setting this argument to TRUE will first generate scores for each group (or measure), then estimate SSM parameters for these scores, and finally calculate the difference between the parameters. To keep the code simpler and to discourage "fishing expeditions," only two groups or measures can be compared at a time.

### Contrasts between groups' means
To compare the mean profiles for females and males, we can start with the same syntax we had before and then add a `contrast` argument.

```{r model_contrast}
results4 <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(), 
  grouping = "Gender", 
  contrast = TRUE
)
summary(results4)
```

Note that we have the profile blocks for each group as well as a contrast block. The contrast is made by subtracting the first level of the grouping variable from the second level (e.g., Male - Female). This provides an indication of the direction of the contrast. We can again generate a table and figure to display the results.

```r
ssm_table(results4)
```

```{r model_table, echo = FALSE}
ssm_table(results4, render = FALSE) %>% 
  knitr::kable(caption = circumplex:::dcaption(results4)) %>% 
  kableExtra::kable_styling(full_width = TRUE, font_size = 12)
```

```{r model_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"}
ssm_plot_contrast(results4)
```

### Contrasts between measures in a group
Comparing measures in a group is very similar. Again, all we need to do is add the `contrast` argument to the function call containing `measures`. Here we will use a parameter contrast to see what they look like.

```{r measure_contrast}
results5 <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(), 
  measures = c("NARPD", "ASPD"),
  contrast = TRUE
)
summary(results5)
```

```r
ssm_table(results5)
```

```{r measure_contrast_table, echo = FALSE, results = "asis"}
ssm_table(results5, render = FALSE) %>% 
  knitr::kable(caption = circumplex:::dcaption(results5)) %>% 
  kableExtra::kable_styling(full_width = TRUE, font_size = 12)
```

```{r measure_contrast_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"}
ssm_plot_contrast(results5)
```

Here, instead of a circle plot, we see a contrast plot showing the difference between the two measures' SSM parameters and their 95% confidence intervals. Because the confidence interval for the elevation parameter does not include zero, this parameter is significantly different between the measures.

### Contrasts between groups' correlations
Finally, we might want to compare a single measure's profiles in two different groups. To do so, we need to specify the `measures`, the `grouping` variable, and the type of `contrast`. In this case, we need to ensure that we are providing only a single measure and a grouping variable with just two levels (as again only two things can be contrasted at a time). Note that the contrast name in this case will take the form of "Measure: Group2 - Group1".

```{r group_contrast}
results6 <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(), 
  measures = "BORPD", 
  grouping = "Gender", 
  contrast = TRUE
)
summary(results6)
```

```r
ssm_table(results6)
```

```{r group_contrast_table, echo = FALSE, results = "asis"}
ssm_table(results6, render = FALSE) %>% 
  knitr::kable(caption = circumplex:::dcaption(results6)) %>% 
  kableExtra::kable_styling(full_width = TRUE, font_size = 12)
```

```{r group_contrast_plot, fig.width = 7.5, fig.height = 4, out.width = "100%"}
ssm_plot_contrast(results6)
```

## 3. Taxonomy of analysis types
Although all SSM analyses are based on the idea of decomposing circumplex scores into the parameters of a cosine curve, there are actually many different ways to implement it. Each of these implementations allows different questions to be explored. In the table below, a list of the SSM analysis types that are currently implemented in the `circumplex` package is provided. This table also provides the specific combination of arguments needed to implement each analysis using the `ssm_analyze()` function. Specifying which analysis to run is simply a matter of providing the correct arguments to the function; this allows a single function to essentially do the work of seven and makes for a more intuitive user experience.

```{r taxonomy, echo = FALSE}
msr <- c("FALSE", "FALSE", "FALSE", "TRUE",  "TRUE",  "TRUE",  "TRUE")
grp <- c("FALSE", "TRUE",  "TRUE",  "FALSE", "FALSE", "TRUE",  "TRUE")
ctr <- c("FALSE", "FALSE", "TRUE",  "FALSE", "TRUE",  "FALSE", "TRUE")
tab <- data.frame(
  Usage = c(
    "Examine overall mean profile",
    "Examine groups' mean profiles",
    "Compare groups' mean profiles",
    "Examine variables' correlation profiles",
    "Compare variables' correlation profiles",
    "Examine groups' correlation profiles",
    "Compare groups' correlation profiles"
  ),
  measures = msr,
  grouping = grp,
  contrast = ctr
)
knitr::kable(tab, escape = FALSE) %>% 
  column_spec(1, width = "3in") %>% 
  add_header_above(c("", "Arguments Needed" = 3))
```

The three main questions to ask when conducting a new SSM analysis are:

1. **Do we want to project non-circumplex measures into the circumplex space?**
If yes, we must specify one or more `measures`, and the scores that get modeled using SSM will be the correlations between the circumplex scales and these measures. If no, we must omit the `measures` argument, and the scores that get modeled using SSM will be the mean scores on the circumplex scales.

1. **Do we want to perform analyses separately for groups within the dataset?**
If yes, we must specify a `grouping` variable, and the output will contain results for each group (i.e., unique value of this variable). If no, we must omit the `grouping` argument, and a single set of results for all data will be output.

1. **Do we want to contrast/compare two sets of results?** If yes, we must specify `contrast` as TRUE and ensure that we are only requesting two sets of results (i.e., two groups or two measures). If no, we must omit the `contrast` argument or set it to FALSE, and the results themselves will be output, rather than their contrast.

## 4. Working with SSM tables

### Basic customizations of tables
Additional arguments to the `ssm_table()` function can be explored using the `?ssm_table` command. Two useful options are the `drop_xy` argument, which shows or hides the x-value and y-value columns, and the `caption` argument which allows a custom string to be printed above the table. Note as well that the return object of this function is just a data frame, which can be easily edited to add, change, or remove text. To change the formatting of the table, see the `htmlTable` or `kableExtra` packages.

## 5. Working with SSM figures

### Exporting figures as files
All SSM plots are created using the `ggplot2` package, which is incredibly flexible and powerful. It also offers the `ggsave()` function to export figures to external files of various types. See the documentation for this function (`?ggsave`) to learn more, but some useful arguments are `filename`, `plot`, `width`, `height`, and `units`. We can save the figure as a raster image file (e.g., "png", "jpeg", "tiff"), a vector image file (e.g., "svg"), or a portable document (e.g., "pdf" or "tex"). We can also control the exact width and height of the image in different units (i.e., "in", "cm", or "mm"). Because the underlying graphics are vectorized in R, they can be easily scaled to any size without loss of quality and used in manuscripts, presentations, or posters.

```r
ssm_plot(results6)
ggsave("bordpd_gender.png", width = 7.5, height = 4, units = "in")
```

## Wrap-up
In this vignette, we learned how to generalize the SSM analyses to multiple groups and measures, how to conduct contrast analyses, how to make basic customizations to tables and figures, and how to export tables and figures to external files. In the next vignette, "Advanced Circumplex Visualization," we will learn more advanced customization options for the SSM figures and other circumplex visualizations. (Note that the next vignette is still in progress.)

## References
* Gurtman, M. B. (1992). Construct validity of interpersonal personality measures: The interpersonal circumplex as a nomological net. _Journal of Personality and Social Psychology, 63_(1), 105–118.

* Gurtman, M. B., & Pincus, A. L. (2003). The circumplex model: Methods and research applications. In J. A. Schinka & W. F. Velicer (Eds.), _Handbook of psychology. Volume 2: Research methods in psychology_ (pp. 407–428). Hoboken, NJ: John Wiley & Sons, Inc.

* Wright, A. G. C., Pincus, A. L., Conroy, D. E., & Hilsenroth, M. J. (2009). Integrating methods to optimize circumplex description and comparison of groups. _Journal of Personality Assessment, 91_(4), 311–322.

* Zimmermann, J., & Wright, A. G. C. (2017). Beyond description in interpersonal construct validation: Methodological advances in the circumplex Structural Summary Approach. _Assessment, 24_(1), 3–23.

# Welcome to Circumplex

[![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/pre-commit/pre-commit)
[![Tests status][tests-badge]][tests-link]
[![Linting status][linting-badge]][linting-link]
[![Documentation status][documentation-badge]][documentation-link]
[![License][license-badge]](./license.md)

<!-- prettier-ignore-start -->
[tests-badge]:              https://github.com/MitchellAcoustics/python-circumplex/actions/workflows/tests.yml/badge.svg
[tests-link]:               https://github.com/MitchellAcoustics/python-circumplex/actions/workflows/tests.yml
[linting-badge]:            https://github.com/MitchellAcoustics/python-circumplex/actions/workflows/linting.yml/badge.svg
[linting-link]:             https://github.com/MitchellAcoustics/python-circumplex/actions/workflows/linting.yml
[documentation-badge]:      https://github.com/MitchellAcoustics/python-circumplex/actions/workflows/docs.yml/badge.svg
[documentation-link]:       https://github.com/MitchellAcoustics/python-circumplex/actions/workflows/docs.yml
[license-badge]:            https://img.shields.io/badge/License-GPL%20v3-blue.svg
<!-- prettier-ignore-end -->

## Overview

![Image title](img/logo-light.png#only-light)
![Image title](img/logo-dark.png#only-dark)

The `circumplex` package provides tools for analyzing and visualizing circumplex data, particularly in psychology research. It implements the **Structural Summary Method (SSM)** with bootstrap confidence intervals for statistical analysis of circular data patterns commonly found in:

- Interpersonal functioning and behavior
- Mood and affect measurement
- Vocational preferences and interests
- Other psychological constructs with circular structure

 This project is a Python implementation based on the R [circumplex](https://circumplex.jmgirard.com/) package. Our goal is to provide a similar functionality and experience for Python users.

!!! note
    This project is still under development. We're working hard to make it as good as possible, but there may be bugs or missing features. If you find any issues, please let us know by submitting an issue on Github.

## Key Features

- **SSM Analysis**: Mean-based and correlation-based structural summary methods
- **Bootstrap Confidence Intervals**: Robust statistical inference for SSM parameters
- **Visualization**: Publication-ready circular plots and curve plots
- **Built-in Instruments**: IIP-SC, CSIG, IPIP-IPC with normative data
- **Flexible Instrument System**: Easy registration of custom circumplex measures
- **Data Tidying**: Ipsatization, scoring, and normative standardization

## Quick Start

### Installation

Install the latest development version:

```bash
pip install git+https://github.com/MitchellAcoustics/python-circumplex.git
```

### Basic Usage

```python
from circumplex import load_dataset, ssm_analyze, OCTANTS

# Load sample data
data = load_dataset('jz2017')

# Define scale columns and their angular positions
scales = ['PA', 'BC', 'DE', 'FG', 'HI', 'JK', 'LM', 'NO']
angles = OCTANTS  # [90, 135, 180, 225, 270, 315, 360, 45]

# Perform SSM analysis
results = ssm_analyze(data, scales=scales, angles=angles)

# View results
results.summary()

# Create visualizations
results.plot_circle()
results.plot_curve()
```

## Documentation

- **[API Reference](api/index.md)** - Complete API documentation
- **[GitHub Repository](https://github.com/MitchellAcoustics/circumplex)** - Source code and issue tracker

## Requirements

- Python 3.11, 3.12, or 3.13
- NumPy, Pandas, SciPy, Matplotlib, Seaborn

## Project Status

This project is in active development. Core SSM analysis functionality is implemented with full numerical parity to the R package (validated to 3+ decimal places). Additional features and documentation are being added continuously.

## Contributing

Contributions are welcome! Please see the [GitHub repository](https://github.com/MitchellAcoustics/python-circumplex) for guidelines.

## Acknowledgments

This project is developed in collaboration with the [Centre for Advanced Research Computing](https://ucl.ac.uk/arc), University College London.

**Author:** Andrew Mitchell ([andrew.mitchell.research@gmail.com](mailto:andrew.mitchell.research@gmail.com))

**Based on:** The R circumplex package by Jeffrey Girard and colleagues

## License

---
title: "Introduction to SSM Analysis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to SSM Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
set.seed(12345)
library(ggforce)
library(kableExtra)
library(ggplot2)
library(knitr)
```

```{r setup}
library(circumplex)
```

## 1. Background and Motivation

### Circumplex models, scales, and data
Circumplex models are popular within many areas of psychology because they offer a parsimonious account of complex psychological domains, such as emotion and interpersonal functioning. This parsimony is achieved by understanding phenomena in a domain as being a "blend" of two primary dimensions. For instance, circumplex models of emotion typically represent affective phenomena as a blend of *valence* (pleasantness versus unpleasantness) and *arousal* (activity versus passivity), whereas circumplex models of interpersonal functioning typically represent interpersonal phenomena as a blend of *communion* (affiliation versus separation) and *agency* (dominance versus submissiveness). These models are often depicted as circles around the intersection of the two dimensions (see figure). Any given phenomenon can be located within this circular space through reference to the two underlying dimensions (e.g., anger is a blend of unpleasantness and activity).

Circumplex scales contain multiple subscales that attempt to measure different blends of the two primary dimensions (i.e., different parts of the circle). Although there have historically been circumplex scales with as many as sixteen subscales, it has become most common to use eight subscales: one for each "pole" of the two primary dimensions and one for each "quadrant" that combines the two dimensions. In order for a set of subscales to be considered circumplex, they must exhibit certain properties. Circumplex fit analyses can be used to quantify these properties.

Circumplex data is composed of scores on a set of circumplex scales for one or more participants (e.g., persons or organizations). Such data is usually collected via self-report, informant-report, or observational ratings in order to locate psychological phenomena within the circular space of the circumplex model. For example, a therapist might want to understand the interpersonal problems encountered by an individual patient, a social psychologist might want to understand the emotional experiences of a group of participants during an experiment, and a personality psychologist might want to understand what kind of interpersonal behaviors are associated with a trait (e.g., extraversion).

```{r model, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"}
angles <- c(90, 135, 180, 225, 270, 315, 360, 45)
alabel <- c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO")

# Create plot ------------------------------------------------------------------

ggplot2::ggplot() +
  # Require plot to be square and remove default styling
  ggplot2::coord_fixed() +
  ggplot2::theme_void() +
  # Expand the axes multiplicatively to fit the labels
  ggplot2::scale_x_continuous(expand = c(0.10, 0)) +
  ggplot2::scale_y_continuous(expand = c(0.10, 0)) +
  # Draw line segments corresponding to the octants
  ggplot2::geom_segment(
    ggplot2::aes(
      x = 0,
      y = 0,
      xend = 5 * cos(angles[c(1, 3, 5, 7)] * pi / 180),
      yend = 5 * sin(angles[c(1, 3, 5, 7)] * pi / 180)
    ),
    color = "gray60",
    linewidth = 1
  ) +
  # Draw inner labels for the octant angles
  ggplot2::geom_label(
    ggplot2::aes(
      x = 3 * cos(angles * pi / 180),
      y = 3 * sin(angles * pi / 180),
      label = sprintf("%d\u00B0", angles)
    ),
    size = 5,
    color = "black",
    label.size = NA,
    hjust = "center",
    vjust = "center"
  ) +
  # Draw the circle
  ggforce::geom_circle(ggplot2::aes(x0 = 0, y0 = 0, r = 5),
    color = "gray50", linewidth = 1.5
  ) +
  # Draw outer labels for octant abbreviations
  ggplot2::geom_label(
    ggplot2::aes(
      x = 5.1 * cos(angles * pi / 180),
      y = 5.1 * sin(angles * pi / 180),
      label = alabel
    ),
    size = 5,
    color = "black",
    label.size = NA,
    hjust = "outward",
    vjust = "outward"
  )
```


### The Structural Summary Method
The Structural Summary Method (SSM) is a technique for analyzing circumplex data that offers practical and interpretive benefits over alternative techniques. It  consists of fitting a cosine curve to the data, which captures the pattern of correlations among scores associated with a circumplex scale (i.e., mean scores on circumplex scales or correlations between circumplex scales and an external  measure). By plotting a set of example scores below, we can gain a visual intuition that a cosine curve makes sense in this case. First, we can examine the scores with a bar chart ignoring the circular relationship among them.

```{r column, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"}
data("jz2017")
rmat <- stats::cor(jz2017[c("NARPD", PANO())], method = "pearson")
r <- rmat[2:9, 1]

# Format data for plotting
dat_r <- data.frame(Scale = factor(PANO()), r = r)

# Create column plot
ggplot2::ggplot(dat_r, ggplot2::aes(x = .data$Scale, y = .data$r)) +
  ggplot2::geom_hline(yintercept = 0, linewidth = 1.25, color = "darkgray") +
  ggplot2::geom_col(position = ggplot2::position_dodge(.9), fill = "red") +
  ggplot2::scale_y_continuous(limits = c(-0.02, 0.5)) +
  ggplot2::labs(title = "Scores") +
  ggplot2::theme(
    axis.title = ggplot2::element_blank(),
    panel.grid.major = ggplot2::element_line(linewidth = 1.0),
    panel.grid.minor.y = ggplot2::element_line(linewidth = 0.5),
    panel.grid.minor.x = ggplot2::element_blank()
  )
```

Next, we can leverage the fact that these subscales have specific angular displacements in the circumplex model (and that 0 and 360 degrees are the same) to create a path diagram.

```{r path, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"}
dat_r <- data.frame(
  Scale = factor(PANO()),
  est = r,
  Angle = octants()
)

dat_r <- dat_r[order(dat_r$Angle), ]

# Plot correlations as connected point ranges with 95% CI ranges
ggplot2::ggplot(dat_r, ggplot2::aes(x = .data$Angle, y = .data$est)) +
  ggplot2::geom_hline(yintercept = 0, linewidth = 1.25, color = "darkgray") +
  ggplot2::geom_point(size = 3, color = "red") +
  ggplot2::geom_path(linewidth = 1.25, color = "red") +
  ggplot2::geom_label(ggplot2::aes(label = Scale), nudge_y = 0.075) +
  ggplot2::scale_x_continuous(
    breaks = octants(),
    expand = c(0.05, 0),
    labels = function(x) sprintf("%.0f\U00B0", x)
  ) +
  ggplot2::scale_y_continuous(
    limits = c(-0.02, 0.5)
  ) +
  ggplot2::labs(title = "Scores by Angle") +
  ggplot2::theme(
    axis.title = ggplot2::element_blank(),
    plot.margin = ggplot2::unit(c(10, 30, 10, 10), "points"),
    panel.grid.major = ggplot2::element_line(linewidth = 1.0),
    panel.grid.minor.y = ggplot2::element_line(linewidth = 0.5),
    panel.grid.minor.x = ggplot2::element_blank()
  )
```

This already looks like a cosine curve, and we can finally use the SSM to  estimate the parameters of the curve that best fits the observed data. By plotting it alongside the data, we can get a sense of how well the model fits our example data.

```{r curve, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"}
# Calculate SSM parameters
sp <- ssm_parameters(r, octants())

# Create function for SSM cosine model
f <- function(x) {
  sp$Elev + sp$Ampl * cos((x - sp$Disp) * pi / 180)
}

# Plot correlations along with SSM cosine model
ggplot2::ggplot(dat_r, ggplot2::aes(x = .data$Angle, y = .data$est)) +
  ggplot2::geom_hline(yintercept = 0, linewidth = 1.25, color = "darkgray") +
  ggplot2::geom_point(size = 3) +
  ggplot2::geom_path(linewidth = 1.25) +
  ggplot2::stat_function(fun = f, linewidth = 2, color = "red") +
  ggplot2::scale_x_continuous(
    breaks = octants(),
    expand = c(0.01, 0),
    labels = function(x) sprintf("%.0f\U00B0", x)
  ) +
  ggplot2::scale_y_continuous(
    limits = c(-0.02, 0.5)
  ) +
  ggplot2::labs(title = "Cosine Curve Estimated by SSM") +
  ggplot2::theme(
    axis.title = ggplot2::element_blank(),
    plot.margin = ggplot2::unit(c(10, 30, 10, 10), "points"),
    panel.grid.major = ggplot2::element_line(linewidth = 1.0),
    panel.grid.minor.y = ggplot2::element_line(linewidth = 0.5),
    panel.grid.minor.x = ggplot2::element_blank()
  )
```

### Understanding the SSM parameters
The SSM estimates a cosine curve to the data using the following equation:
$$S_i = e + a \times \cos(\theta_i - d)$$
where $S_i$ and $\theta_i$ are the score and angle on scale $i$, respectively, and $e$, $a$, and $d$ are the elevation, amplitude, and displacement parameters, respectively. Before we discuss these parameters, however, we can also estimate the fit of the SSM model. This is essentially how close the cosine curve is to the observed data points. Deviations (in red, below) will lower model fit.

```{r residuals, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"}
# Plot correlations as path, SSM cosine model, and differences
ggplot2::ggplot(dat_r, ggplot2::aes(x = .data$Angle, y = .data$est)) +
  ggplot2::geom_hline(yintercept = 0, linewidth = 1.25, color = "darkgray") +
  ggplot2::stat_function(fun = f, linewidth = 2, color = "gray20") +
  ggplot2::geom_point(size = 5.5, color = "black") +
  ggplot2::geom_path(linewidth = 1.25, color = "black") +
  ggplot2::geom_segment(
    ggplot2::aes(
      x = .data$Angle, 
      xend = .data$Angle, 
      y = .data$est, 
      yend = f(.data$Angle)
    ),
    linewidth = 4, 
    linetype = "solid", 
    color = "red"
  ) +
  ggplot2::scale_x_continuous(
    breaks = octants(),
    expand = c(0.01, 0),
    labels = function(x) sprintf("%.0f\U00B0", x)
  ) +
  ggplot2::scale_y_continuous(
    limits = c(-0.02, 0.475)
  ) +
  ggplot2::labs(title = sprintf("Fit = %.2f", sp[[6]])) +
  ggplot2::theme(
    axis.title = ggplot2::element_blank(),
    plot.margin = ggplot2::unit(c(10, 30, 10, 10), "points"),
    panel.grid.major = ggplot2::element_line(linewidth = 1.0),
    panel.grid.minor.y = ggplot2::element_line(linewidth = 0.5),
    panel.grid.minor.x = ggplot2::element_blank()
  )
```

If fit is less than 0.70, it is considered "unacceptable" and only the elevation parameter should be interpreted. If fit is between 0.70 and 0.80, it is  considered "adequate," and if it is greater than 0.80, it is considered "good." Sometimes SSM model fit is called prototypicality or denoted using $R^2$.

The first SSM parameter is elevation or $e$, which is calculated as the mean of all scores. It is the size of the general factor in the circumplex model and its interpretation varies from scale to scale. For measures of interpersonal problems, it is interpreted as generalized interpersonal distress. When using correlation-based SSM, $|e|\ge.15$ is considered "marked" and $|e|<.15$ is considered "modest."

```{r elev, echo = FALSE, out.width = "100%"}
knitr::include_graphics("VIG1-e.gif")
```

The second SSM parameter is amplitude or $a$, which is calculated as the difference between the highest point of the curve and the curve's mean. It is interpreted as the distinctiveness or differentiation of a profile: how much it is peaked versus flat. Similar to elevation, when using correlation-based SSM, $a\ge.15$ is considered "marked" and $a<.15$ is considered "modest."

```{r ampl, echo = FALSE, out.width = "100%"}
knitr::include_graphics("VIG1-a.gif")
```

The final SSM parameter is displacement or $d$, which is calculated as the angle at which the curve reaches its highest point. It is interpreted as the style of the profile. For instance, if $d=90^\circ$ and we are using a circumplex scale that defines 90 degrees as "domineering," then the profile's style is domineering.

```{r disp, echo = FALSE, out.width = "100%"}
knitr::include_graphics("VIG1-d.gif")
```

By interpreting these three parameters, we can understand a profile much more parsimoniously than by trying to interpret all eight subscales individually. This approach also leverages the circumplex relationship (i.e., dependency) among subscales. It is also possible to transform the amplitude and displacement parameters into estimates of distance from the x-axis and y-axis, which will be shown in the output discussed below.

## 2. Example data: jz2017
To illustrate the SSM functions, we will use the example dataset `jz2017`, which was provided by Zimmermann & Wright (2017) and reformatted for this package. This dataset includes self-report data from 1166 undergraduate students. Students completed a circumplex measure of interpersonal problems with eight subscales (PA, BC, DE, FG, HI, JK, LM, and NO) and a measure of personality disorder symptoms with ten subscales (PARPD, SCZPD, SZTPD, ASPD, BORPD, HISPD, NARPD, AVPD, DPNPD, and OCPD). More information about these variables can be accessed using the `?jz2017` command in R.

```{r jz2017}
data("jz2017")
head(jz2017)
```

The circumplex scales in `jz2017` come from the Inventory of Interpersonal Problems - Short Circumplex (IIP-SC). These scales can be arranged into the following circular model, which is organized around the two primary dimensions of agency (y-axis) and communion (x-axis). Note that the two-letter scale abbreviations and angular values are based in convention. A high score on PA indicates that one has interpersonal problems related to being "domineering" or too high on agency, whereas a high score on DE indicates problems related to being "cold" or too low on communion. Scales that are not directly on the y-axis or x-axis (i.e., BC, FG, JK, and NO) represent blends of agency and communion.

```{r iipsc, echo = FALSE, fig.width = 7.5, fig.height = 4, out.width = "100%"}
angles <- c(90, 135, 180, 225, 270, 315, 360, 45)
flabel <- c(
  "Domineering",
  "Vindictive",
  "Cold",
  "Socially\nAvoidant",
  "Nonassertive",
  "Easily\nExploited",
  "Overly\nNurturant",
  "Intrusive"
)
alabel <- c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO")

# Create plot ------------------------------------------------------------------

ggplot2::ggplot() +
  # Require plot to be square and remove default styling
  ggplot2::coord_fixed() +
  ggplot2::theme_void() +
  # Expand both axes multiplicatively to fit the labels
  ggplot2::scale_x_continuous(expand = c(0.30, 0)) +
  ggplot2::scale_y_continuous(expand = c(0.10, 0)) +
  # Draw line segments corresponding to the octants
  ggplot2::geom_segment(
    ggplot2::aes(
      x = 0,
      y = 0,
      xend = 5 * cos(angles * pi / 180),
      yend = 5 * sin(angles * pi / 180)
    ),
    color = "gray60",
    linewidth = 0.5
  ) +
  # Draw inner labels for the octant abbreviations
  ggplot2::geom_label(
    ggplot2::aes(
      x = 3.75 * cos(angles * pi / 180),
      y = 3.75 * sin(angles * pi / 180),
      label = alabel
    ),
    size = 5,
    color = "gray40",
    label.size = NA,
    hjust = "center",
    vjust = "center"
  ) +
  # Draw inner labels for the octant angles
  ggplot2::geom_label(
    ggplot2::aes(
      x = 2 * cos(angles * pi / 180),
      y = 2 * sin(angles * pi / 180),
      label = sprintf("%d\u00B0", angles)
    ),
    size = 4,
    color = "gray50",
    label.size = NA,
    hjust = "center",
    vjust = "center"
  ) +
  # Draw the circle
  ggforce::geom_circle(
    ggplot2::aes(x0 = 0, y0 = 0, r = 5),
    color = "gray50", 
    linewidth = 1.5
  ) +
  # Draw outer labels for the octant names
  ggplot2::geom_label(
    ggplot2::aes(
      x = 5.1 * cos(angles * pi / 180),
      y = 5.1 * sin(angles * pi / 180),
      label = flabel
    ),
    size = 5,
    color = "black",
    label.size = NA,
    hjust = "outward",
    vjust = "outward"
  )
```

## 3. Mean-based SSM Analysis

### Conducting SSM for a group's mean scores
To begin, let's say that we want to use the SSM to describe the interpersonal problems of the average individual in the entire dataset. We will use the raw scores contained in `jz2017`, although note that the results may be more interpretable if we standardized the scores first (e.g., using the `norm_standardize()` function as described in the "Using Circumplex Instruments" vignette). 

We can use the `ssm_analyze()` function to perform the SSM analysis. The first three arguments to this function are `data` (the data frame containing our scores), `scales` (a vector of the names or numbers of columns containing the circumplex scales), and `angles` (a vector of the circular angle of each scale, in degrees). Below, I spell out the names of all the `scales` and the numbers of all the `angles` for teaching purposes. Later on, we will learn some shortcuts.

```{r analyze}
results <- ssm_analyze(
  data = jz2017,
  scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
  angles = c(90, 135, 180, 225, 270, 315, 360, 45)
)
```

The output of the function has been saved in the `results` object, which we can examine in detail using the `summary()` function. This will output the call we made to create the output, as well as some of the default options that we didn't bother changing (see `?ssm_analyze` to learn how to change them) and, most importantly, the estimated SSM parameter values with bootstrapped confidence intervals.

```{r summary1a}
summary(results)
```

That was pretty easy! We can now write up these results. However, the `circumplex` package has some features that can make what we just did even easier. First, because we organized the `jz2017` data frame to have the circumplex scale variables adjacent and in order from PA to NO, we can simplify their specification by using the `PANO()` shortcut. Second, because the use of octant scales is so common, we can use the `octants()` shortcut. Note that, even when using these shortcuts, the results are the same except for minor stochastic differences in the confidence intervals due to the randomness inherent to bootstrapping.

```{r summary1b}
results2 <- ssm_analyze(data = jz2017, scales = PANO(), angles = octants())
summary(results2)
```

### Visualizing the results with a table and figure
Next, we can produce a table to display our results. With only a single set of parameters, this table is probably overkill, but in future analyses we will see how this function saves a lot of time and effort. To create the table, simply pass the `results` (or `results2`) object to the `ssm_table()` function.

```r
ssm_table(results2)
```

```{r table1, echo = FALSE}
ssm_table(results2, render = FALSE) %>%
  knitr::kable(caption = circumplex:::dcaption(results2)) %>%
  kableExtra::kable_styling(full_width = TRUE, font_size = 14)
```

Next, let's leverage the fact that we are working within a circumplex space by creating a nice-looking circular plot by mapping the amplitude parameter to the points' distance from the center of the circle and the displacement parameter to the points' rotation from due-east (as is conventional). This, again, is as simple as passing the `results` object to the `ssm_plot_circle()` or `ssm_plot_curve()` functions.

```{r plot1, fig.width = 7.2, fig.height = 4, out.width = "100%"}
ssm_plot_circle(results2)
```

```{r plot2, fig.width = 7.2, fig.height = 4, out.width = "100%"}
ssm_plot_curve(results2)
```

## 4. Correlation-based SSM Analysis

### Conducting SSM for a group's correlations with an external measure
Next, let's say that we are interested in analyzing not the mean scores on the circumplex scales but rather their correlations with an external measure. This is sometimes referred to as "projecting" that external measure into the circumplex space. As an example, let's project the NARPD variable, which captures symptoms of narcissistic personality disorder, into the circumplex space defined by the IIP-SC. Based on theory and previous findings, we can expect this measure to be associated with some general interpersonal distress and a style that is generally high in agency.

To conduct this analysis, we can start with the syntax from the mean-based analysis. All SSM analyses use the `ssm_analyze()` function and the data, scales, and angles are the same as before. However, we also need to let the function know that we want to analyze correlations with NARPD as opposed to scale means. To do this, we add an additional argument `measures`.

```{r summary2}
results3 <- ssm_analyze(
  data = jz2017, 
  scales = PANO(), 
  angles = octants(),
  measures = "NARPD"
)
summary(results3)
```

Note that this output looks very similar to the mean-based output except that the statistical basis is now correlation scores instead of mean scores and instead of saying "Profile [All]" it now says "Profile [NARPD]".

### Visualizing the results with a table and figure
We can also create a similar table and figure using the exact same syntax as before. The `ssm_table()` and `ssm_plot()` functions are smart enough to know whether the results are mean-based or correlation-based and will work in both cases.

```r
ssm_table(results3)
```

```{r table2, echo = FALSE}
ssm_table(results3, render = FALSE) %>%
  knitr::kable(caption = circumplex:::dcaption(results3)) %>%
  kableExtra::kable_styling(full_width = TRUE, font_size = 14)
```

From the table, we can see that the model fit is good (>.80) and that all three SSM parameters are significantly different from zero, i.e., their confidence intervals do not include zero. Furthermore, the confidence intervals for the elevation and amplitude parameters are greater than or equal to 0.15, which can be interpreted as being "marked." So, consistent with our hypotheses, NARPD was associated with marked general interpersonal distress (elevation) and was markedly distinctive in its profile (amplitude). The displacement parameter was somewhere between 100 and 120 degrees; to interpret this we would need to either consult the mapping between scales and angles or plot the results.

```{r plot3, fig.width = 7.5, fig.height = 4, out.width = "100%"}
ssm_plot_circle(results3)
```

From this figure, it is very easy to see that, consistent with our hypotheses, the displacement for NARPD was associated with high agency and was somewhere between the "domineering" and "vindictive" octants.

```{r plot4, fig.width = 7.5, fig.height = 4, out.width = "100%"}
ssm_plot_curve(results3)
```

## 5. Wrap-up
In this vignette, we learned about circumplex models, scales, and data as well as the Structural Summary Method (SSM) for analyzing such data. We learned about the `circumplex` package and how to use the `ssm_analyze()` function to generate SSM results for a single group's mean scores and for correlations with a single external measure. We learned several shortcuts for making calls to this function easier and then explored the basics of SSM visualization by creating simple tables and circular plots. In the next vignette, "Intermediate SSM Analysis", we will build upon this knowledge to learn how to (1) generalize our analyses to multiple groups and multiple measures, (2) perform contrast analyses to compare groups or measures, and (3) export and make basic changes to tables and figures.

## References
* Gurtman, M. B. (1992). Construct validity of interpersonal personality measures: The interpersonal circumplex as a nomological net. _Journal of Personality and Social Psychology, 63_(1), 105–118.

* Gurtman, M. B., & Pincus, A. L. (2003). The circumplex model: Methods and research applications. In J. A. Schinka & W. F. Velicer (Eds.), _Handbook of psychology. Volume 2: Research methods in psychology_ (pp. 407–428). Hoboken, NJ: John Wiley & Sons, Inc.

* Wright, A. G. C., Pincus, A. L., Conroy, D. E., & Hilsenroth, M. J. (2009). Integrating methods to optimize circumplex description and comparison of groups. _Journal of Personality Assessment, 91_(4), 311–322.

* Zimmermann, J., & Wright, A. G. C. (2017). Beyond description in interpersonal construct validation: Methodological advances in the circumplex Structural Summary Approach. _Assessment, 24_(1), 3–23.

---
title: "Using Circumplex Instruments"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using Circumplex Instruments}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

```{r setup, message=FALSE}
library(circumplex)
```


## 1. Overview of Instrument-related Functions
Although the circumplex package is capable of analyzing and visualizing data in a "source-agnostic" manner (i.e., without knowing what the numbers correspond to), it can be helpful to both the user and the package to have more contextual information about which instrument/questionnaire the data come from. For example, knowing the specific instrument used can enable the package to automatically score item-level responses and standardize these scores using normative data. Furthermore, a centralized repository of information about circumplex instruments would provide a convenient and accessible way for users to discover and begin using new instruments. To address these needs, a suite of new instrument-related functions (and underlying data to power them) was added to the circumplex package in version 0.2.0.

The first part of this vignette will discuss how to preview the instruments currently available in the circumplex package, how to load information about a specific instrument for use in analysis, and how to extract general and specific information about that instrument. The following functions will be discussed: `instruments()`, `print()`, `summary()`, `scales()`, `items()`, `anchors()`, `norms()`, and `View()`.

The second part of this vignette will discuss how to use the information about an instrument to transform and summarize circumplex data. It will demonstrate how to ipsatize item-level responses (i.e., apply deviation scoring across variables), how to calculate scale scores from item-level responses (with or without imputing/prorating missing values), and how to standardize scale scores using normative/comparison data. The following functions will be discussed: `ipsatize()`, `score()`, and `norm_standardize()`.

## 2. Loading and Examining Instrument Objects

### Previewing the available instruments
Our goal is to eventually include all circumplex instruments in this package. However, due to time-constraints and copyright issues, this goal may be delayed or even impossible to fully realize. However, you can always preview the list of currently available instruments using the `instruments()` function. This function will print the abbreviation, name, and (in parentheses) the "code" for each available instrument. We will return to the code in the next section.

```{r}
instruments()
```

If there are additional circumplex instruments you would like to have added to the package (especially if you are the author/copyright-holder), please [contact me](mailto:me@jmgirard.com). Note that some information (e.g., item text and normative data) may not be available for all instruments due to copyright issues. However, many instruments are released as open-access and have full item text and normative data included in the package.

### Loading a specific instrument
Instrument information is loaded into R's memory when the circumplex package is loaded. We can just refer to each by "code," such as the `csip` code for the Circumplex Scales of Interpersonal Problems. The code is an object in R's environment, so do not put it in quotes.

```{r}
csip
```

### Examining an instrument in-depth
To examine the information available about a loaded instrument, there are several options. To print a long list of formatted information about the instrument, use the `summary()` function. This will return information about the instrument's scales, rating scale anchors, items, and normative data set(s). The summary of each instrument is also available from the package  [reference](https://circumplex.jmgirard.com/reference/index.html#section-instrument-data) page.

```{r}
summary(ipipipc)
```

Specific subsections of this output can be returned individually using the `scales()`, `anchors()`, `items()`, and `norms()` functions. These functions are especially useful when you need to know a specific bit of information about an instrument and don't want the console to be flooded with unneeded information.

```{r}
anchors(ipipipc)
```


```{r}
norms(ipipipc)
```

Some of these functions also have additional arguments to customize their output. For instance, the `scales()` function has the `items` argument, which will display the items for each scale when set to `TRUE`.

```{r}
scales(ipipipc, items = TRUE)
```

## 3. Instrument-related Tidying Functions
It is a good idea in practice to digitize and save each participant's response to each item on an instrument, rather than just their scores on each scale. Having access to item-level data will make it easier to spot and correct mistakes, will enable more advanced analysis of missing data, and will enable latent variable models that account for measurement error (e.g., structural equation modeling). Furthermore, the functions described below will make it easy to transform and summarize such item-level data into scale scores.

First, however, we need to make sure the item-level data is in the expected format. Your data should be stored in a data frame where each row corresponds to one observation (e.g., participant, organization, or timepoint) and each column corresponds to one variable describing these observations (e.g., item responses, demographic characteristics, scale scores). The [tidyverse](https://tidyverse.org) packages provide excellent tools for getting your data into this format from a variety of different file types and formats.

For the purpose of illustration, we will work with a small-scale data set, which includes item-level responses to the Inventory of Interpersonal Problems, Short Circumplex (IIP-SC) for just 10 participants. As will become important later on, this data set contains a small amount of missing values (represented as `NA`). This data set is included as part of the circumplex package and can be previewed as follows:

```{r}
raw_iipsc
```

### Ipsatizing item-level data
For some forms of circumplex data analysis (e.g., analysis of circumplex fit) but not others (e.g., structural summary method), it can be helpful to transform item-level responses by subtracting each participant's mean across all items from his or her response on each item. This practice is called "ipsatizing" or, more precisely, deviation scoring across variables. This practice will attenuate the general factor across all items and recasts the item-level responses as deviations from one's own mean rather than absolute responses. To perform ipsatizing and create a new set of ipsatized responses to each item, use the `ipsatize()` function.

```{r}
ips_iipsc <- ipsatize(data = raw_iipsc, items = 1:32, append = FALSE)
print(ips_iipsc)
```

Above, we told the function to take all the variables from column 1 to column 32 and calculate new ipsatized variables ending in `_i` (although different `prefix`es and `suffix`es can be used). By default, the mean for each observation/row is calculated after ignoring any missing values, but we could have changed this by adding `na.rm = FALSE`. By setting `append` to FALSE, only the ipsatized versions are returned, but if we changed this to TRUE then the original variables would have been included as well.

We can check that the ipsatization was successful by calculating the mean of each row (i.e., each participant's mean response) in the original and ipsatized data frames. We do this below using the `rowMeans()` function; we also apply the `round()` function to make the results fit on one row. As expected, we find below that the mean of each participant is zero in the ipsatized data frame but not in the original.

```{r}
round(rowMeans(raw_iipsc, na.rm = TRUE), 2)
round(rowMeans(ips_iipsc, na.rm = TRUE), 2)
```

### Scoring item-level data
For many forms of circumplex data analysis (e.g., structural summary method), it can be very helpful to summarize item-level responses by calculating scale scores. This is typically done by averaging a set of items that all measure the same underlying construct (e.g., location in the circumplex model). For example, the IIP-SC has 32 items in total that measure 8 scales representing octants of the interpersonal circumplex model. Thus, a participant's score on each scale is calculated as the arithmetic mean of his or her responses to four specific items. Using the aggregate of multiple similar items produces scale scores with higher reliability than would be achieved by using only a single item per scale.

```{r}
scales(iipsc)
```

Although calculating the arithmetic mean of a handful of items is not terribly difficult mathematically, doing so manually (e.g., by hand) across multiple scales and multiple participants can be tedious and error-prone. To address these issues, the circumplex package offers the `score()` function, which automatically calculates scale scores from item-level data.

To demonstrate, let's return to the `raw_iipsc` data set. We need to give the `score()` function a data frame containing the item-level data (i.e., the data set), a list of variables from that data frame that contain the item-level responses to be scored, and an instrument object containing instructions on how to score the data. In order for scoring to work properly, **the list of items must be in ascending order from the first to the last item and the ordering of the items must be the same as that assumed by the package**. Be sure to check your item numbers against those displayed by the `items()` function, especially if you shuffle your items.

```{r}
scale_scores <- score(
  data = raw_iipsc,
  items = 1:32,
  instrument = iipsc,
  append = FALSE
)
print(scale_scores)
```

Because we set `append` to FALSE, the `scale_scores` data frame contains only the scale score variables. These were named using the scale abbreviations shown by the `scales()` function (i.e., two-letter abbreviations from PA to NO). You can customize the naming of these variables by using the `prefix` and `suffix` arguments (e.g., to make them IIP_PA to IIP_NO).

Note that the `na.rm` argument for the `score()` function defaulted to `TRUE`, which means that missing values were ignored in calculating the scale scores. This practice is common in the literature, but is technically a form of single imputation and thus can produce biased results when data are not missing completely at random (MCAR). Please examine and report the amount and patterns of missingness in your data.

### Standardizing scale-level data
Finally, it can often be helpful to transform scale-level data through reference to a normative or comparison sample. This is often called "norm standardizing" and involves subtracting the normative sample's mean score on a scale from each participant's score on that scale and then dividing this difference by the normative sample's standard deviation. This rescales the scale scores to be in standard deviation units and to describe the magnitude of each participant's difference from the normative average.

For many circumplex instruments, the data needed to perform standardization is included in its instrument object. Some instruments even have multiple (e.g., different or overlapping) normative samples for comparisons that are matched in terms of gender, age, or nationality. In selecting a normative sample to compare to, it is important to consider both the size and the appropriateness of the sample.

To demonstrate, let's examine the normative data sets available for the IIP-SC. Below we see that there are two options: a rather large sample of American college students and a rather small sample of American psychiatric outpatients.

```{r}
norms(iipsc)
```

Assuming our example data also come from a non-psychiatric community sample of mostly college students, the first normative sample seems like a better choice, especially since it is so much larger and therefore subject to less sampling error. However, there may be times when the second normative sample would be the more appropriate comparison, even despite its smaller sample.

To transform the scale scores we calculated during the last section, we can call the `norm_standardize()` function and give it the `scale_scores` object we created above. We will save the output of this function to a data frame named `z_scales` to reflect the idea that standardized scores are often called "z-scores."

```{r}
z_scales <- norm_standardize(
  data = scale_scores,
  scales = 1:8,
  instrument = iipsc,
  sample = 1,
  append = FALSE
)
print(z_scales)
```

Again, because we set `append` to FALSE, the output contains only the norm standardized scale-level variables. The new variables are named the same as the scale score variables except with a configurable `prefix` and `suffix` (by default, they are given only a suffix of `_z`). These variables are the ones we are most likely to use in subsequent analyses (e.g., the structural summary method).

## 4. Wrap-up
In this vignette, we learned how to preview the instruments available in the circumplex package, load and examine the information contained in one of these instrument objects, ipsatize item-level data, calculate scale scores from item-level data, and standardize those scale scores using normative data included in the package. We are now in an excellent position to discover and implement new circumplex instruments. Later vignettes describe analyses and visualizations that make use of the data collected using these tools.

Special thanks to the authors and publishers who granted permission to include information about their instruments in this package: Chloe Bliton, Michael Boudreaux, Robert Hatcher, Christopher Hopwood, Leonard Horowitz, Kenneth Locke, Patrick Markey, Aaron Pincus, Elisa Trucco, and MindGarden Inc.

---
title: "Using Circumplex Instruments"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using Circumplex Instruments}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

```{r setup, message=FALSE}
library(circumplex)
```


## 1. Overview of Instrument-related Functions
Although the circumplex package is capable of analyzing and visualizing data in a "source-agnostic" manner (i.e., without knowing what the numbers correspond to), it can be helpful to both the user and the package to have more contextual information about which instrument/questionnaire the data come from. For example, knowing the specific instrument used can enable the package to automatically score item-level responses and standardize these scores using normative data. Furthermore, a centralized repository of information about circumplex instruments would provide a convenient and accessible way for users to discover and begin using new instruments. To address these needs, a suite of new instrument-related functions (and underlying data to power them) was added to the circumplex package in version 0.2.0.

The first part of this vignette will discuss how to preview the instruments currently available in the circumplex package, how to load information about a specific instrument for use in analysis, and how to extract general and specific information about that instrument. The following functions will be discussed: `instruments()`, `print()`, `summary()`, `scales()`, `items()`, `anchors()`, `norms()`, and `View()`.

The second part of this vignette will discuss how to use the information about an instrument to transform and summarize circumplex data. It will demonstrate how to ipsatize item-level responses (i.e., apply deviation scoring across variables), how to calculate scale scores from item-level responses (with or without imputing/prorating missing values), and how to standardize scale scores using normative/comparison data. The following functions will be discussed: `ipsatize()`, `score()`, and `norm_standardize()`.

## 2. Loading and Examining Instrument Objects

### Previewing the available instruments
Our goal is to eventually include all circumplex instruments in this package. However, due to time-constraints and copyright issues, this goal may be delayed or even impossible to fully realize. However, you can always preview the list of currently available instruments using the `instruments()` function. This function will print the abbreviation, name, and (in parentheses) the "code" for each available instrument. We will return to the code in the next section.

```{r}
instruments()
```

If there are additional circumplex instruments you would like to have added to the package (especially if you are the author/copyright-holder), please [contact me](mailto:me@jmgirard.com). Note that some information (e.g., item text and normative data) may not be available for all instruments due to copyright issues. However, many instruments are released as open-access and have full item text and normative data included in the package.

### Loading a specific instrument
Instrument information is loaded into R's memory when the circumplex package is loaded. We can just refer to each by "code," such as the `csip` code for the Circumplex Scales of Interpersonal Problems. The code is an object in R's environment, so do not put it in quotes.

```{r}
csip
```

### Examining an instrument in-depth
To examine the information available about a loaded instrument, there are several options. To print a long list of formatted information about the instrument, use the `summary()` function. This will return information about the instrument's scales, rating scale anchors, items, and normative data set(s). The summary of each instrument is also available from the package  [reference](https://circumplex.jmgirard.com/reference/index.html#section-instrument-data) page.

```{r}
summary(ipipipc)
```

Specific subsections of this output can be returned individually using the `scales()`, `anchors()`, `items()`, and `norms()` functions. These functions are especially useful when you need to know a specific bit of information about an instrument and don't want the console to be flooded with unneeded information.

```{r}
anchors(ipipipc)
```


```{r}
norms(ipipipc)
```

Some of these functions also have additional arguments to customize their output. For instance, the `scales()` function has the `items` argument, which will display the items for each scale when set to `TRUE`.

```{r}
scales(ipipipc, items = TRUE)
```

## 3. Instrument-related Tidying Functions
It is a good idea in practice to digitize and save each participant's response to each item on an instrument, rather than just their scores on each scale. Having access to item-level data will make it easier to spot and correct mistakes, will enable more advanced analysis of missing data, and will enable latent variable models that account for measurement error (e.g., structural equation modeling). Furthermore, the functions described below will make it easy to transform and summarize such item-level data into scale scores.

First, however, we need to make sure the item-level data is in the expected format. Your data should be stored in a data frame where each row corresponds to one observation (e.g., participant, organization, or timepoint) and each column corresponds to one variable describing these observations (e.g., item responses, demographic characteristics, scale scores). The [tidyverse](https://www.tidyverse.org) packages provide excellent tools for getting your data into this format from a variety of different file types and formats.

For the purpose of illustration, we will work with a small-scale data set, which includes item-level responses to the Inventory of Interpersonal Problems, Short Circumplex (IIP-SC) for just 10 participants. As will become important later on, this data set contains a small amount of missing values (represented as `NA`). This data set is included as part of the circumplex package and can be previewed as follows:

```{r}
raw_iipsc
```

### Ipsatizing item-level data
For some forms of circumplex data analysis (e.g., analysis of circumplex fit) but not others (e.g., structural summary method), it can be helpful to transform item-level responses by subtracting each participant's mean across all items from his or her response on each item. This practice is called "ipsatizing" or, more precisely, deviation scoring across variables. This practice will attenuate the general factor across all items and recasts the item-level responses as deviations from one's own mean rather than absolute responses. To perform ipsatizing and create a new set of ipsatized responses to each item, use the `ipsatize()` function.

```{r}
ips_iipsc <- ipsatize(data = raw_iipsc, items = 1:32, append = FALSE)
print(ips_iipsc)
```

Above, we told the function to take all the variables from column 1 to column 32 and calculate new ipsatized variables ending in `_i` (although different `prefix`es and `suffix`es can be used). By default, the mean for each observation/row is calculated after ignoring any missing values, but we could have changed this by adding `na.rm = FALSE`. By setting `append` to FALSE, only the ipsatized versions are returned, but if we changed this to TRUE then the original variables would have been included as well.

We can check that the ipsatization was successful by calculating the mean of each row (i.e., each participant's mean response) in the original and ipsatized data frames. We do this below using the `rowMeans()` function; we also apply the `round()` function to make the results fit on one row. As expected, we find below that the mean of each participant is zero in the ipsatized data frame but not in the original.

```{r}
round(rowMeans(raw_iipsc, na.rm = TRUE), 2)
round(rowMeans(ips_iipsc, na.rm = TRUE), 2)
```

### Scoring item-level data
For many forms of circumplex data analysis (e.g., structural summary method), it can be very helpful to summarize item-level responses by calculating scale scores. This is typically done by averaging a set of items that all measure the same underlying construct (e.g., location in the circumplex model). For example, the IIP-SC has 32 items in total that measure 8 scales representing octants of the interpersonal circumplex model. Thus, a participant's score on each scale is calculated as the arithmetic mean of his or her responses to four specific items. Using the aggregate of multiple similar items produces scale scores with higher reliability than would be achieved by using only a single item per scale. 

```{r}
scales(iipsc)
```

Although calculating the arithmetic mean of a handful of items is not terribly difficult mathematically, doing so manually (e.g., by hand) across multiple scales and multiple participants can be tedious and error-prone. To address these issues, the circumplex package offers the `score()` function, which automatically calculates scale scores from item-level data.

To demonstrate, let's return to the `raw_iipsc` data set. We need to give the `score()` function a data frame containing the item-level data (i.e., the data set), a list of variables from that data frame that contain the item-level responses to be scored, and an instrument object containing instructions on how to score the data. In order for scoring to work properly, **the list of items must be in ascending order from the first to the last item and the ordering of the items must be the same as that assumed by the package**. Be sure to check your item numbers against those displayed by the `items()` function, especially if you shuffle your items.

```{r}
scale_scores <- score(
  data = raw_iipsc, 
  items = 1:32, 
  instrument = iipsc, 
  append = FALSE
)
print(scale_scores)
```

Because we set `append` to FALSE, the `scale_scores` data frame contains only the scale score variables. These were named using the scale abbreviations shown by the `scales()` function (i.e., two-letter abbreviations from PA to NO). You can customize the naming of these variables by using the `prefix` and `suffix` arguments (e.g., to make them IIP_PA to IIP_NO).

Note that the `na.rm` argument for the `score()` function defaulted to `TRUE`, which means that missing values were ignored in calculating the scale scores. This practice is common in the literature, but is technically a form of single imputation and thus can produce biased results when data are not missing completely at random (MCAR). Please examine and report the amount and patterns of missingness in your data. 

### Standardizing scale-level data
Finally, it can often be helpful to transform scale-level data through reference to a normative or comparison sample. This is often called "norm standardizing" and involves subtracting the normative sample's mean score on a scale from each participant's score on that scale and then dividing this difference by the normative sample's standard deviation. This rescales the scale scores to be in standard deviation units and to describe the magnitude of each participant's difference from the normative average.

For many circumplex instruments, the data needed to perform standardization is included in its instrument object. Some instruments even have multiple (e.g., different or overlapping) normative samples for comparisons that are matched in terms of gender, age, or nationality. In selecting a normative sample to compare to, it is important to consider both the size and the appropriateness of the sample.

To demonstrate, let's examine the normative data sets available for the IIP-SC. Below we see that there are two options: a rather large sample of American college students and a rather small sample of American psychiatric outpatients.

```{r}
norms(iipsc)
```

Assuming our example data also come from a non-psychiatric community sample of mostly college students, the first normative sample seems like a better choice, especially since it is so much larger and therefore subject to less sampling error. However, there may be times when the second normative sample would be the more appropriate comparison, even despite its smaller sample.

To transform the scale scores we calculated during the last section, we can call the `norm_standardize()` function and give it the `scale_scores` object we created above. We will save the output of this function to a data frame named `z_scales` to reflect the idea that standardized scores are often called "z-scores."

```{r}
z_scales <- norm_standardize(
  data = scale_scores, 
  scales = 1:8, 
  instrument = iipsc,
  sample = 1,
  append = FALSE
)
print(z_scales)
```

Again, because we set `append` to FALSE, the output contains only the norm standardized scale-level variables. The new variables are named the same as the scale score variables except with a configurable `prefix` and `suffix` (by default, they are given only a suffix of `_z`). These variables are the ones we are most likely to use in subsequent analyses (e.g., the structural summary method).

## 4. Wrap-up
In this vignette, we learned how to preview the instruments available in the circumplex package, load and examine the information contained in one of these instrument objects, ipsatize item-level data, calculate scale scores from item-level data, and standardize those scale scores using normative data included in the package. We are now in an excellent position to discover and implement new circumplex instruments. Later vignettes describe analyses and visualizations that make use of the data collected using these tools.

Special thanks to the authors and publishers who granted permission to include information about their instruments in this package: Chloe Bliton, Michael Boudreaux, Robert Hatcher, Christopher Hopwood, Leonard Horowitz, Kenneth Locke, Patrick Markey, Aaron Pincus, Elisa Trucco, and MindGarden Inc.

# unit classes are working

    Code
      octants()
    Output
      90 135 180 225 270 315 360 45 
      Degrees

---

    Code
      as_radian(octants())
    Output
      1.571 2.356 3.142 3.927 4.712 5.498 6.283 0.785 
      Radians

# The print method for the S3 instrument class produces the right output

    ISC: Interpersonal Sensitivities Circumplex
    64 items, 8 scales, 1 normative data sets
    Hopwood et al. (2011)
    <https://doi.org/10.1111/j.1467-6494.2011.00696.x>

# The summary method for the S3 instrument class produces the right output

    ISC: Interpersonal Sensitivities Circumplex
    64 items, 8 scales, 1 normative data sets
    Hopwood et al. (2011)
    <https://doi.org/10.1111/j.1467-6494.2011.00696.x>
    
    The ISC contains 8 circumplex scales.
    PA: Sensitive to Control (90 degrees)
    BC: Sensitive to Antagonism (135 degrees)
    DE: Sensitive to Remoteness (180 degrees)
    FG: Sensitive to Timidity (225 degrees)
    HI: Sensitive to Passivity (270 degrees)
    JK: Sensitive to Dependence (315 degrees)
    LM: Sensitive to Affection (360 degrees)
    NO: Sensitive to Attention-Seeking (45 degrees)
    
    The ISC is rated using the following 8-point scale.
    1. Not at all, never bothers me
    2. Very little, rarely bothers me
    3. A little, occasionally bothers me
    4. Slightly, bothers me less than half the time
    5. Somewhat, bothers me more than half of the time
    6. Quite, bothers me frequently
    7. Very much, bothers me most of the time
    8. Extremely, always bothers me
    
    The ISC contains 64 items (open-access):
    Prefix: It bothers me when a person...
    1. Thinks they are my boss
    2. Disregards my feelings
    3. Doesn't show any feelings
    4. Won't engage in conversation
    5. Cannot assert themselves
    6. Avoids conflict at all costs
    7. Wants to spend lots of time with me
    8. Doesn't respect my privacy
    9. Orders me around
    10. Doesn't respond to me
    11. Doesn't want to be friends
    12. Is aloof
    13. Doesn't have a backbone
    14. Always needs support
    15. Tries to get close to me really fast
    16. Talks themselves up
    17. Talks down to me
    18. Is hostile
    19. Walls themselves off from me
    20. Acts like a wallflower
    21. Is weak
    22. Always seems to need my help
    23. Tells me they love me
    24. Shows off
    25. Will do anything to get what they want
    26. Is deceitful
    27. Won't share their feelings with me
    28. Won't step up to the plate
    29. Allows themselves to be dominated by others
    30. Acts helpless
    31. Expresses concern about me
    32. Talks about themselves
    33. Has to call the shots
    34. Mistrusts me
    35. Is unresponsive
    36. Doesn't share their ideas
    37. Is always submissive
    38. Laughs at all my jokes
    39. Acts like we're friends when we don't even know each other
    40. Has to be right
    41. Intimidates me
    42. Doesn't care about my feelings
    43. Avoids me
    44. Won't participate
    45. Cannot make decisions
    46. Is dependent on me
    47. Humors me
    48. Interrupts
    49. Is bossy
    50. Expects the worst out of me
    51. Never gets in touch with me
    52. Is really shy
    53. Is very passive
    54. Believes everything I say
    55. Believes I can do no wrong
    56. Tries to show me how to do things
    57. Always puts themselves first
    58. Is mean-spirited
    59. Pulls away from me
    60. Refuses to lead
    61. Is easily controlled
    62. Is soft
    63. Is clingy
    64. Takes control
    
    The ISC currently has 1 normative data set(s):
    1. 649 American college students
    Hopwood et al. (2011)
    <https://doi.org/10.1111/j.1467-6494.2011.00696.x>

---

    ISC: Interpersonal Sensitivities Circumplex
    64 items, 8 scales, 1 normative data sets
    Hopwood et al. (2011)
    <https://doi.org/10.1111/j.1467-6494.2011.00696.x>

---

    ISC: Interpersonal Sensitivities Circumplex
    64 items, 8 scales, 1 normative data sets
    Hopwood et al. (2011)
    <https://doi.org/10.1111/j.1467-6494.2011.00696.x>
    
    The ISC contains 8 circumplex scales.
    PA: Sensitive to Control (90 degrees)
    BC: Sensitive to Antagonism (135 degrees)
    DE: Sensitive to Remoteness (180 degrees)
    FG: Sensitive to Timidity (225 degrees)
    HI: Sensitive to Passivity (270 degrees)
    JK: Sensitive to Dependence (315 degrees)
    LM: Sensitive to Affection (360 degrees)
    NO: Sensitive to Attention-Seeking (45 degrees)

---

    ISC: Interpersonal Sensitivities Circumplex
    64 items, 8 scales, 1 normative data sets
    Hopwood et al. (2011)
    <https://doi.org/10.1111/j.1467-6494.2011.00696.x>
    
    The ISC is rated using the following 8-point scale.
    1. Not at all, never bothers me
    2. Very little, rarely bothers me
    3. A little, occasionally bothers me
    4. Slightly, bothers me less than half the time
    5. Somewhat, bothers me more than half of the time
    6. Quite, bothers me frequently
    7. Very much, bothers me most of the time
    8. Extremely, always bothers me

---

    ISC: Interpersonal Sensitivities Circumplex
    64 items, 8 scales, 1 normative data sets
    Hopwood et al. (2011)
    <https://doi.org/10.1111/j.1467-6494.2011.00696.x>
    
    The ISC contains 64 items (open-access):
    Prefix: It bothers me when a person...
    1. Thinks they are my boss
    2. Disregards my feelings
    3. Doesn't show any feelings
    4. Won't engage in conversation
    5. Cannot assert themselves
    6. Avoids conflict at all costs
    7. Wants to spend lots of time with me
    8. Doesn't respect my privacy
    9. Orders me around
    10. Doesn't respond to me
    11. Doesn't want to be friends
    12. Is aloof
    13. Doesn't have a backbone
    14. Always needs support
    15. Tries to get close to me really fast
    16. Talks themselves up
    17. Talks down to me
    18. Is hostile
    19. Walls themselves off from me
    20. Acts like a wallflower
    21. Is weak
    22. Always seems to need my help
    23. Tells me they love me
    24. Shows off
    25. Will do anything to get what they want
    26. Is deceitful
    27. Won't share their feelings with me
    28. Won't step up to the plate
    29. Allows themselves to be dominated by others
    30. Acts helpless
    31. Expresses concern about me
    32. Talks about themselves
    33. Has to call the shots
    34. Mistrusts me
    35. Is unresponsive
    36. Doesn't share their ideas
    37. Is always submissive
    38. Laughs at all my jokes
    39. Acts like we're friends when we don't even know each other
    40. Has to be right
    41. Intimidates me
    42. Doesn't care about my feelings
    43. Avoids me
    44. Won't participate
    45. Cannot make decisions
    46. Is dependent on me
    47. Humors me
    48. Interrupts
    49. Is bossy
    50. Expects the worst out of me
    51. Never gets in touch with me
    52. Is really shy
    53. Is very passive
    54. Believes everything I say
    55. Believes I can do no wrong
    56. Tries to show me how to do things
    57. Always puts themselves first
    58. Is mean-spirited
    59. Pulls away from me
    60. Refuses to lead
    61. Is easily controlled
    62. Is soft
    63. Is clingy
    64. Takes control

---

    ISC: Interpersonal Sensitivities Circumplex
    64 items, 8 scales, 1 normative data sets
    Hopwood et al. (2011)
    <https://doi.org/10.1111/j.1467-6494.2011.00696.x>
    
    The ISC currently has 1 normative data set(s):
    1. 649 American college students
    Hopwood et al. (2011)
    <https://doi.org/10.1111/j.1467-6494.2011.00696.x>

# The sub-summary functions produce the expected output

    The ISC contains 8 circumplex scales.
    PA: Sensitive to Control (90 degrees)
    BC: Sensitive to Antagonism (135 degrees)
    DE: Sensitive to Remoteness (180 degrees)
    FG: Sensitive to Timidity (225 degrees)
    HI: Sensitive to Passivity (270 degrees)
    JK: Sensitive to Dependence (315 degrees)
    LM: Sensitive to Affection (360 degrees)
    NO: Sensitive to Attention-Seeking (45 degrees)

---

    The ISC contains 8 circumplex scales.
    PA: Sensitive to Control (90 degrees)
        1. Thinks they are my boss
        9. Orders me around
        17. Talks down to me
        25. Will do anything to get what they want
        33. Has to call the shots
        41. Intimidates me
        49. Is bossy
        57. Always puts themselves first
    BC: Sensitive to Antagonism (135 degrees)
        2. Disregards my feelings
        10. Doesn't respond to me
        18. Is hostile
        26. Is deceitful
        34. Mistrusts me
        42. Doesn't care about my feelings
        50. Expects the worst out of me
        58. Is mean-spirited
    DE: Sensitive to Remoteness (180 degrees)
        3. Doesn't show any feelings
        11. Doesn't want to be friends
        19. Walls themselves off from me
        27. Won't share their feelings with me
        35. Is unresponsive
        43. Avoids me
        51. Never gets in touch with me
        59. Pulls away from me
    FG: Sensitive to Timidity (225 degrees)
        4. Won't engage in conversation
        12. Is aloof
        20. Acts like a wallflower
        28. Won't step up to the plate
        36. Doesn't share their ideas
        44. Won't participate
        52. Is really shy
        60. Refuses to lead
    HI: Sensitive to Passivity (270 degrees)
        5. Cannot assert themselves
        13. Doesn't have a backbone
        21. Is weak
        29. Allows themselves to be dominated by others
        37. Is always submissive
        45. Cannot make decisions
        53. Is very passive
        61. Is easily controlled
    JK: Sensitive to Dependence (315 degrees)
        6. Avoids conflict at all costs
        14. Always needs support
        22. Always seems to need my help
        30. Acts helpless
        38. Laughs at all my jokes
        46. Is dependent on me
        54. Believes everything I say
        62. Is soft
    LM: Sensitive to Affection (360 degrees)
        7. Wants to spend lots of time with me
        15. Tries to get close to me really fast
        23. Tells me they love me
        31. Expresses concern about me
        39. Acts like we're friends when we don't even know each other
        47. Humors me
        55. Believes I can do no wrong
        63. Is clingy
    NO: Sensitive to Attention-Seeking (45 degrees)
        8. Doesn't respect my privacy
        16. Talks themselves up
        24. Shows off
        32. Talks about themselves
        40. Has to be right
        48. Interrupts
        56. Tries to show me how to do things
        64. Takes control

---

    The ISC contains 64 items (open-access):
    Prefix: It bothers me when a person...
    1. Thinks they are my boss
    2. Disregards my feelings
    3. Doesn't show any feelings
    4. Won't engage in conversation
    5. Cannot assert themselves
    6. Avoids conflict at all costs
    7. Wants to spend lots of time with me
    8. Doesn't respect my privacy
    9. Orders me around
    10. Doesn't respond to me
    11. Doesn't want to be friends
    12. Is aloof
    13. Doesn't have a backbone
    14. Always needs support
    15. Tries to get close to me really fast
    16. Talks themselves up
    17. Talks down to me
    18. Is hostile
    19. Walls themselves off from me
    20. Acts like a wallflower
    21. Is weak
    22. Always seems to need my help
    23. Tells me they love me
    24. Shows off
    25. Will do anything to get what they want
    26. Is deceitful
    27. Won't share their feelings with me
    28. Won't step up to the plate
    29. Allows themselves to be dominated by others
    30. Acts helpless
    31. Expresses concern about me
    32. Talks about themselves
    33. Has to call the shots
    34. Mistrusts me
    35. Is unresponsive
    36. Doesn't share their ideas
    37. Is always submissive
    38. Laughs at all my jokes
    39. Acts like we're friends when we don't even know each other
    40. Has to be right
    41. Intimidates me
    42. Doesn't care about my feelings
    43. Avoids me
    44. Won't participate
    45. Cannot make decisions
    46. Is dependent on me
    47. Humors me
    48. Interrupts
    49. Is bossy
    50. Expects the worst out of me
    51. Never gets in touch with me
    52. Is really shy
    53. Is very passive
    54. Believes everything I say
    55. Believes I can do no wrong
    56. Tries to show me how to do things
    57. Always puts themselves first
    58. Is mean-spirited
    59. Pulls away from me
    60. Refuses to lead
    61. Is easily controlled
    62. Is soft
    63. Is clingy
    64. Takes control

---

    The ISC is rated using the following 8-point scale.
    1. Not at all, never bothers me
    2. Very little, rarely bothers me
    3. A little, occasionally bothers me
    4. Slightly, bothers me less than half the time
    5. Somewhat, bothers me more than half of the time
    6. Quite, bothers me frequently
    7. Very much, bothers me most of the time
    8. Extremely, always bothers me

---

    The ISC currently has 1 normative data set(s):
    1. 649 American college students
    Hopwood et al. (2011)
    <https://doi.org/10.1111/j.1467-6494.2011.00696.x>

# The instruments function produces the expected output

    The circumplex package currently includes 14 instruments:
     1. CAIS: Child and Adolescent Interpersonal Survey (cais)
     2. CSIE: Circumplex Scales of Interpersonal Efficacy (csie)
     3. CSIG: Circumplex Scales of Intergroup Goals (csig)
     4. CSIP: Circumplex Scales of Interpersonal Problems (csip)
     5. CSIV: Circumplex Scales of Interpersonal Values (csiv)
     6. IEI: Interpersonal Emotion Inventory (iei)
     7. IGI-CR: Interpersonal Goals Inventory for Children, Revised Version (igicr)
     8. IIP-32: Inventory of Interpersonal Problems, Brief Version (iip32)
     9. IIP-64: Inventory of Interpersonal Problems (iip64)
     10. IIP-SC: Inventory of Interpersonal Problems, Short Circumplex (iipsc)
     11. IIS-32: Inventory of Interpersonal Strengths, Brief Version (iis32)
     12. IIS-64: Inventory of Interpersonal Strengths (iis64)
     13. IIT-C: Inventory of Influence Tactics Circumplex (iitc)
     14. IPIP-IPC: IPIP Interpersonal Circumplex (ipipipc)
     15. ISC: Interpersonal Sensitivities Circumplex (isc)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{ipipipc}
\alias{ipipipc}
\title{IPIP Interpersonal Circumplex}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Markey, P. M., & Markey, C. N. (2009). A brief assessment of the
interpersonal circumplex: The IPIP-IPC. \emph{Assessment, 16}(4), 352-361.

\doi{10.1177/1073191109340382}
}
\usage{
ipipipc
}
\description{
Information about the IPIP Interpersonal Circumplex (IPIP-IPC).
}
\examples{
summary(ipipipc)
scales(ipipipc, items = TRUE)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{iipsc}
\alias{iipsc}
\title{Inventory of Interpersonal Problems, Short Circumplex}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Soldz, S., Budman, S., Demby, A., & Merry, J. (1995). A short form of
the Inventory of Interpersonal Problems Circumplex Scales. \emph{Assessment,
2}(1), 53-63.

\doi{10.1080/00223890802388665}
}
\usage{
iipsc
}
\description{
Information about the Inventory of Interpersonal Problems Short Circumplex
(IIP-SC).
}
\examples{
summary(iipsc)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidying_functions.R
\name{self_standardize}
\alias{self_standardize}
\title{Standardize circumplex scales using sample data}
\usage{
self_standardize(
  data,
  scales,
  na.rm = TRUE,
  prefix = "",
  suffix = "_z",
  append = TRUE
)
}
\arguments{
\item{data}{Required. A data frame or matrix containing at least circumplex
scales.}

\item{scales}{Required. A character vector containing the column names, or a
numeric vector containing the column indexes, for the variables (scale
scores) to be standardized.}

\item{na.rm}{Optional. A logical that determines whether to remove missing
values from scales when calculating the means and SDs used for
standardization (default = TRUE).}

\item{prefix}{Optional. A string to include at the beginning of the newly
calculated scale variables' names, before the scale name and \code{suffix}
(default = "").}

\item{suffix}{Optional. A string to include at the end of the newly
calculated scale variables' names, after the scale name and \code{prefix}
(default = "_z").}

\item{append}{Optional. A logical that determines whether the calculated
standardized scores should be added as columns to \code{data} in the output or
the standardized scores alone should be output (default = TRUE).}
}
\value{
A data frame that contains the self-standardized versions of
\code{scales}.
}
\description{
Take in a data frame containing circumplex scales (or items) and return that
same data frame with each specified variable transformed into standard scores
(i.e., z-scores) based on observed means and SDs.
}
\examples{
self_standardize(aw2009, scales = 1:8)
}
\seealso{
Other tidying functions: 
\code{\link{ipsatize}()},
\code{\link{norm_standardize}()},
\code{\link{score}()}
}
\concept{tidying functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{iei}
\alias{iei}
\title{Interpersonal Emotion Inventory}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Horner, M. S., Locke, K. D., & Hulsey, T. L. (in press). Assessing
affective dimensions of the interpersonal circumplex: development and
validation of the interpersonal emotion inventory. \emph{Journal of Personality
Assessment}. Advanced online publication.

\doi{10.1080/00223891.2024.2400266}

\url{https://kennethlocke.org/IEI/IEI.html}
}
\usage{
iei
}
\description{
Information about the Interpersonal Emotion Inventory (IEI).
}
\examples{
summary(iei)
scales(iei, items = TRUE)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{ggsave}
\alias{ggsave}
\title{Save a ggplot with sensible defaults}
\description{
See \code{ggplot2} for details.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_oop.R
\name{instruments}
\alias{instruments}
\title{List all available instruments}
\usage{
instruments()
}
\description{
The circumplex package includes information about numerous circumplex
instruments including instructions for scoring and standardizing items.
Individual instruments can be loaded using the \code{instrument} function.
}
\examples{
instruments()
}
\seealso{
Other instrument functions: 
\code{\link{anchors}()},
\code{\link{items}()},
\code{\link{norms}()},
\code{\link{scales}()}
}
\concept{instrument functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience_functions.R
\name{quadrants}
\alias{quadrants}
\title{Angular displacements for quadrant circumplex scales}
\usage{
quadrants()
}
\value{
A numeric vector with eight elements, each corresponding to the
angular displacement (in degrees) of a subscale, in the following order:
BC, FG, JK, NO.
}
\description{
Return a vector of angular displacements, in degrees, for four equally spaced
circumplex scales corresponding to the circumplex quadrants. Can be passed to
the \code{angles} parameter of other functions in this package.
}
\examples{
quadrants()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{iip64}
\alias{iip64}
\title{Inventory of Interpersonal Problems}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Horowitz, L. M., Alden, L. E., Wiggins, J. S., & Pincus, A. L.
(2000). \emph{IIP-64/IIP-32 professional manual.} San Antonio, TX: The
Psychological Corporation.

\url{https://www.mindgarden.com/113-inventory-of-interpersonal-problems}
}
\usage{
iip64
}
\description{
Information about the Inventory of Interpersonal Problems (IIP-64). Note
that, although we have permission to provide some information about the
IIP-64, Mind Garden Inc. has exclusive rights to distribute it in full.
}
\examples{
summary(iip64)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidying_functions.R
\name{ipsatize}
\alias{ipsatize}
\title{Ipsatize circumplex items using deviation scoring across variables}
\usage{
ipsatize(data, items, na.rm = TRUE, prefix = "", suffix = "_i", append = TRUE)
}
\arguments{
\item{data}{Required. A data frame or matrix containing at least circumplex
scales.}

\item{items}{Required. A character vector containing the column names, or a
numeric vector containing column indexes, of item variables in \code{data} to be
ipsatized.}

\item{na.rm}{Optional. A logical that determines whether missing values
should be ignored during the calculation of the mean during ipsatization
(default = TRUE).}

\item{prefix}{Optional. A string that will be added to the start of each
\code{items} name in the output (default = "").}

\item{suffix}{Optional. A string that will be added to the end of each
\code{items} name in the output (default = "_i").}

\item{append}{Optional. A logical that determines whether to append the
ipsatized scores to \code{data} in the output or just return the ipsatized
scores alone (default = TRUE).}
}
\value{
A data frame that matches \code{data} except that the variables specified
in \code{items} have been rescored using ipsatization.
}
\description{
Rescore each circumplex item using deviation scoring across variables. In
other words, subtract each observation's mean response from each response.
This effectively removes the presence of a general factor, which can make
certain circumplex fit analyses more powerful.
}
\examples{
data("raw_iipsc")
ipsatize(raw_iipsc, items = 1:32)
ipsatize(raw_iipsc, items = sprintf("IIP\%02d", 1:32))
}
\seealso{
Other tidying functions: 
\code{\link{norm_standardize}()},
\code{\link{score}()},
\code{\link{self_standardize}()}
}
\concept{tidying functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience_functions.R
\name{PANO}
\alias{PANO}
\title{Two-letter abbreviations for octant circumplex scales}
\usage{
PANO(case = "upper")
}
\arguments{
\item{case}{An optional string the determines whether the abbreviations should be
in uppercase or lowercase. (default = "upper")}
}
\value{
A character vector with eight elements, each corresponding to the
abbreviation of an octant subscale: PA, BC, DE, FG, HI, JK, LM, NO.
}
\description{
Return a vector of abbreviations for octant circumplex scales, from PA to NO.
}
\examples{
PANO()
PANO(case = "lower")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ssm_plot.R
\name{ssm_plot_contrast}
\alias{ssm_plot_contrast}
\title{Create a Difference Plot of SSM Contrast Results}
\usage{
ssm_plot_contrast(
  ssm_object,
  drop_xy = FALSE,
  sig_color = "#fc8d62",
  ns_color = "white",
  linesize = 1.25,
  fontsize = 12,
  ...
)
}
\arguments{
\item{ssm_object}{Required. The results output of \code{ssm_analyze()}.}

\item{drop_xy}{A logical determining whether the X-Value and Y-Value
parameters should be removed from the plot (default = FALSE).}

\item{sig_color}{Optional. A string corresponding to the color to use to
denote significant contrasts (default = "#fc8d62").}

\item{ns_color}{Optional. A string corresponding to the color to use to
denote non-significant contrasts (default = "white").}

\item{linesize}{Optional. A positive number corresponding to the size of the
point range elements in mm (default = 1.5).}

\item{fontsize}{Optional. A positive number corresponding to the size of the
axis labels, numbers, and facet headings in pt (default = 12).}

\item{...}{Additional arguments will be ignored.}
}
\value{
A ggplot variable containing difference point-ranges faceted by SSM
parameter. An interval that does not contain the value of zero has p<.05.
}
\description{
Take in the results of a Structural Summary Method analysis with pairwise
contrasts and plot the point and interval estimates for each parameter's
contrast (e.g., between groups or measures).
}
\examples{
\donttest{
data("jz2017")
res <- ssm_analyze(
  jz2017,
  scales = 2:9,
  measures = c("NARPD", "ASPD"),
  contrast = TRUE
)
ssm_plot_contrast(res)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{iis64}
\alias{iis64}
\title{Inventory of Interpersonal Strengths}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Hatcher, R. L., & Rogers, D. T. (2009). Development and validation of
a measure of interpersonal strengths: The Inventory of Interpersonal
Strengths. \emph{Psychological Assessment, 21}(4), 554-569.

\doi{10.1037/a0017269}
}
\usage{
iis64
}
\description{
Information about the Inventory of Interpersonal Strengths (IIS-64).
}
\examples{
summary(iis64)
scales(iis64, items = TRUE)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ssm_analysis.R
\name{ssm_analyze}
\alias{ssm_analyze}
\title{Perform analyses using the Structural Summary Method}
\usage{
ssm_analyze(
  data,
  scales,
  angles = octants(),
  measures = NULL,
  grouping = NULL,
  contrast = FALSE,
  boots = 2000,
  interval = 0.95,
  listwise = TRUE,
  measures_labels = NULL
)
}
\arguments{
\item{data}{Required. A data frame containing at least circumplex scales.}

\item{scales}{Required. A character vector of column names, or a numeric
vector of column indexes, from \code{data} that contains the circumplex scale
scores to be analyzed.}

\item{angles}{Optional. A numeric vector containing the angular displacement
of each circumplex scale included in \code{scales} (in degrees). (default =
\code{octants()}).}

\item{measures}{Optional. Either \code{NULL} or a character vector of column names
from \code{data} that contains one or more variables to be correlated with the
circumplex scales and analyzed using correlation-based SSM analyses.}

\item{grouping}{Optional. Either \code{NULL} or a string that contains the column
name from \code{data} of the variable that indicates the group membership of
each observation.}

\item{contrast}{Optional. A logical indicating whether to output the
difference between two measures' or two groups' SSM parameters. Can only be
set to TRUE when there are exactly two measures and one group, one measure
and two groups, or no measures and two groups (default = FALSE).}

\item{boots}{Optional. A single positive whole number indicating how many
bootstrap resamples to use when estimating the confidence intervals
(default = 2000).}

\item{interval}{Optional. A single positive number between 0 and 1
(exclusive) that indicates what confidence level to use when estimating the
confidence intervals (default = 0.95).}

\item{listwise}{Optional. A logical indicating whether missing values should
be handled by listwise deletion (TRUE) or pairwise deletion (FALSE). Note
that pairwise deletion may result in different missing data patterns in
each bootstrap resample and is slower to compute (default = TRUE).}

\item{measures_labels}{Optional. Either \code{NULL} or a character vector
providing a label for each measure provided in \code{measures} (in the same
order) to appear in the results as well as tables and plots derived from
the results.}
}
\value{
A list containing the results and description of the analysis.
\item{results}{A data frame with the SSM parameter estimates}
\item{details}{A list with the number of bootstrap resamples (boots),
the confidence interval percentage level (interval), and the angular
displacement of scales (angles)}
\item{call}{A language object containing the function call that created
this object}
\item{scores}{A data frame containing the mean scale scores} \item{type}{A
string indicating what type of SSM analysis was done}
}
\description{
Calculate SSM parameters with bootstrapped confidence intervals for a variety
of different analysis types. Depending on what arguments are supplied, either
mean-based or correlation-based analyses will be performed, one or more
groups will be used to stratify the data, and contrasts between groups or
measures will be calculated.
}
\examples{
# Load example data
data("jz2017")

# Single-group mean-based SSM
ssm_analyze(
  jz2017,
  scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO")
)

# Single-group correlation-based SSM
ssm_analyze(
  jz2017,
  scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
  measures = c("NARPD", "ASPD")
)
\donttest{
# Multiple-group mean-based SSM
ssm_analyze(
  jz2017,
  scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
  grouping = "Gender"
)

# Multiple-group mean-based SSM with contrast
ssm_analyze(
  jz2017,
  scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
  grouping = "Gender",
  contrast = TRUE
)

# Single-group correlation-based SSM with contrast
ssm_analyze(
  jz2017,
  scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
  measures = c("NARPD", "ASPD"),
  contrast = TRUE
)

# Multiple-group correlation-based SSM
ssm_analyze(
  jz2017,
  scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
  measures = "NARPD",
  grouping = "Gender"
)

# Multiple-group correlation-based SSM with contrast
ssm_analyze(
  jz2017,
  scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO"),
  measures = "NARPD",
  grouping = "Gender",
  contrast = TRUE
)
}

}
\seealso{
Other ssm functions: 
\code{\link{ssm_parameters}()},
\code{\link{ssm_score}()},
\code{\link{ssm_table}()}

Other analysis functions: 
\code{\link{ssm_parameters}()},
\code{\link{ssm_score}()}
}
\concept{analysis functions}
\concept{ssm functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_oop.R
\name{scales}
\alias{scales}
\title{Display the scales of a circumplex instrument}
\usage{
scales(x, items = FALSE)
}
\arguments{
\item{x}{Required. An object of the instrument class.}

\item{items}{Optional. A logical determining whether the items for each scale
should be displayed below its other information (default = FALSE).}
}
\value{
The same input object. Prints text to console.
}
\description{
Display the scales of a circumplex instrument including the total number of
scales and each scale's abbreviation, hypothetical angle, and text label.
}
\examples{
scales(csip)
scales(csip, items = TRUE)
}
\seealso{
Other instrument functions: 
\code{\link{anchors}()},
\code{\link{instruments}()},
\code{\link{items}()},
\code{\link{norms}()}
}
\concept{instrument functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_oop.R
\name{anchors}
\alias{anchors}
\title{Display the anchors of a circumplex instrument}
\usage{
anchors(x)
}
\arguments{
\item{x}{Required. An object of the instrument class.}
}
\value{
The same input object. Prints text to console.
}
\description{
Display the anchors of a circumplex instrument including the total number of
anchors and each anchor's numerical value and text label. Anchors are the
response options that respondants select from (e.g., 0 = No, 1 = Yes).
}
\examples{
anchors(csip)
}
\seealso{
Other instrument functions: 
\code{\link{instruments}()},
\code{\link{items}()},
\code{\link{norms}()},
\code{\link{scales}()}
}
\concept{instrument functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_data.R
\docType{data}
\name{aw2009}
\alias{aw2009}
\title{Standardized octant scores on hypothetical circumplex scales}
\format{
A data frame with 5 observations and 8 variables:
\describe{
\item{PA}{circumplex scale at 90 degrees}
\item{BC}{circumplex scale at 135 degrees}
\item{DE}{circumplex scale at 180 degrees}
\item{FG}{circumplex scale at 225 degrees}
\item{HI}{circumplex scale at 270 degrees}
\item{JK}{circumplex scale at 315 degrees}
\item{LM}{circumplex scale at 360 degrees}
\item{NO}{circumplex scale at 45 degrees}
}
}
\source{
\doi{10.1080/00223890902935696}
}
\usage{
aw2009
}
\description{
A small example dataset containing standardized scores on eight hypothetical
circumplex scales. Taken from Wright, Pincus, Conroy, & Hilsenroth (2009).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_data.R
\docType{data}
\name{raw_iipsc}
\alias{raw_iipsc}
\title{Raw item responses on real circumplex scales}
\format{
A data frame with 10 observations and 32 variables.
}
\usage{
raw_iipsc
}
\description{
A small example dataset containing raw item responses on the Inventory of
Interpersonal Problems, Short Circumplex (IIP-SC). This data set is useful
for testing functions that operate on item-level data.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ssm_table.R
\name{html_render}
\alias{html_render}
\title{Format and render data frame as HTML table}
\usage{
html_render(df, caption = NULL, align = "l", ...)
}
\arguments{
\item{df}{A data frame to be rendered as an HTML table.}

\item{caption}{A string to be displayed above the table.}

\item{align}{A string indicating the alignment of the cells (default = "l").}

\item{...}{Other arguments to pass to \code{htmlTable}.}
}
\value{
HTML syntax for the \code{df} table.
}
\description{
Format a data frame as an HTML table and render it to the web viewer.
}
\seealso{
Other table functions: 
\code{\link{ssm_table}()}
}
\concept{table functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{csig}
\alias{csig}
\title{Circumplex Scales of Intergroup Goals}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Locke, K. D. (2014). Circumplex scales of intergroup goals: An
interpersonal circle model of goals for interactions between groups.
\emph{Personality and Social Psychology Bulletin, 40}(4), 433-449.

\url{https://kennethlocke.org/CSIG/CSIG.html}
}
\usage{
csig
}
\description{
Information about the Circumplex Scales of Intergroup Goals (CSIG).
}
\examples{
summary(csig)
scales(csig, items = TRUE)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ssm_plot.R
\name{ssm_plot_circle}
\alias{ssm_plot_circle}
\title{Create a Circular Plot of SSM Results}
\usage{
ssm_plot_circle(
  ssm_object,
  amax = NULL,
  legend_font_size = 12,
  scale_font_size = 12,
  drop_lowfit = FALSE,
  repel = FALSE,
  angle_labels = NULL,
  palette = "Set2",
  vary_shapes = FALSE,
  ...
)
}
\arguments{
\item{ssm_object}{Required. The output of \code{ssm_analyze()}.}

\item{amax}{A positive real number corresponding to the radius of the circle.
It is used to scale the amplitude values and will determine which amplitude
labels are drawn.}

\item{legend_font_size}{A positive real number corresponding to the size (in
pt) of the text labels in the legend (default = 12).}

\item{scale_font_size}{A positive real number corresponding to the size (in
pt) of the text labels for the amplitude and displacement scales (default =
12).}

\item{drop_lowfit}{A logical determining whether profiles with low model fit
(<.70) should be omitted or plotted with dashed borders (default = FALSE).}

\item{repel}{An experimental argument for plotting text labels instead of
colors.}

\item{angle_labels}{A character vector specifying text labels to plot around
the circle for each scale. Can also specify NULL to default to numerical
angle labels or a vector of empty strings ("") to hide the labels. If not
NULL, must have the same length and ordering as the \code{angles} argument to
\code{ssm_analyze()}. (default = NULL)}

\item{palette}{A string corresponding to the palette to be used from
ColorBrewer for the color and fill aesthetics. If set to NULL, all points
will appear blue and no legend will be there (useful for showing the
coverage of a high number of variables).}

\item{vary_shapes}{A logical determining whether profiles should each get
their own shape or vary only by fill color. This only works when the number
of profiles is five or less. (default = FALSE)}

\item{...}{Currently ignored.}
}
\value{
A ggplot variable containing a completed circular plot.
}
\description{
Take in the results of a Structural Summary Method analysis and plot the
point and interval estimate for each row (e.g., group or measure) in a
circular space quantified by displacement and amplitude.
}
\examples{
\donttest{
data("jz2017")
res <- ssm_analyze(
  jz2017,
  scales = 2:9,
  measures = c("NARPD", "ASPD")
)
ssm_plot_circle(res)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_oop.R
\name{items}
\alias{items}
\title{Display the items of a circumplex instrument}
\usage{
items(x)
}
\arguments{
\item{x}{Required. An object of the instrument class.}
}
\value{
The same input object. Prints text to console.
}
\description{
Display the items of a circumplex instrument including the total number of
items and each item's number and text. The item ordering/numbering displayed
here is the same ordering/numbering assumed by the \code{score()} function.
}
\examples{
items(csip)
}
\seealso{
Other instrument functions: 
\code{\link{anchors}()},
\code{\link{instruments}()},
\code{\link{norms}()},
\code{\link{scales}()}
}
\concept{instrument functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{iitc}
\alias{iitc}
\title{Inventory of Influence Tactics Circumplex}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Bliton, C. F., & Pincus, A. L. (in press). Construction and
Validation of the Interpersonal Influence Tactics Circumplex (IIT-C)
Scales. \emph{Assessment}.

\doi{10.1177/1073191119864661}
}
\usage{
iitc
}
\description{
Information about the Inventory of Influence Tactics Circumplex (IIT-C).
}
\examples{
summary(iitc)
scales(iitc, items = TRUE)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_data.R
\docType{data}
\name{jz2017}
\alias{jz2017}
\title{Raw octant scores on real circumplex scales with covariates}
\format{
A data frame with 1166 observations and 19 variables:
\describe{
\item{Gender}{Self-reported Gender}
\item{PA}{Domineering Problems (IIP-SC) 90 degrees}
\item{BC}{Vindictive Problems (IIP-SC) 135 degrees}
\item{DE}{Cold Problems (IIP-SC) 180 degrees}
\item{FG}{Socially Avoidant Problems (IIP-SC) 225 degrees}
\item{HI}{Nonassertive Problems(IIP-SC) 270 degrees}
\item{JK}{Easily Exploited Problems (IIP-SC) 315 degrees}
\item{LM}{Overly Nurturant Problems (IIP-SC) 360 degrees}
\item{NO}{Intrusive Problems (IIP-SC) 45 degrees}
\item{PARPD}{Paranoid PD Symptoms (PDQ-4+)}
\item{SCZPD}{Schizoid PD Symptoms (PDQ-4+)}
\item{SZTPD}{Schizotypal PD Symptoms (PDQ-4+)}
\item{ASPD}{Antisocial PD Symptoms (PDQ-4+)}
\item{BORPD}{Borderline PD Symptoms (PDQ-4+)}
\item{HISPD}{Histrionic PD Symptoms (PDQ-4+)}
\item{NARPD}{Narcissistic PD Symptoms (PDQ-4+)}
\item{AVPD}{Avoidant PD Symptoms (PDQ-4+)}
\item{DPNPD}{Dependent PD Symptoms (PDQ-4+)}
\item{OCPD}{Obsessive-Compulsive PD Symptoms (PDQ-4+)}
}
}
\source{
\doi{10.1177/1073191115621795}
}
\usage{
jz2017
}
\description{
A large example dataset containing gender, raw mean scores on the Inventory
of Interpersonal Problems - Short Circumplex (IIP-SC), and raw sum scores on
the Personality Diagnostic Questionnaire - 4th Edition Plus (PDQ-4+).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience_functions.R
\name{poles}
\alias{poles}
\title{Angular displacements for pole circumplex scales}
\usage{
poles()
}
\value{
A numeric vector with four elements, each corresponding to the
angular displacement (in degrees) of a subscale, in the following order:
PA, DE, HI, LM.
}
\description{
Return a vector of angular displacements, in degrees, for four equally spaced
circumplex scales corresponding to the circumplex poles. Can be passed to the
\code{angles} parameter of other functions in this package.
}
\examples{
poles()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ssm_table.R
\name{ssm_table}
\alias{ssm_table}
\title{Create HTML table from SSM results or contrasts}
\usage{
ssm_table(ssm_object, caption = NULL, drop_xy = FALSE, render = TRUE)
}
\arguments{
\item{ssm_object}{Required. The results output of \code{ssm_analyze()}.}

\item{caption}{A string to be displayed above the table (default = NULL).}

\item{drop_xy}{A logical indicating whether the x-value and y-value parameters
should be omitted from the output (default = FALSE).}

\item{render}{A logical indicating whether the table should be displayed in
the RStudio viewer or web browser (default = TRUE).}
}
\value{
A data frame containing the information for the HTML table. As a
side-effect, may also output the HTML table to the web viewer.
}
\description{
Take in the results of an SSM analysis and return an HTML table with the
desired formatting.
}
\examples{
\donttest{
# Load example data
data("jz2017")

# Create table of profile results
res <- ssm_analyze(
  jz2017,
  scales = 2:9,
  measures = c("NARPD", "ASPD")
)
ssm_table(res)

# Create table of contrast results
res <- ssm_analyze(
  jz2017,
  scales = 2:9,
  measures = c("NARPD", "ASPD"), 
  contrast = TRUE
)
ssm_table(res)
}

}
\seealso{
Other ssm functions: 
\code{\link{ssm_analyze}()},
\code{\link{ssm_parameters}()},
\code{\link{ssm_score}()}

Other table functions: 
\code{\link{html_render}()}
}
\concept{ssm functions}
\concept{table functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{cais}
\alias{cais}
\title{Child and Adolescent Interpersonal Survey}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Sodano, S. M., & Tracey, T. J. G. (2006). Interpersonal traits in
childhood: Development of the Child and Adolescent Interpersonal Survey.
\emph{Journal of Personality Assessment, 87}(3), 317–329.

\doi{10.1207/s15327752jpa8703_12}
}
\usage{
cais
}
\description{
Information about the Child and Adolescent Interpersonal Survey (CAIS).
}
\examples{
summary(cais)
scales(cais, items = TRUE)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{csie}
\alias{csie}
\title{Circumplex Scales of Interpersonal Efficacy}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Locke, K. D., & Sadler, P. (2007). Self-efficacy, values, and
complementarity in dyadic interactions: Integrating interpersonal and
social-cognitive theory. \emph{Personality and Social Psychology Bulletin,
33}(1), 94-109.

\url{https://kennethlocke.org/CSIE/CSIE.html}
}
\usage{
csie
}
\description{
Information about the Circumplex Scales of Interpersonal Efficacy (CSIE).
}
\examples{
summary(csie)
scales(csie, items = TRUE)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{igicr}
\alias{igicr}
\title{Interpersonal Goals Inventory for Children, Revised Version}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Trucco, E. M., Wright, A. G. C., & Colder, C. R. (2013). A revised
interpersonal circumplex inventory of children’s social goals. \emph{Assessment,
20}(1), 98-113.

\doi{10.1177/1073191111411672}
}
\usage{
igicr
}
\description{
Information about the Interpersonal Goals Inventory for Children, Revised
Version (IGI-CR).
}
\examples{
summary(igicr)
scales(igicr, items = TRUE)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidying_functions.R
\name{norm_standardize}
\alias{norm_standardize}
\title{Standardize circumplex scales using normative data}
\usage{
norm_standardize(
  data,
  scales,
  angles = octants(),
  instrument,
  sample = 1,
  prefix = "",
  suffix = "_z",
  append = TRUE
)
}
\arguments{
\item{data}{Required. A data frame or matrix containing at least circumplex
scales.}

\item{scales}{Required. A character vector containing the column names, or a
numeric vector containing the column indexes, for the variables (scale
scores) to be standardized.}

\item{angles}{Required. A numeric vector containing the angular displacement
of each circumplex scale included in \code{scales} (in degrees). Can use the
\code{octants()}, \code{poles()}, or \code{quadrants()} convenience functions.}

\item{instrument}{Required. An instrument object from the package. To see the
available circumplex instruments, see \code{instruments()}.}

\item{sample}{Required. An integer corresponding to the normative sample to
use in standardizing the scale scores (default = 1). See \code{?norms} to
see the normative samples available for an instrument.}

\item{prefix}{Optional. A string to include at the beginning of the newly
calculated scale variables' names, before the scale name and \code{suffix}
(default = "").}

\item{suffix}{Optional. A string to include at the end of the newly
calculated scale variables' names, after the scale name and \code{prefix}
(default = "_z").}

\item{append}{Optional. A logical that determines whether the calculated
standardized scores should be added as columns to \code{data} in the output or
the standardized scores alone should be output (default = TRUE).}
}
\value{
A data frame that contains the norm-standardized versions of \code{scales}.
}
\description{
Take in a data frame containing circumplex scales, angle definitions for each
scale, and normative data (from the package or custom) and return that same
data frame with each specified circumplex scale transformed into standard
scores (i.e., z-scores) based on comparison to the normative data.
}
\examples{
data("jz2017")
norm_standardize(jz2017, scales = 2:9, instrument = iipsc, sample = 1)
}
\seealso{
Other tidying functions: 
\code{\link{ipsatize}()},
\code{\link{score}()},
\code{\link{self_standardize}()}
}
\concept{tidying functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_oop.R
\name{norms}
\alias{norms}
\title{Display the norms for a circumplex instrument}
\usage{
norms(x)
}
\arguments{
\item{x}{Required. An object of the instrument class.}
}
\value{
The same input object. Prints text to console.
}
\description{
Display the norms for a circumplex instrument including the total number of
normative data sets available and each data set's number, sample size,
population, and source reference and hyperlink. If another normative data set
exists that is not yet included in the package, please let us know.
}
\examples{
norms(csip)
}
\seealso{
Other instrument functions: 
\code{\link{anchors}()},
\code{\link{instruments}()},
\code{\link{items}()},
\code{\link{scales}()}
}
\concept{instrument functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convenience_functions.R
\name{octants}
\alias{octants}
\title{Angular displacements for octant circumplex scales}
\usage{
octants()
}
\value{
A numeric vector with eight elements, each corresponding to the
angular displacement (in degrees) of a subscale, in the following order:
PA, BC, DE, FG, HI, JK, LM, NO.
}
\description{
Return a vector of angular displacements, in degrees, for eight equally
spaced circumplex scales corresponding to the circumplex octants. Can be
passed to the \code{angles} parameter of other functions in this package.
}
\examples{
octants()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{csip}
\alias{csip}
\title{Circumplex Scales of Interpersonal Problems}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Boudreaux, M. J., Ozer, D. J., Oltmanns, T. F., & Wright, A. G. C.
(2018). Development and validation of the circumplex scales of
interpersonal problems. \emph{Psychological Assessment, 30}(5), 594-609.

\doi{10.1037/pas0000505}
}
\usage{
csip
}
\description{
Information about the Circumplex Scales of Interpersonal Problems (CSIP).
}
\examples{
summary(csip)
scales(csip, items = TRUE)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/circumplex-package.R
\docType{package}
\name{circumplex-package}
\alias{circumplex}
\alias{circumplex-package}
\title{Analysis and visualization of circumplex data}
\description{
circumplex provides functions for analyzing and visualizing circumplex data.
}
\details{
Its goal is to unify, modernize, and extend existing methods of working with
circumplex data. Its functions share an underlying design philosophy and
grammar. The three guiding principles it aspires to are:

\itemize{
\item \strong{Accessibility:} zero cost, open source, libre; works on many
platforms; easy to use across skill levels
\item \strong{Flexibility:} customizable by the user; extendable for other uses;
plays nicely with other packages
\item \strong{Consistency:} unit-tested quality control; firm naming conventions;
data flows between functions
}

To learn more about circumplex, start with the vignettes:
\code{browseVignettes(package = "circumplex")}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/jmgirard/circumplex}
  \item \url{http://circumplex.jmgirard.com/}
  \item Report bugs at \url{https://github.com/jmgirard/circumplex/issues}
}

}
\author{
\strong{Maintainer}: Jeffrey Girard \email{me@jmgirard.com} (\href{https://orcid.org/0000-0002-7359-3746}{ORCID})

Authors:
\itemize{
  \item Johannes Zimmermann (\href{https://orcid.org/0000-0001-6975-2356}{ORCID})
  \item Aidan Wright (\href{https://orcid.org/0000-0002-2369-0601}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidying_functions.R
\name{score}
\alias{score}
\title{Score circumplex scales from item responses}
\usage{
score(
  data,
  items,
  instrument,
  na.rm = TRUE,
  prefix = "",
  suffix = "",
  append = TRUE
)
}
\arguments{
\item{data}{Required. A data frame containing at least circumplex scales.}

\item{items}{Required. The variable names or column numbers for the variables
in \code{.data} that contain all the circumplex items from a single
circumplex measure, in ascending order from item 1 to item N.}

\item{instrument}{Required. An instrument object from the package. To see the
available circumplex instruments, use \code{instruments()}.}

\item{na.rm}{Optional. A logical that determines if missing values should be
omitted from the calculation of scores (default = TRUE). When set to TRUE,
scales with missing data are essentially calculated with mean imputation.}

\item{prefix}{Optional. A string to include at the beginning of the newly
calculated scale variables' names, before \code{Abbrev} from \code{key} and
\code{suffix} (default = "").}

\item{suffix}{Optional. A string to include at the end of the newly
calculated scale variables' names, after \code{Abbrev} from \code{key} and
\code{prefix} (default = "").}

\item{append}{Optional. A logical that determines whether the calculated
score variables will be appended to \code{data} or returned on their own
(default = TRUE).}
}
\value{
A data frame that matches \code{.data} except that new variables are
appended that contain mean scores on each variable included in \code{key}.
}
\description{
Calculate mean scores on circumplex scales from item responses by using a set
of scoring instructions, which may be loaded from the package or created as a
custom data frame.
}
\examples{
data("raw_iipsc")
score(raw_iipsc, items = 1:32, instrument = iipsc, prefix = "IIPSC_")
}
\seealso{
Other tidying functions: 
\code{\link{ipsatize}()},
\code{\link{norm_standardize}()},
\code{\link{self_standardize}()}
}
\concept{tidying functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{iis32}
\alias{iis32}
\title{Inventory of Interpersonal Strengths, Brief Version}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Hatcher, R. L., & Rogers, D. T. (2012). The IIS-32: A brief inventory
of interpersonal strengths. \emph{Journal of Personality Assessment, 94}(6),
638-646.

\doi{10.1080/00223891.2012.681818}
}
\usage{
iis32
}
\description{
Information about the Inventory of Interpersonal Strengths, Brief Version
(IIS-32).
}
\examples{
summary(iis32)
scales(iis32, items = TRUE)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ssm_plot.R
\name{ssm_plot_curve}
\alias{ssm_plot_curve}
\title{Create a Curve Plot of SSM Results}
\usage{
ssm_plot_curve(
  ssm_object,
  angle_labels = NULL,
  base_size = 11,
  drop_lowfit = FALSE,
  ...
)
}
\arguments{
\item{ssm_object}{Required. The results output of \code{ssm_analyze()}.}

\item{angle_labels}{Optional. Either NULL or a character vector that
determines the x-axis labels. If NULL, the labels will be the angle
numbers. If a character vector, must be the same length and in the same
order as the \code{angles} argument to \code{ssm_analyze()} (default = NULL).}

\item{base_size}{Optional. A positive number corresponding to the base font
size in pts (default = 11).}

\item{drop_lowfit}{Optional. A logical indicating whether to omit profiles with
low fit (<.70) or include them with dashed lines (default = FALSE).}

\item{...}{Additional arguments will be ignored.}
}
\value{
A ggplot object depicting the SSM curve(s) of each profile.
}
\description{
Take in the results of a Structural Summary Method analysis and plot the
scores by angle and the estimated SSM curve.
}
\examples{
\donttest{
data("jz2017")
res <- ssm_analyze(
  jz2017,
  scales = 2:9,
  measures = 10:13
)
ssm_plot_curve(res)
ssm_plot_curve(res, angle_lables = PANO())
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ssm_analysis.R
\name{ssm_parameters}
\alias{ssm_parameters}
\title{Calculate Structural Summary Method parameters for a set of scores}
\usage{
ssm_parameters(
  scores,
  angles = octants(),
  prefix = "",
  suffix = "",
  e_label = "Elev",
  x_label = "Xval",
  y_label = "Yval",
  a_label = "Ampl",
  d_label = "Disp",
  f_label = "Fit"
)
}
\arguments{
\item{scores}{Required. A numeric vector (or single row data frame)
containing one score for each of a set of circumplex scales.}

\item{angles}{Required. A numeric vector containing the angular displacement
of each circumplex scale included in \code{scores} (in degrees).}

\item{prefix}{Optional. A string to append to the beginning of all of the SSM
parameters' variable names (default = "").}

\item{suffix}{Optional. A string to append to the end of all of the SSM
parameters' variable names (default = "").}

\item{e_label}{Optional. A string representing the variable name of the SSM
elevation parameter (default = "Elev").}

\item{x_label}{Optional. A string representing the variable name of the SSM
x-value parameter (default = "Xval").}

\item{y_label}{Optional. A string representing the variable name of the SSM
y-value parameter (default = "Yval").}

\item{a_label}{Optional. A string representing the variable name of the SSM
amplitude parameter (default = "Ampl").}

\item{d_label}{Optional. A string representing the variable name of the SSM
displacement parameter (default = "Disp").}

\item{f_label}{Optional. A string representing the variable name of the SSM
fit or R-squared value (default = "Fit").}
}
\value{
A data frame containing the SSM parameters calculated from \code{scores}.
}
\description{
Calculate SSM parameters (without confidence intervals) for a set of scores
and generate a data frame with customizable labels for each parameter value.
This function requires the input to be a numeric vector (or coercable to one)
and returns only the parameters. See \code{\link{ssm_score}()} for a similar
function that calculates SSM parameters for each row of a data frame.
}
\examples{
# Manually enter octant scores
scores <- c(0.55, 0.58, 0.62, 0.76, 1.21, 1.21, 1.48, 0.90)
ssm_parameters(scores)

# Customize several of the labels
ssm_parameters(scores, x_label = "LOV", y_label = "DOM")

# Add a prefix to all labels
ssm_parameters(scores, prefix = "IIP_")

}
\seealso{
Other ssm functions: 
\code{\link{ssm_analyze}()},
\code{\link{ssm_score}()},
\code{\link{ssm_table}()}

Other analysis functions: 
\code{\link{ssm_analyze}()},
\code{\link{ssm_score}()}
}
\concept{analysis functions}
\concept{ssm functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ssm_analysis.R
\name{ssm_score}
\alias{ssm_score}
\title{Calculate SSM parameters by row and add results as new columns}
\usage{
ssm_score(data, scales, angles = octants(), append = TRUE, ...)
}
\arguments{
\item{data}{Required. A data frame containing at least circumplex scales.}

\item{scales}{Required. The variable names or column numbers for the
variables in \code{.data} that contain circumplex scales to be analyzed.}

\item{angles}{Required. A numeric vector containing the angular displacement
of each circumplex scale included in \code{scales} (in degrees).}

\item{append}{Optional. A logical indicating whether to append the output to
\code{data} or simply return the output (default = "TRUE").}

\item{...}{Optional. Additional parameters to pass to
\code{\link{ssm_parameters}()}, such as \code{prefix} and \code{suffix}.}
}
\value{
A data frame containing \code{.data} plus six additional columns
containing the SSM parameters (calculated rowwise).
}
\description{
Calculate the SSM parameters for each row of a data frame and add the results
as additional columns. This can be useful when the SSM is being used for the
description or visualization of individual data points rather than for
statistical inference on groups of data points.
}
\examples{
data("aw2009")
ssm_score(
  aw2009,
  scales = c("PA", "BC", "DE", "FG", "HI", "JK", "LM", "NO")
)

}
\seealso{
Other ssm functions: 
\code{\link{ssm_analyze}()},
\code{\link{ssm_parameters}()},
\code{\link{ssm_table}()}

Other analysis functions: 
\code{\link{ssm_analyze}()},
\code{\link{ssm_parameters}()}
}
\concept{analysis functions}
\concept{ssm functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{iip32}
\alias{iip32}
\title{Inventory of Interpersonal Problems, Brief Version}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Horowitz, L. M., Alden, L. E., Wiggins, J. S., & Pincus, A. L.
(2000). \emph{IIP-64/IIP-32 professional manual.} San Antonio, TX: The
Psychological Corporation.

\url{https://www.mindgarden.com/113-inventory-of-interpersonal-problems}
}
\usage{
iip32
}
\description{
Information about the Inventory of Interpersonal Problems, Brief Version
(IIP-32). Note that, although we have permission to provide some information
about the IIP-32, Mind Garden Inc. has exclusive rights to distribute it in
full.
}
\examples{
summary(iip32)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/instrument_data.R
\docType{data}
\name{csiv}
\alias{csiv}
\title{Circumplex Scales of Interpersonal Values}
\format{
An object of class \code{circumplex_instrument} of length 5.
}
\source{
Locke, K. D. (2000). Circumplex scales of interpersonal values:
Reliability, validity, and applicability to interpersonal problems and
personality disorders. \emph{Journal of Personality Assessment, 75}(2), 249-267.

\url{https://kennethlocke.org/CSIV/CSIV.html}
}
\usage{
csiv
}
\description{
Information about the Circumplex Scales of Interpersonal Values (CSIV).
}
\examples{
summary(csiv)
scales(csiv, items = TRUE)
}
\keyword{internal}
